@@ -3,33 +3,55 @@
  *  Module    : nntplib.h
  *  Author    : I.Lea
  *  Created   : 1991-04-01
- *  Updated   : 1998-07-31
+ *  Updated   : 2003-09-19
  *  Notes     : nntp.h 1.5.11/1.6 with extensions for tin
- *  Copyright : You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made
+ *
+ * Copyright (c) 1991-2004 Iain Lea <iain@bricbrac.de>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 4. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+
 #ifndef NNTPLIB_H
 #define NNTPLIB_H 1
 
-#ifndef	NNTP_SERVER_FILE
-#	ifdef M_AMIGA
-#		define NNTP_SERVER_FILE	"uulib:nntpserver"
-#	else
-#		define NNTP_SERVER_FILE	"/etc/nntpserver"
-#	endif /* M_AMIGA */
+#ifndef NNTP_SERVER_FILE
+#	define NNTP_SERVER_FILE	"/etc/nntpserver"
 #endif /* !NNTP_SERVER_FILE */
 
-#define	NNTP_TCP_NAME	"nntp"
-#define	NNTP_TCP_PORT	"119"
+#define NNTP_TCP_NAME	"nntp"
+#define NNTP_TCP_PORT	"119"
 
+#if 0 /* unused */
 /*
  * # seconds after which a read from the NNTP will timeout
  * NB: This is different from the NNTP server timing us out due to inactivity
  */
-#define NNTP_READ_TIMEOUT		30
+#	define NNTP_READ_TIMEOUT		30
+#endif /* 0 */
 
 /*
  * # times to try and reconnect to server after timeout
@@ -57,11 +79,13 @@
  *	x3x  Distribution
  *	x4x  Posting
  */
-#define	CHAR_INF		'1'
-#define	CHAR_OK		'2'
-#define	CHAR_CONT		'3'
-#define	CHAR_ERR		'4'
-#define	CHAR_FATAL		'5'
+#if 0 /* unused */
+#	define	CHAR_INF		'1'
+#	define	CHAR_OK		'2'
+#	define	CHAR_CONT		'3'
+#	define	CHAR_ERR		'4'
+#	define	CHAR_FATAL		'5'
+#endif /* 0 */
 
 #define	INF_HELP		100	/* Help text on way */
 #define	INF_AUTH		180	/* Authorization capabilities */
@@ -69,14 +93,16 @@
 
 #define	OK_CANPOST		200	/* Hello; you can post */
 #define	OK_NOPOST		201	/* Hello; you can't post */
-#define	OK_NOIHAVE		202	/* Hello; you can't IHAVE */
+#define	OK_EXTENSIONS	202	/* extensions supported follow */
+#define	OK_NOIHAVE		202	/* Hello; you can't IHAVE; discussed on the IETF-mailinglist */
 #define	OK_SLAVE		202	/* Slave status noted */
-#define	OK_NOPOSTIHAVE	203	/* Hello; you can't POST or IHAVE */
+#define	OK_NOPOSTIHAVE	203	/* Hello; you can't POST or IHAVE; iscussed on the IETF-mailinglist */
 #define	OK_GOODBYE		205	/* Closing connection */
 #define	OK_GROUP		211	/* Group selected */
 #define	OK_GROUPS		215	/* Newsgroups follow */
 #define	OK_MOTD			215	/* News motd follows */
 
+/* obsolete */
 #define	OK_XINDEX		218	/* Tin style index follows */
 
 #define	OK_ARTICLE		220	/* Article (head & body) follows */
@@ -137,6 +163,6 @@
 /*
  * RFC 977 defines this; don't change it.
  */
-#define	NNTP_STRLEN		512
+#define NNTP_STRLEN		512
 
 #endif /* !NNTPLIB_H */
