@@ -3,10 +3,10 @@
  *  Module    : policy.h
  *  Author    : Ralf Doeblitz <doeblitz@gmx.de>
  *  Created   : 1999-01-12
- *  Updated   : 2002-02-19
+ *  Updated   : 2004-01-02
  *  Notes     : #defines and static data for policy configuration
  *
- * Copyright (c) 1999-2002 Ralf Doeblitz <doeblitz@gmx.de>
+ * Copyright (c) 1999-2004 Ralf Doeblitz <doeblitz@gmx.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Ralf Doeblitz.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -84,9 +81,8 @@
  * .do  Dominican Republic             .dz  Algeria
  *
  * .ec  Ecuador                        .ee  Estonia
- * .eg  Egypt                          .eh  Western Sahara
- * .er  Eritrea                        .es  Spain
- * .et  Ethiopia
+ * .eg  Egypt                          .er  Eritrea
+ * .es  Spain                          .et  Ethiopia
  *
  * .fi  Finland                        .fj  Fiji
  * .fk  Falkland Islands (Malvina)     .fm  Micronesia, Federal State of
@@ -119,7 +115,6 @@
  * .ke  Kenya                          .kg  Kyrgystan
  * .kh  Cambodia                       .ki  Kiribati
  * .km  Comoros                        .kn  Saint Kitts and Nevis
- * .kp  Korea, Democratic People's Republic
  * .kr  Korea, Republic of             .kw  Kuwait
  * .ky  Cayman Islands                 .kz  Kazakhstan
  *
@@ -204,20 +199,32 @@
  * .zw  Zimbabwe
  *
  *
- * invalid TLDs:
- * .cs  former Czechoslovakia, now: .cz and .sk
- * .zr  former Zaire, now: .cd
+ * invalid/obsolete TLDs:
+ * .bu  Burma, now .mm
+ * .cs  former Czechoslovakia, now .cz and .sk
+ * .dd  former German Democratic Republic, now .de
  * .fx  France, Metropolitan
+ * .yd  Democratic Yemen, now .ye
+ * .zr  former Zaire, now .cd
+ *
+ * user-assigned code elements:
+ * .aa, .qm, .qz, .xa, .xz, .zz
  *
  * unused TLDs:
+ * .eh  Western Sahara
+ * .kp  Korea, Democratic People's Republic
  * .nt  Neutral Zone
  *
  * requested new TLDs:
- * .eu  EU
+ * .ax  Aaland-Islands (Finnish)
+ * .cs  Serbia and Montenegro
+ * .eu  European Union (planned for 2004-11)
  *
  * 'expireing' TLDs:
- *(.hk  Hong Kong, now: .cn)
- * .su  former USSR, now: .ru
+ * .gb  United Kingdom, use .uk
+ *(.hk  Hong Kong, now .cn)
+ * .su  former USSR, now .ru
+ *(.yu  former Yugoslavia)
  */
 
 static char gnksa_country_codes[26*26] = {
@@ -226,13 +233,13 @@ static char gnksa_country_codes[26*26] = {
 /* B */ 1,1,0,1,1, 1,1,1,1,1, 0,0,1,1,1, 0,0,1,1,1, 0,1,1,0,1,1,
 /* C */ 1,0,1,1,0, 1,1,1,1,0, 1,1,1,1,1, 0,0,1,0,0, 1,1,0,1,1,1,
 /* D */ 0,0,0,0,1, 0,0,0,0,1, 1,0,1,0,1, 0,0,0,0,0, 0,0,0,0,0,1,
-/* E */ 0,0,1,0,1, 0,1,1,0,0, 0,0,0,0,0, 0,0,1,1,1, 0,0,0,0,0,0,
+/* E */ 0,0,1,0,1, 0,1,0,0,0, 0,0,0,0,0, 0,0,1,1,1, 0,0,0,0,0,0,
 /* F */ 0,0,0,0,0, 0,0,0,1,1, 1,0,1,0,1, 0,0,1,0,0, 0,0,0,0,0,0,
 /* G */ 1,1,0,1,1, 1,1,1,1,0, 0,1,1,1,0, 1,1,1,1,1, 1,0,1,0,1,0,
 /* H */ 0,0,0,0,0, 0,0,0,0,0, 1,0,1,1,0, 0,0,1,0,1, 1,0,0,0,0,0,
 /* I */ 0,0,0,1,1, 0,0,0,0,0, 0,1,1,1,1, 0,1,1,1,1, 0,0,0,0,0,0,
 /* J */ 0,0,0,0,1, 0,0,0,0,0, 0,0,1,0,1, 1,0,0,0,0, 0,0,0,0,0,0,
-/* K */ 0,0,0,0,1, 0,1,1,1,0, 0,0,1,1,0, 1,0,1,0,0, 0,0,1,0,1,1,
+/* K */ 0,0,0,0,1, 0,1,1,1,0, 0,0,1,1,0, 0,0,1,0,0, 0,0,1,0,1,1,
 /* L */ 1,1,1,0,0, 0,0,0,1,0, 1,0,0,0,0, 0,0,1,1,1, 1,1,0,0,1,0,
 /* M */ 1,0,1,1,0, 0,1,1,0,0, 1,1,1,1,1, 1,1,1,1,1, 1,1,1,1,1,1,
 /* N */ 1,0,1,0,1, 1,1,0,1,0, 0,1,0,0,1, 1,0,1,0,0, 1,0,0,0,0,1,
@@ -264,15 +271,14 @@ static const char *gnksa_domain_list[] = {
 	"mil",	/* US Dept of Defense */
 	"net",	/* Networks */
 	"org",	/* Organizations */
+	"pro",	/* Accountants, lawyers, and physicians */
 	"aero",	/* Air-transport industry */
 	"arpa",	/* Address and Routing Parameter Area */
 	"coop",	/* Non-profit cooperatives */
 	"info",	/* Unrestricted use */
 	"name",	/* For registration by individuals */
 	"museum",	/* Museums */
-#	if 0		/* not yet valid */
-	"pro",	/* Accountants, lawyers, and physicians */
-#	endif /* 0 */
+	/* 3 more gTLDs to come in 2004 (.travel is likely to be one of them) */
 #	if 0		/* $DEAD */
 	"uucp",
 	"bitnet",
