@@ -3,10 +3,10 @@
  *  Module    : version.h
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 1999-11-13
+ *  Updated   : 2003-09-19
  *  Notes     :
  *
- * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2004 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Iain Lea.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -43,25 +40,21 @@
 
 #	define PRODUCT		"tin"
 #	ifndef TIN_AUTOCONF_H
-#		define VERSION		"1.5.12"
+#		define VERSION		"1.7.5"
 #	endif /* !TIN_AUTOCONF_H */
-#	define RELEASEDATE	"20020311"
-#	define RELEASENAME	"Toxicity"
-#	define TINRC_VERSION	"1.2"
-
-#	ifdef M_AMIGA
-#		define OSNAME	"AMIGA"
-#		define AMIVER	VERSION
-#	endif /* M_AMIGA */
+#	define RELEASEDATE	"20040615"
+#	define RELEASENAME	"Gighay"
+/* config-file versions - must by dotted triples */
+#	define TINRC_VERSION	"1.3.7"
+#	define ATTRIBUTES_VERSION	"1.0.3"
+#	define FILTER_VERSION	"1.0.0"
+#	define SERVERCONFIG_VERSION	"1.0.0"
 
 #	ifdef VMS
 #		define OSNAME	"VMS"
 #	endif /* VMS */
 
 #	ifdef M_UNIX
-#		ifdef __amiga
-#			define OSNAME	"AMIGA"
-#		endif /* !__amiga */
 #		ifdef __BEOS__
 #			define OSNAME	"BeOS"
 #		endif /* __BEOS__ */
