@@ -3,10 +3,10 @@
  *  Module    : active.c
  *  Author    : I. Lea
  *  Created   : 1992-02-16
- *  Updated   : 1999-11-19
+ *  Updated   : 2004-03-16
  *  Notes     :
  *
- * Copyright (c) 1992-2002 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1992-2004 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Iain Lea.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -51,10 +48,6 @@
  */
 #define ACTIVE_SEP	" \n"
 
-t_bool force_reread_active_file = FALSE;
-
-static time_t active_timestamp;	/* time active file read (local) */
-
 /*
  * if you are using C-News nntpd set NUM_SIMULTANEOUS_GROUP_COMMAND to 1
  */
@@ -62,46 +55,31 @@ static time_t active_timestamp;	/* time active file read (local) */
 #	define NUM_SIMULTANEOUS_GROUP_COMMAND 50
 #endif /* NNTP_ABLE */
 
+t_bool force_reread_active_file = FALSE;
+static time_t active_timestamp;	/* time active file read (local) */
+
 
 /*
  * Local prototypes
  */
-static int find_newnews_index (char *cur_newnews_host);
-static void active_add (struct t_group *ptr, long count, long max, long min, const char *moderated);
-static void append_group_line (char *active_file, char *group_path, long art_max, long art_min, char *base_dir);
-static void check_for_any_new_groups (void);
-static void make_group_list (char *active_file, char *base_dir, char *group_path);
-static void make_group_name (char *base_dir, char *group_name, char *group_path);
-static void read_active_file (void);
-static void read_newsrc_active_file (void);
-static void subscribe_new_group (char *group, char *autosubscribe, char *autounsubscribe);
-
-
-/*
- * Get default array size for active[] from environment (AmigaDOS)
- * or just return the standard default.
- */
-int
-get_active_num (
-	void)
-{
-#ifdef ENV_VAR_GROUPS /* M_AMIGA only */
-	char *ptr;
-	int num;
-
-	if ((ptr = getenv (ENV_VAR_GROUPS)) != (char *) 0)
-		return ((num = atoi (ptr)) ? num : DEFAULT_ACTIVE_NUM);
-#endif /* ENV_VAR_GROUPS */
-	return DEFAULT_ACTIVE_NUM;
-}
+static FILE *open_newgroups_fp(int idx);
+static FILE *open_news_active_fp(void);
+static void active_add(struct t_group *ptr, long count, long max, long min, const char *moderated);
+static void append_group_line(char *active_file, char *group_path, long art_max, long art_min, char *base_dir);
+static void check_for_any_new_groups(void);
+static void make_group_list(char *active_file, char *base_dir, char *group_path);
+static void make_group_name(char *base_dir, char *group_name, char *group_path);
+static void read_active_file(void);
+static void read_newsrc_active_file(void);
+static void subscribe_new_group(char *group, char *autosubscribe, char *autounsubscribe);
 
 
 t_bool
-need_reread_active_file (
+need_reread_active_file(
 	void)
 {
 	return (force_reread_active_file || (tinrc.reread_active_file_secs != 0 &&
-		(int)(time(NULL) - active_timestamp) >= tinrc.reread_active_file_secs));
+		(int) (time(NULL) - active_timestamp) >= tinrc.reread_active_file_secs));
 }
 
 
@@ -111,41 +89,39 @@ need_reread_active_file (
  * Return TRUE if a reread was performed
  */
 t_bool
-resync_active_file (
+resync_active_file(
 	void)
 {
-	char old_group[HEADER_LEN];
+	char *old_group = NULL;
 	t_bool command_line = FALSE;
 
-	if (!need_reread_active_file ())
+	if (!need_reread_active_file())
 		return FALSE;
 
 	reread_active_for_posted_arts = FALSE;
 
 	if (selmenu.curr >= 0 && selmenu.max)
-		strcpy (old_group, CURR_GROUP.name);
-	else
-		old_group[0] = '\0';
+		old_group = my_strdup(CURR_GROUP.name);
 
-	write_newsrc ();
-	read_news_active_file ();
+	write_newsrc();
+	read_news_active_file();
 
 #ifdef HAVE_MH_MAIL_HANDLING
-	read_mail_active_file ();
+	read_mail_active_file();
 #endif /* HAVE_MH_MAIL_HANDLING */
 
-	if (read_cmd_line_groups ())
+	if (read_cmd_line_groups())
 		command_line = TRUE;
 
-	read_newsrc (newsrc, bool_not(command_line));
+	read_newsrc(newsrc, bool_not(command_line));
 
 	if (command_line)		/* Can't show only unread groups with cmd line groups */
 		tinrc.show_only_unread_groups = FALSE;
 	else
-		toggle_my_groups (tinrc.show_only_unread_groups, old_group);
+		toggle_my_groups(old_group);
 
-	set_groupname_len (FALSE);
-	show_selection_page ();
+	FreeAndNull(old_group);
+	show_selection_page();
 
 	return TRUE;
 }
@@ -157,7 +133,7 @@ resync_active_file (
  * TODO: 2) Lump count/max/min/moderat into a t_active, big patch but much cleaner throughout tin
  */
 static void
-active_add (
+active_add(
 	struct t_group *ptr,
 	long count,
 	long max,
@@ -183,11 +159,11 @@ active_add (
 	ptr->newsrc.xbitmap = (t_bitmap *) 0;
 	ptr->attribute = (struct t_attribute *) 0;
 	ptr->glob_filter = &glob_filter;
-	set_default_bitmap (ptr);
+	set_default_bitmap(ptr);
 
 	if (moderated[0] == '/') {
 		ptr->type = GROUP_TYPE_SAVE;
-		ptr->spooldir = my_strdup(moderated);
+		ptr->spooldir = my_strdup(moderated); /* TODO: Unix'ism, other OSs need transformation */
 	} else {
 		ptr->type = GROUP_TYPE_NEWS;
 		ptr->spooldir = spooldir;		/* another global - sigh */
@@ -203,18 +179,18 @@ active_add (
  * is written.
  */
 t_bool
-process_bogus (
+process_bogus(
 	char *name) /* return value is always ignored */
 {
 	struct t_group *ptr;
 
-	if (read_saved_news || tinrc.strip_bogus != BOGUS_ASK)
+	if (read_saved_news || tinrc.strip_bogus != BOGUS_SHOW)
 		return FALSE;
 
 	if ((ptr = group_add(name)) == NULL)
 		return FALSE;
 
-	active_add (ptr, 0L, 1L, 0L, "n");
+	active_add(ptr, 0L, 1L, 0L, "n");
 	ptr->bogus = TRUE;		/* Mark it bogus */
 
 	if (my_group_add(name) < 0)
@@ -228,7 +204,7 @@ process_bogus (
  * Parse line from news or mail active files
  */
 t_bool
-parse_active_line (
+parse_active_line(
 	char *line,
 	long *max,
 	long *min,
@@ -240,17 +216,17 @@ parse_active_line (
 		return FALSE;
 
 	(void) strtok(line, ACTIVE_SEP);		/* skip group name */
-	p = strtok((char *)0, ACTIVE_SEP);	/* group max count */
-	q = strtok((char *)0, ACTIVE_SEP);	/* group min count */
-	r = strtok((char *)0, ACTIVE_SEP);	/* mod status or path to mailgroup */
+	p = strtok(NULL, ACTIVE_SEP);	/* group max count */
+	q = strtok(NULL, ACTIVE_SEP);	/* group min count */
+	r = strtok(NULL, ACTIVE_SEP);	/* mod status or path to mailgroup */
 
 	if (!p || !q || !r) {
-		error_message (_(txt_bad_active_file), line);
+		error_message(_(txt_bad_active_file), line);
 		return FALSE;
 	}
 
-	*max = atol (p);
-	*min = atol (q);
+	*max = atol(p);
+	*min = atol(q);
 	strcpy(moderated, r);
 
 	return TRUE;
@@ -263,15 +239,16 @@ parse_active_line (
  * Parse a line from the .newsrc file
  * Send GROUP command to NNTP server directly to keep window.
  * We can't know the 'moderator' status and always return 'y'
- * But we don't change if the 'moderator' status is alredy checked by
+ * But we don't change if the 'moderator' status is already checked by
  * read_active_file()
  */
 static void
-read_newsrc_active_file (
+read_newsrc_active_file(
 	void)
 {
 	FILE *fp;
 	char *ptr;
+	char *p;
 	char moderated[PATH_LEN];
 	int window = 0;
 	long count = -1L, min = 1L, max = 0L;
@@ -288,7 +265,7 @@ read_newsrc_active_file (
 	 * return immediately if no .newsrc can be found or .newsrc is empty
 	 * when function asked to use .newsrc
 	 */
-	if ((fp = fopen (newsrc, "r")) == (FILE *) 0)
+	if ((fp = fopen(newsrc, "r")) == NULL)
 		return;
 
 	if (file_size(newsrc) <= 0L) {
@@ -297,11 +274,11 @@ read_newsrc_active_file (
 	}
 
 	if (!batch_mode)
-		wait_message (0, _(txt_reading_news_newsrc_file));
+		wait_message(0, _(txt_reading_news_newsrc_file));
 
-	while ((ptr = tin_fgets (fp, FALSE)) != (char *)0 || window != 0) {
+	while ((ptr = tin_fgets(fp, FALSE)) != NULL || window != 0) {
 		if (ptr) {
-			char *p = strpbrk (ptr, ":!");
+			p = strpbrk(ptr, ":!");
 
 			if (!p || *p != SUBSCRIBED)	/* Invalid line or unsubscribed */
 				continue;
@@ -310,6 +287,7 @@ read_newsrc_active_file (
 			/*
 			 * 128 should be enough for a groupname, >256 and we overflow buffers
 			 * later on
+			 * TODO: check RFCs for possible max. size
 			 */
 			strncpy(ngname, ptr, 128);
 			ptr = ngname;
@@ -321,16 +299,16 @@ read_newsrc_active_file (
 			char line[NNTP_STRLEN];
 			if (window < NUM_SIMULTANEOUS_GROUP_COMMAND && ptr) {
 				ngnames[index_i] = my_strdup(ptr);
-				sprintf (buf, "GROUP %s", ngnames[index_i]);
+				snprintf(buf, sizeof(buf), "GROUP %s", ngnames[index_i]);
 #	ifdef DEBUG
-				debug_nntp ("read_newsrc_active_file", buf);
+				debug_nntp("read_newsrc_active_file", buf);
 #	endif /* DEBUG */
-				put_server (buf);
+				put_server(buf);
 				index_i = (index_i + 1) % NUM_SIMULTANEOUS_GROUP_COMMAND;
 				window++;
 			}
 			if (window == NUM_SIMULTANEOUS_GROUP_COMMAND || ptr == NULL) {
-				int respcode = get_respcode(line);
+				int respcode = get_respcode(line, sizeof(line));
 
 				if (reconnected_in_last_get_server) {
 					/*
@@ -342,11 +320,11 @@ read_newsrc_active_file (
 					int i;
 					int j = index_o;
 					for (i = 0; i < window - 1; i++) {
-						sprintf (buf, "GROUP %s", ngnames[j]);
+						snprintf(buf, sizeof(buf), "GROUP %s", ngnames[j]);
 #	ifdef DEBUG
-						debug_nntp ("read_newsrc_active_file", buf);
+						debug_nntp("read_newsrc_active_file", buf);
 #	endif /* DEBUG */
-						put_server (buf);
+						put_server(buf);
 						j = (j + 1) % NUM_SIMULTANEOUS_GROUP_COMMAND;
 					}
 					if (--index_o < 0)
@@ -363,12 +341,11 @@ read_newsrc_active_file (
 						{
 							char fmt[20];
 
-							sprintf(fmt, "%%ld %%ld %%ld %%%ds", NNTP_STRLEN);
-
-							if (sscanf (line, fmt, &count, &min, &max, ngname) != 4)
-								error_message(_("Invalid response to GROUP command, %s"), line); /* FIXME: -> lang.c */
+							snprintf(fmt, sizeof(fmt), "%%ld %%ld %%ld %%%ds", NNTP_STRLEN);
+							if (sscanf(line, fmt, &count, &min, &max, ngname) != 4)
+								error_message(_(txt_error_invalid_response_to_group), line);
 							if (strcmp(ngname, ngnames[index_o]) != 0)
-								error_message(_("Wrong newsgroup name in response of GROUP command, %s for %s"), line, ngnames[index_o]); /* FIXME: -> lang.c */
+								error_message(_(txt_error_wrong_newsgroupname_in_group_response), line, ngnames[index_o]);
 							ptr = ngname;
 							free(ngnames[index_o]);
 							index_o = (index_o + 1) % NUM_SIMULTANEOUS_GROUP_COMMAND;
@@ -383,13 +360,14 @@ read_newsrc_active_file (
 						continue;
 
 					case ERR_ACCESS:
-						error_message (cCRLF "%s", line);
-						tin_done (NNTP_ERROR_EXIT);
+						error_message("%s%s", cCRLF, line);
+						tin_done(NNTP_ERROR_EXIT);
 						/* keep lint quiet: */
 						/* FALLTHROUGH */
+
 					default:
 #	ifdef DEBUG
-						debug_nntp ("NOT_OK", line);
+						debug_nntp("NOT_OK", line);
 #	endif /* DEBUG */
 						free(ngnames[index_o]);
 						index_o = (index_o + 1) % NUM_SIMULTANEOUS_GROUP_COMMAND;
@@ -400,14 +378,14 @@ read_newsrc_active_file (
 				continue;
 #endif /* NNTP_ABLE */
 		} else {
-			if (group_get_art_info (spooldir, ptr, GROUP_TYPE_NEWS, &count, &max, &min))
+			if (group_get_art_info(spooldir, ptr, GROUP_TYPE_NEWS, &count, &max, &min))
 				continue;
 		}
 
-		strcpy (moderated, "y");
+		strcpy(moderated, "y");
 
 		if (++processed % 5 == 0)
-			spin_cursor ();
+			spin_cursor();
 
 		/*
 		 * Load group into group hash table
@@ -430,7 +408,7 @@ read_newsrc_active_file (
 			}
 			if (changed) {
 				grpptr->count = count;
-				expand_bitmap(grpptr, 0); /* expand_bitmap(grpptr,grpptr->xmin) should be enough */
+				expand_bitmap(grpptr, 0); /* TODO: expand_bitmap(grpptr,grpptr->xmin) should be enough */
 			}
 			continue;
 		}
@@ -438,7 +416,7 @@ read_newsrc_active_file (
 		/*
 		 * Load the new group in active[]
 		 */
-		active_add (grpptr, count, max, min, moderated);
+		active_add(grpptr, count, max, min, moderated);
 	}
 
 	fclose(fp);
@@ -447,8 +425,11 @@ read_newsrc_active_file (
 	 * Exit if active file wasn't read correctly or is empty
 	 */
 	if (tin_errno || !num_active) {
-		error_message (_(txt_active_file_is_empty), (read_news_via_nntp ? _(txt_servers_active) : news_active_file));
-		tin_done (EXIT_FAILURE);
+		if (newsrc_active && !num_active)
+			error_message(_(txt_error_server_has_no_listed_groups), newsrc);
+		else
+			error_message(_(txt_active_file_is_empty), (read_news_via_nntp ? _(txt_servers_active) : news_active_file));
+		tin_done(EXIT_FAILURE);
 	}
 
 	if (!batch_mode)
@@ -456,14 +437,30 @@ read_newsrc_active_file (
 }
 
 
+/*
+ * Open the news active file locally or send the LIST command
+ */
+static FILE *
+open_news_active_fp(
+	void)
+{
+#ifdef NNTP_ABLE
+	if (read_news_via_nntp && !read_saved_news)
+		return (nntp_command("LIST", OK_GROUPS, NULL, 0));
+	else
+#endif /* NNTP_ABLE */
+		return (fopen(news_active_file, "r"));
+}
+
+
 /*
  * Load the active file into active[]
  */
 static void
-read_active_file (
+read_active_file(
 	void)
 {
-	FILE *fp = (FILE *) 0;
+	FILE *fp = NULL;
 	char *ptr;
 	char moderated[PATH_LEN];
 	long count = -1L, min = 1L, max = 0L;
@@ -471,31 +468,32 @@ read_active_file (
 	struct t_group *grpptr;
 
 	if (!batch_mode)
-		wait_message (0, _(txt_reading_news_active_file));
-
-	if ((fp = open_news_active_fp ()) == (FILE *) 0) {
+		wait_message(0, _(txt_reading_news_active_file));
 
+	if ((fp = open_news_active_fp()) == NULL) {
 		if (cmd_line && !batch_mode)
-			my_fputc ('\n', stderr);
+			my_fputc('\n', stderr);
 
-#if defined(NNTP_ABLE) || defined(NNTP_ONLY)
+#ifdef NNTP_ABLE
 		if (read_news_via_nntp)
-#endif /* NNTP_ABLE || NNTP_ONLY */
-			error_message (_(txt_cannot_open), news_active_file);
-#if defined(NNTP_ABLE) || defined(NNTP_ONLY)
+			error_message(_(txt_cannot_retrieve), ACTIVE_FILE);
+#	ifndef NNTP_ONLY
 		else
-			error_message (_(txt_cannot_open_active_file), news_active_file, tin_progname);
-#endif /* NNTP_ABLE || NNTP_ONLY */
+			error_message(_(txt_cannot_open_active_file), news_active_file, tin_progname);
+#	endif /* !NNTP_ONLY */
+#else
+		error_message(_(txt_cannot_open), news_active_file);
+#endif /* NNTP_ABLE */
 
-		tin_done (EXIT_FAILURE);
+		tin_done(EXIT_FAILURE);
 	}
 
-	while ((ptr = tin_fgets (fp, FALSE)) != (char *)0) {
-		if (!parse_active_line (ptr, &max, &min, moderated))
+	while ((ptr = tin_fgets(fp, FALSE)) != NULL) {
+		if (!parse_active_line(ptr, &max, &min, moderated))
 			continue;
 
 		if (++processed % MODULO_COUNT_NUM == 0)
-			spin_cursor ();
+			spin_cursor();
 
 		/*
 		 * Load group into group hash table
@@ -503,7 +501,6 @@ read_active_file (
 		 * This call may implicitly ++num_active
 		 */
 		if ((grpptr = group_add(ptr)) == NULL) {
-
 			if ((grpptr = group_find(ptr)) == NULL)
 				continue;
 
@@ -523,17 +520,17 @@ read_active_file (
 		/*
 		 * Load the new group in active[]
 		 */
-		active_add (grpptr, count, max, min, moderated);
+		active_add(grpptr, count, max, min, moderated);
 	}
 
-	TIN_FCLOSE (fp);
+	TIN_FCLOSE(fp);
 
 	/*
 	 * Exit if active file wasn't read correctly or is empty
 	 */
 	if (tin_errno || !num_active) {
-		error_message (_(txt_active_file_is_empty), (read_news_via_nntp ? _(txt_servers_active) : news_active_file));
-		tin_done (EXIT_FAILURE);
+		error_message(_(txt_active_file_is_empty), (read_news_via_nntp ? _(txt_servers_active) : news_active_file));
+		tin_done(EXIT_FAILURE);
 	}
 
 	if (!batch_mode)
@@ -546,39 +543,83 @@ read_active_file (
  * Check and preload any new newgroups into my_group[]
  */
 void
-read_news_active_file (
+read_news_active_file(
 	void)
 {
-	FILE *fp = 0;
+	FILE *fp = NULL;
 
 	/*
 	 * Ignore -n if no .newsrc can be found or .newsrc is empty
 	 */
 	if (newsrc_active) {
-		if ((fp = fopen (newsrc, "r")) == (FILE *) 0) {
+		if ((fp = fopen(newsrc, "r")) == NULL) {
 			list_active = TRUE;
 			newsrc_active = FALSE;
-		} else
+		} else {
 			fclose(fp);
-
-		if (file_size(newsrc) <= 0L) {
-			list_active = TRUE;
-			newsrc_active = FALSE;
+			if (file_size(newsrc) <= 0L) {
+				list_active = TRUE;
+				newsrc_active = FALSE;
+			}
 		}
 	}
 
 	/* Read an active file if it is allowed */
 	if (list_active)
-		read_active_file ();
+		read_active_file();
 
 	/* Read .newsrc and check each group */
 	if (newsrc_active)
-		read_newsrc_active_file ();
+		read_newsrc_active_file();
 
-	(void) time (&active_timestamp);
+	(void) time(&active_timestamp);
 	force_reread_active_file = FALSE;
 
-	check_for_any_new_groups ();
+	/*
+	 * check_for_any_new_groups() also does $AUTOSUBSCRIBE
+	 */
+	if (check_for_new_newsgroups)
+		check_for_any_new_groups();
+
+	/*
+	 * finally we have a list of all groups an can set the attributes
+	 */
+	read_attributes_files();
+}
+
+
+/*
+ * Open the active.times file locally or send the NEWGROUPS command
+ *
+ * NEWGROUPS yymmdd hhmmss
+ */
+static FILE *
+open_newgroups_fp(
+	int idx)
+{
+#ifdef NNTP_ABLE
+	char line[NNTP_STRLEN];
+	struct tm *ngtm;
+
+	if (read_news_via_nntp && !read_saved_news) {
+		if (idx == -1)
+			return (FILE *) 0;
+
+		ngtm = localtime(&newnews[idx].time);
+		/*
+		 * in the current draft, NEWGROUPS is allowed to take a 4 digit year
+		 * component - but even with a 2 digit year component it is y2k
+		 * compliant... we should switch over to ngtm->tm_year + 1900
+		 * when most servers can handle the new format
+		 */
+		snprintf(line, sizeof(line), "NEWGROUPS %02d%02d%02d %02d%02d%02d",
+			ngtm->tm_year % 100, ngtm->tm_mon + 1, ngtm->tm_mday,
+			ngtm->tm_hour, ngtm->tm_min, ngtm->tm_sec);
+
+		return (nntp_command(line, OK_NEWGROUPS, NULL, 0));
+	} else
+#endif /* NNTP_ABLE */
+		return (fopen(active_times_file, "r"));
 }
 
 
@@ -592,77 +633,62 @@ read_news_active_file (
  * Format:   (as active file) Groupname Maxart Minart moderated
  */
 static void
-check_for_any_new_groups (
+check_for_any_new_groups(
 	void)
 {
 	FILE *fp;
 	char *autosubscribe, *autounsubscribe;
 	char *ptr, *line, buf[NNTP_STRLEN];
 	char old_newnews_host[PATH_LEN];
-	char new_newnews_host[PATH_LEN];
 	int newnews_index;
 	time_t old_newnews_time;
 	time_t new_newnews_time;
 
-	if (!check_for_new_newsgroups /* || !batch_mode */)
-		return;
+	if (!batch_mode /* || verbose */)
+		wait_message(0, _(txt_checking_new_groups));
 
-#if 0
-	if (!batch_mode /* || (batch_mode && verbose) */)
-#endif /* 0 */
-		wait_message (0, _(txt_checking_new_groups));
-
-	(void) time (&new_newnews_time);
-	strcpy (new_newnews_host, (read_news_via_nntp ? nntp_server : "local")); /* What if nntp server called local ? */
+	(void) time(&new_newnews_time);
 
 	/*
 	 * find out if we have read news from here before otherwise -1
 	 */
-	if ((newnews_index = find_newnews_index (new_newnews_host)) >= 0) {
-		strcpy (old_newnews_host, newnews[newnews_index].host);
+	if ((newnews_index = find_newnews_index(nntp_server)) >= 0) {
+		STRCPY(old_newnews_host, newnews[newnews_index].host);
 		old_newnews_time = newnews[newnews_index].time;
 	} else {
-		strcpy (old_newnews_host, "UNKNOWN");
+		STRCPY(old_newnews_host, "UNKNOWN");
 		old_newnews_time = (time_t) 0;
 	}
 
 #ifdef DEBUG
 	if (debug == 2) {
 		error_message("Newnews old=[%lu]  new=[%lu]", (unsigned long int) old_newnews_time, (unsigned long int) new_newnews_time);
-		(void) sleep (2);
+		(void) sleep(2);
 	}
 #endif /* DEBUG */
 
-	if ((fp = open_newgroups_fp (newnews_index)) != (FILE *) 0) {
-
+	if ((fp = open_newgroups_fp(newnews_index)) != NULL) {
 		/*
 		 * Need these later. They list user-defined groups to be
 		 * automatically subscribed or unsubscribed.
 		 */
-		autosubscribe = getenv ("AUTOSUBSCRIBE");
-		autounsubscribe = getenv ("AUTOUNSUBSCRIBE");
-
-		while ((line = tin_fgets (fp, FALSE)) != (char *) 0) {
+		autosubscribe = getenv("AUTOSUBSCRIBE");
+		autounsubscribe = getenv("AUTOUNSUBSCRIBE");
 
+		while ((line = tin_fgets(fp, FALSE)) != NULL) {
 			/*
 			 * Split the group name off and subscribe. If we're reading local,
 			 * we must check the creation date manually
 			 */
-			if ((ptr = strchr (line, ' ')) != (char *) 0) {
-				if (!read_news_via_nntp && ((time_t) atol (ptr) < old_newnews_time || old_newnews_time == (time_t) 0))
+			if ((ptr = strchr(line, ' ')) != NULL) {
+				if (!read_news_via_nntp && ((time_t) atol(ptr) < old_newnews_time || old_newnews_time == (time_t) 0))
 					continue;
 
 				*ptr = '\0';
 			}
-
-			subscribe_new_group (line, autosubscribe, autounsubscribe);
+			subscribe_new_group(line, autosubscribe, autounsubscribe);
 		}
-
-		TIN_FCLOSE (fp);
-
-		free_attributes_array ();
-		read_attributes_file (global_attributes_file, TRUE);
-		read_attributes_file (local_attributes_file, FALSE);
+		TIN_FCLOSE(fp);
 
 		if (tin_errno)
 			return;				/* Don't update the time if we quit */
@@ -676,10 +702,9 @@ check_for_any_new_groups (
 	if (newnews_index >= 0)
 		newnews[newnews_index].time = new_newnews_time;
 	else {
-		sprintf (buf, "%s %lu", new_newnews_host, (unsigned long int) new_newnews_time);
-		load_newnews_info (buf);
+		snprintf(buf, sizeof(buf), "%s %lu", nntp_server, (unsigned long int) new_newnews_time);
+		load_newnews_info(buf);
 	}
-	my_fputc('\n', stdout);
 }
 
 
@@ -693,7 +718,7 @@ check_for_any_new_groups (
  * Otherwise, mark it as New for inclusion in selection screen
  */
 static void
-subscribe_new_group (
+subscribe_new_group(
 	char *group,
 	char *autosubscribe,
 	char *autounsubscribe)
@@ -704,7 +729,7 @@ subscribe_new_group (
 	/*
 	 * If we explicitly don't auto subscribe to this group, then don't bother going on
 	 */
-	if ((autounsubscribe != (char *) 0) && match_group_list (group, autounsubscribe))
+	if ((autounsubscribe != NULL) && match_group_list(group, autounsubscribe))
 		return;
 
 	/*
@@ -726,10 +751,16 @@ subscribe_new_group (
 			return;
 	}
 
-	if (!no_write && (autosubscribe != (char *) 0) && match_group_list (group, autosubscribe)) {
-		my_printf (_(txt_autosubscribed), group);
+	if (!no_write && (autosubscribe != NULL) && match_group_list(group, autosubscribe)) {
+		if (!batch_mode || verbose)
+			my_printf(_(txt_autosubscribed), group);
 
-		subscribe (&active[my_group[idx]], SUBSCRIBED);
+		/*
+		 * as subscribe_new_group() is called from check_for_any_new_groups()
+		 * which has pending data on the socket if reading via NNTP we are not
+		 * allowed to issue any NNTP comands yet
+		 */
+		subscribe(&active[my_group[idx]], SUBSCRIBED, bool_not(read_news_via_nntp));
 		/*
 		 * Bad kluge to stop group later appearing in New newsgroups. This
 		 * effectively loses the group, and it has now been subscribed to and
@@ -747,17 +778,16 @@ subscribe_new_group (
  * The same degree of wildcarding as used elsewhere in tin is allowed
  */
 t_bool
-match_group_list (
+match_group_list(
 	const char *group,
 	const char *group_list)
 {
 	char *separator;
 	char pattern[HEADER_LEN];
 	size_t group_len, list_len;
-	t_bool accept, negate;
+	t_bool negate, accept = FALSE;
 
-	accept = FALSE;
-	list_len = strlen (group_list);
+	list_len = strlen(group_list);
 	/*
 	 * walk through comma-separated entries in list
 	 */
@@ -765,8 +795,8 @@ match_group_list (
 		/*
 		 * find end/length of this entry
 		 */
-		separator = strchr (group_list, ',');
-		group_len = ((separator == (char *) 0) ? list_len : (size_t)(separator - group_list));
+		separator = strchr(group_list, ',');
+		group_len = MIN(((separator == NULL) ? list_len : (size_t) (separator - group_list)), sizeof(pattern) - 1);
 
 		if ((negate = ('!' == *group_list))) {
 			/*
@@ -779,13 +809,13 @@ match_group_list (
 		/*
 		 * copy out the entry and terminate it properly
 		 */
-		strncpy (pattern, group_list, group_len);
+		strncpy(pattern, group_list, group_len);
 		pattern[group_len] = '\0';
 		/*
 		 * case-insensitive wildcard match
 		 */
 		if (GROUP_MATCH(group, pattern, TRUE))
-			accept = !negate;	/* matched! */
+			accept = bool_not(negate);	/* matched! */
 
 		/*
 		 * now examine next entry if any
@@ -809,7 +839,7 @@ match_group_list (
  *   'info' is modified. Caller should not depend on it.
  */
 void
-load_newnews_info (
+load_newnews_info(
 	char *info)
 {
 	char *ptr;
@@ -829,21 +859,21 @@ load_newnews_info (
 	/*
 	 * Split 'info' into hostname and time
 	 */
-	if ((ptr = strchr (info, ' ')) == (char *) 0)
+	if ((ptr = strchr(info, ' ')) == NULL)
 		return;
 
 	*ptr++ = '\0';
-	new_time = (time_t) atol (ptr);
+	new_time = (time_t) atol(ptr);
 
 	/*
 	 * If this is a new host entry, set it up
 	 */
-	if ((i = find_newnews_index (info)) == -1) {
+	if ((i = find_newnews_index(info)) == -1) {
 		i = num_newnews++;
 
 		if (i >= max_newnews)
-			expand_newnews ();
-		newnews[i].host = my_strdup (info);
+			expand_newnews();
+		newnews[i].host = my_strdup(info);
 	}
 
 	newnews[i].time = new_time;
@@ -858,9 +888,9 @@ load_newnews_info (
 /*
  * Return the index of cur_newnews_host in newnews[] or -1 if not found
  */
-static int
-find_newnews_index (
-	char *cur_newnews_host)
+int
+find_newnews_index(
+	const char *cur_newnews_host)
 {
 	int i;
 
@@ -876,19 +906,25 @@ find_newnews_index (
 /*
  * Get a single status char from the moderated field. Used on selection screen
  * and in header of group screen
+ *
+ * TODO: what about 'j' groups? active(5) says:
+ *       "local postings to that group should not be generated"
  */
 char
-group_flag (
+group_flag(
 	char ch)
 {
 	switch (ch) {
 		case 'm':
 			return 'M';
+
 		case 'x':
 		case 'n':
 			return 'X';
+
 		case '=':
 			return '=';
+
 		default:
 			return ' ';
 	}
@@ -897,27 +933,27 @@ group_flag (
 
 /* ex actived.c functions */
 void
-create_save_active_file (
+create_save_active_file(
 	void)
 {
 	char group_path[PATH_LEN];
 	char local_save_active_file[PATH_LEN];
 
-	snprintf (local_save_active_file, sizeof(local_save_active_file) - 1, "%s/%s/%s", homedir, RCDIR, ACTIVE_SAVE_FILE);
+	joinpath(local_save_active_file, rcdir, ACTIVE_SAVE_FILE);
 
-	if (no_write && file_size (local_save_active_file) != -1L)
+	if (no_write && file_size(local_save_active_file) != -1L)
 		return;
 
-	my_printf (_(txt_creating_active));
+	my_printf(_(txt_creating_active));
 
-	print_active_head (local_save_active_file);
-	strcpy (group_path, tinrc.savedir);
-	make_group_list (local_save_active_file, tinrc.savedir, group_path);
+	print_active_head(local_save_active_file);
+	strcpy(group_path, tinrc.savedir);
+	make_group_list(local_save_active_file, tinrc.savedir, group_path);
 }
 
 
 static void
-make_group_list (
+make_group_list(
 	char *active_file,
 	char *base_dir,
 	char *group_path)
@@ -932,29 +968,27 @@ make_group_list (
 	struct stat stat_info;
 	t_bool is_dir;
 
-	if ((dir = opendir (group_path)) != (DIR *) 0) {
+	if ((dir = opendir(group_path)) != NULL) {
 		is_dir = FALSE;
-		while ((direntry = readdir (dir)) != (DIR_BUF *) 0) {
+		while ((direntry = readdir(dir)) != NULL) {
 			STRCPY(filename, direntry->d_name);
-			sprintf (path, "%s/%s", group_path, filename);
+			joinpath(path, group_path, filename);
 
 			if (!(filename[0] == '.' && filename[1] == '\0') &&
 				!(filename[0] == '.' && filename[1] == '.' && filename[2] == '\0')) {
-				if (stat (path, &stat_info) != -1) {
+				if (stat(path, &stat_info) != -1) {
 					if (S_ISDIR(stat_info.st_mode))
 						is_dir = TRUE;
 				}
 			}
 			if (is_dir) {
 				is_dir = FALSE;
-				strcpy (group_path, path);
+				strcpy(group_path, path);
 
-				make_group_list (active_file, base_dir, group_path);
-				find_art_max_min (group_path, &art_max, &art_min);
-				append_group_line (active_file, group_path, art_max, art_min, base_dir);
-
-				ptr = strrchr (group_path, '/');
-				if (ptr != (char *) 0)
+				make_group_list(active_file, base_dir, group_path);
+				find_art_max_min(group_path, &art_max, &art_min);
+				append_group_line(active_file, group_path, art_max, art_min, base_dir);
+				if ((ptr = strrchr(group_path, '/')) != NULL) /* TODO: Unix'ism */
 					*ptr = '\0';
 			}
 		}
@@ -964,7 +998,7 @@ make_group_list (
 
 
 static void
-append_group_line (
+append_group_line(
 	char *active_file,
 	char *group_path,
 	long art_max,
@@ -980,21 +1014,20 @@ append_group_line (
 
 	file_tmp = get_tmpfilename(active_file);
 
-	if (!backup_file (active_file, file_tmp)) {
-		free (file_tmp);
+	if (!backup_file(active_file, file_tmp)) {
+		free(file_tmp);
 		return;
 	}
 
-	if ((fp = fopen (active_file, "a+")) != (FILE *) 0) {
-		make_group_name (base_dir, group_name, group_path);
-		my_printf ("Appending=[%s %ld %ld %s]\n", group_name, art_max, art_min, base_dir);
-		print_group_line (fp, group_name, art_max, art_min, base_dir);
-		if (ferror(fp) || fclose (fp))
-			/* TODO: issue warning? */
-			rename (file_tmp, active_file);
+	if ((fp = fopen(active_file, "a+")) != NULL) {
+		make_group_name(base_dir, group_name, group_path);
+		my_printf("Appending=[%s %ld %ld %s]\n", group_name, art_max, art_min, base_dir);
+		print_group_line(fp, group_name, art_max, art_min, base_dir);
+		if (ferror(fp) || fclose(fp)) /* TODO: issue warning? */
+			rename(file_tmp, active_file);
 	}
-	unlink (file_tmp);
-	free (file_tmp);
+	unlink(file_tmp);
+	free(file_tmp);
 }
 
 
@@ -1005,7 +1038,7 @@ append_group_line (
  * newsgroup = alt.sources
  */
 static void
-make_group_name (
+make_group_name(
 	char *base_dir,
 	char *group_name,
 	char *group_path)
@@ -1021,9 +1054,9 @@ make_group_name (
 		base_ptr++;
 		path_ptr++;
 	}
-	strcpy (group_name, ++path_ptr);
+	strcpy(group_name, ++path_ptr);
 
 	name_ptr = group_name;
-	while ((name_ptr = strchr (name_ptr, '/')) != (char *) 0)
+	while ((name_ptr = strchr(name_ptr, '/')) != NULL)
 		*name_ptr = '.';
 }
