@@ -1,13 +1,13 @@
 /*
  *  Project   : tin - a Usenet reader
  *  Module    : auth.c
- *  Author    : Dirk Nimmich <nimmich@uni-muenster.de>
+ *  Author    : Dirk Nimmich <nimmich@muenster.de>
  *  Created   : 1997-04-05
- *  Updated   : 2000-01-22
+ *  Updated   : 2003-05-09
  *  Notes     : Routines to authenticate to a news server via NNTP.
  *              DON'T USE get_respcode() THROUGHOUT THIS CODE.
  *
- * Copyright (c) 1997-2002 Dirk Nimmich <nimmich@uni-muenster.de>
+ * Copyright (c) 1997-2004 Dirk Nimmich <nimmich@muenster.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -18,10 +18,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Dirk Nimmich.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -55,10 +52,10 @@
 /*
  * local prototypes
  */
-static int do_authinfo_original (char *server, char *authuser, char *authpass);
-static t_bool authinfo_generic (void);
-static t_bool read_newsauth_file (char *server, char *authuser, char *authpass);
-static t_bool authinfo_original (char *server, char *authuser, t_bool startup);
+static int do_authinfo_original(char *server, char *authuser, char *authpass);
+static t_bool authinfo_generic(void);
+static t_bool read_newsauth_file(char *server, char *authuser, char *authpass);
+static t_bool authinfo_original(char *server, char *authuser, t_bool startup);
 
 
 /*
@@ -67,7 +64,7 @@ static t_bool authinfo_original (char *server, char *authuser, t_bool startup);
  * FALSE means failed
  */
 static t_bool
-authinfo_generic (
+authinfo_generic(
 	void)
 {
 	char *authcmd;
@@ -75,67 +72,66 @@ authinfo_generic (
 	char tmpbuf[NNTP_STRLEN];
 	static int cookiefd = -1;
 	t_bool builtinauth = FALSE;
-#ifdef HAVE_PUTENV
+#if !defined(HAVE_SETENV) && defined(HAVE_PUTENV)
 	char *new_env;
-	static char *old_env = 0;
-#endif /* HAVE_PUTENV */
+	static char *old_env = NULL;
+#endif /* !HAVE_SETENV && HAVE_PUTENV */
 
 #ifdef DEBUG
-	debug_nntp ("authorization", "authinfo generic");
+	debug_nntp("authorization", "authinfo generic");
 #endif /* DEBUG */
 
 	/*
 	 * If we have authenticated before, NNTP_AUTH_FDS already
 	 * exists, pull out the cookiefd. Just in case we've nested.
 	 */
-	if (cookiefd == -1 && (authcmd = getenv ("NNTP_AUTH_FDS")))
-		(void) sscanf (authcmd, "%*d.%*d.%d", &cookiefd);
+	if (cookiefd == -1 && (authcmd = getenv("NNTP_AUTH_FDS")))
+		(void) sscanf(authcmd, "%*d.%*d.%d", &cookiefd);
 
 	if (cookiefd == -1) {
 		char tempfile[PATH_LEN];
 		if ((cookiefd = my_tmpfile_only(tempfile)) == -1) {
 #	ifdef DEBUG
-			debug_nntp ("authorization", txt_cannot_create_uniq_name);
+			debug_nntp("authorization", txt_cannot_create_uniq_name);
 #	endif /* DEBUG */
 			return FALSE;
 		} else {
 			close(cookiefd);
 			if (tempfile[0] != '\0')
-				(void) unlink (tempfile);
+				(void) unlink(tempfile);
 		}
 	}
 
-	strcpy (tmpbuf, "AUTHINFO GENERIC ");
-	STRCPY(authval, get_val ("NNTPAUTH", ""));
-	if (strlen (authval))
-		strcat (tmpbuf, authval);
+	strcpy(tmpbuf, "AUTHINFO GENERIC ");
+	STRCPY(authval, get_val("NNTPAUTH", ""));
+	if (strlen(authval))
+		strcat(tmpbuf, authval);
 	else {
-		strcat (tmpbuf, "ANY ");
-		strcat (tmpbuf, userid);
+		strcat(tmpbuf, "ANY ");
+		strcat(tmpbuf, userid);
 		builtinauth = TRUE;
 	}
-	put_server (tmpbuf);
+	put_server(tmpbuf);
 
-#ifdef HAVE_PUTENV
-	sprintf (tmpbuf, "NNTP_AUTH_FDS=%d.%d.%d",
-			fileno (get_nntp_fp(FAKE_NNTP_FP)),
-			fileno (get_nntp_wr_fp(FAKE_NNTP_FP)), cookiefd);
-	new_env = (char *) my_malloc (strlen (tmpbuf) + 1);
-	strcpy (new_env, tmpbuf);
-	(void) putenv (new_env);
-	FreeIfNeeded (old_env);
-	old_env = new_env;
+#ifdef HAVE_SETENV
+	snprintf(tmpbuf, sizeof(tmpbuf), "%d.%d.%d",
+			fileno(get_nntp_fp(FAKE_NNTP_FP)),
+			fileno(get_nntp_wr_fp(FAKE_NNTP_FP)), cookiefd);
+	setenv("NNTP_AUTH_FDS", tmpbuf, 1);
 #else
-#	ifdef HAVE_SETENV
-	sprintf (tmpbuf, "%d.%d.%d",
-			fileno (get_nntp_fp(FAKE_NNTP_FP)),
-			fileno (get_nntp_wr_fp(FAKE_NNTP_FP)), cookiefd);
-	setenv ("NNTP_AUTH_FDS", tmpbuf, 1);
-#	endif /* HAVE_SETENV */
-#endif /* HAVE_PUTENV */
+#	ifdef HAVE_PUTENV
+	snprintf(tmpbuf, sizeof(tmpbuf), "NNTP_AUTH_FDS=%d.%d.%d",
+			fileno(get_nntp_fp(FAKE_NNTP_FP)),
+			fileno(get_nntp_wr_fp(FAKE_NNTP_FP)), cookiefd);
+	new_env = my_strdup(tmpbuf);
+	(void) putenv(new_env);
+	FreeIfNeeded(old_env);
+	old_env = new_env;	/* we are 'leaking' the last malloced mem at exit here */
+#	endif /* HAVE_PUTENV */
+#endif /* HAVE_SETENV */
 
-	/* TODO - is it possible that we should have drained server here ? */
-	return (builtinauth ? (get_only_respcode(NULL) == OK_AUTH) : (invoke_cmd (authval) ? TRUE : FALSE));
+	/* TODO: is it possible that we should have drained server here? */
+	return (builtinauth ? (get_only_respcode(NULL, 0) == OK_AUTH) : (invoke_cmd(authval) ? TRUE : FALSE));
 }
 
 
@@ -146,7 +142,7 @@ authinfo_generic (
  * no .newsauth file or no matching server.
  */
 static t_bool
-read_newsauth_file (
+read_newsauth_file(
 	char *server,
 	char *authuser,
 	char *authpass)
@@ -156,48 +152,56 @@ read_newsauth_file (
 	char *ptr;
 	char line[PATH_LEN];
 	int found = 0;
+	int fd = -1;
 	struct stat statbuf;
 
-	joinpath (line, homedir, ".newsauth");
+	joinpath(line, homedir, ".newsauth");
 
-	if (stat (line, &statbuf) == -1)
-		return FALSE;
-	else {
-		if (S_ISREG(statbuf.st_mode) && (statbuf.st_mode|S_IRUSR|S_IWUSR) != (S_IRUSR|S_IWUSR|S_IFREG)) {
-			/* FIXME: -> lang.c */
-			error_message (_("Insecure permissions of %s (%o)"), line, statbuf.st_mode);
-			sleep(2);
+	if ((fp = fopen(line, "r"))) {
+		if ((fd = fileno(fp)) == -1) {
+			fclose(fp);
+			return FALSE;
+		}
+		if (fstat(fd, &statbuf) == -1) {
+			fclose(fp);
+			return FALSE;
 		}
-	}
 
-	if ((fp = fopen (line, "r"))) {
+		if (S_ISREG(statbuf.st_mode) && (statbuf.st_mode|S_IRUSR|S_IWUSR) != (S_IRUSR|S_IWUSR|S_IFREG)) {
+			error_message(_(txt_error_insecure_permissions), line, statbuf.st_mode);
+			sleep(2);
+			/*
+			 * TODO: fix permssions?
+			 * fchmod(fd, S_IRUSR|S_IWUSR);
+			 */
+		}
 
 		/*
 		 * Search through authorization file for correct NNTP server
 		 * File has format:  'nntp-server' 'password' ['username']
 		 */
-		while (fgets (line, PATH_LEN, fp) != (char *) 0) {
+		while (fgets(line, PATH_LEN, fp) != NULL) {
 
 			/*
 			 * strip trailing newline character
 			 */
 
-			ptr = strchr (line, '\n');
-			if (ptr != (char *) 0)
+			ptr = strchr(line, '\n');
+			if (ptr != NULL)
 				*ptr = '\0';
 
 			/*
 			 * Get server from 1st part of the line
 			 */
 
-			ptr = strpbrk (line, " \t");
+			ptr = strpbrk(line, " \t");
 
-			if (ptr == (char *) 0)		/* no passwd, no auth, skip */
+			if (ptr == NULL)		/* no passwd, no auth, skip */
 				continue;
 
 			*ptr++ = '\0';		/* cut off server part */
 
-			if ((strcasecmp (line, server)))
+			if ((strcasecmp(line, server)))
 				continue;		/* wrong server, keep on */
 
 			/*
@@ -210,8 +214,8 @@ read_newsauth_file (
 			_authpass = ptr;
 
 			if (*_authpass == '"') {	/* skip "embedded" password string */
-				ptr = strrchr (_authpass, '"');
-				if ((ptr != (char *) 0) && (ptr > _authpass)) {
+				ptr = strrchr(_authpass, '"');
+				if ((ptr != NULL) && (ptr > _authpass)) {
 					_authpass++;
 					*ptr++ = '\0';	/* cut off trailing " */
 				} else	/* no matching ", proceede as normal */
@@ -222,19 +226,19 @@ read_newsauth_file (
 			 * Get user from 3rd part of the line
 			 */
 
-			ptr = strpbrk (ptr, " \t");	/* find next separating blank */
+			ptr = strpbrk(ptr, " \t");	/* find next separating blank */
 
-			if (ptr != (char *) 0) {	/* a 3rd argument follows */
+			if (ptr != NULL) {	/* a 3rd argument follows */
 				while (*ptr == ' ' || *ptr == '\t')	/* skip any blanks */
 					*ptr++ = '\0';
 				if (*ptr != '\0')	/* if its not just empty */
-					strcpy (authuser, ptr);	/* so will replace default user */
+					strcpy(authuser, ptr);	/* so will replace default user */
 			}
-			strcpy (authpass, _authpass);
+			strcpy(authpass, _authpass);
 			found++;
 			break;	/* if we end up here, everything seems OK */
 		}
-		fclose (fp);
+		fclose(fp);
 		return (found > 0);
 	} else
 		return FALSE;
@@ -246,7 +250,7 @@ read_newsauth_file (
  * code from server.
  */
 static int
-do_authinfo_original (
+do_authinfo_original(
 	char *server,
 	char *authuser,
 	char *authpass)
@@ -254,28 +258,30 @@ do_authinfo_original (
 	char line[PATH_LEN];
 	int ret;
 
-	sprintf (line, "AUTHINFO USER %s", authuser);
+	snprintf(line, sizeof(line), "AUTHINFO USER %s", authuser);
 #ifdef DEBUG
-	debug_nntp ("authorization", line);
+	debug_nntp("authorization", line);
 #endif /* DEBUG */
-	put_server (line);
-	if ((ret = get_only_respcode(NULL)) != NEED_AUTHDATA)
+	put_server(line);
+	if ((ret = get_only_respcode(NULL, 0)) != NEED_AUTHDATA)
 		return ret;
 
-	if ((authpass == (char *) 0) || (*authpass == '\0')) {
+	if ((authpass == NULL) || (*authpass == '\0')) {
 #ifdef DEBUG
-		debug_nntp ("authorization", "failed: no password");
+		debug_nntp("authorization", "failed: no password");
 #endif /* DEBUG */
-		error_message (_(txt_nntp_authorization_failed), server);
+		error_message(_(txt_nntp_authorization_failed), server);
 		return ERR_AUTHBAD;
 	}
 
-	sprintf (line, "AUTHINFO PASS %s", authpass);
+	snprintf(line, sizeof(line), "AUTHINFO PASS %s", authpass);
 #ifdef DEBUG
-	debug_nntp ("authorization", line);
+	debug_nntp("authorization", line);
 #endif /* DEBUG */
-	put_server (line);
-	wait_message(2, (((ret = get_only_respcode(line)) == OK_AUTH) ? _(txt_authorization_ok) : _(txt_authorization_fail)), authuser);
+	put_server(line);
+	ret = get_only_respcode(line, sizeof(line));
+	if (!batch_mode || verbose || ret != OK_AUTH)
+		wait_message(2, (ret == OK_AUTH ? _(txt_authorization_ok) : _(txt_authorization_fail)), authuser);
 	return ret;
 }
 
@@ -291,7 +297,7 @@ do_authinfo_original (
  *   etc.
  */
 static t_bool
-authinfo_original (
+authinfo_original(
 	char *server,
 	char *authuser,
 	t_bool startup)
@@ -304,11 +310,11 @@ authinfo_original (
 	static t_bool already_failed = FALSE;
 
 #ifdef DEBUG
-	debug_nntp ("authorization", "original authinfo");
+	debug_nntp("authorization", "original authinfo");
 #endif /* DEBUG */
 
 	authpassword[0] = '\0';
-	authuser = strncpy (authusername, authuser, PATH_LEN);
+	authuser = strncpy(authusername, authuser, sizeof(authusername) - 1);
 	authpass = authpassword;
 
 	/*
@@ -324,13 +330,13 @@ authinfo_original (
 	 * and restart tin or change to another server and back in order to get
 	 * it read again.
 	 */
-	if ((changed = strcmp (server, last_server)) || (!changed && !already_failed)) {
+	if ((changed = strcmp(server, last_server)) || (!changed && !already_failed)) {
 		already_failed = FALSE;
-		if (read_newsauth_file (server, authuser, authpass)) {
-			ret = do_authinfo_original (server, authuser, authpass);
+		if (read_newsauth_file(server, authuser, authpass)) {
+			ret = do_authinfo_original(server, authuser, authpass);
 			if (!(already_failed = (ret != OK_AUTH))) {
 #ifdef DEBUG
-				debug_nntp ("authorization", "succeeded");
+				debug_nntp("authorization", "succeeded");
 #endif /* DEBUG */
 				return TRUE;
 			}
@@ -343,7 +349,7 @@ authinfo_original (
 	 * matching username/password for the current server. If we are not at
 	 * startup we ask the user to enter such a pair by hand. Don't ask him
 	 * startup except if requested by -A option because if he doesn't need
-	 * authenticate (we don't know), the "Server expects authentication"
+	 * authenticate(we don't know), the "Server expects authentication"
 	 * messages are annoying (and even wrong).
 	 * UNSURE: Maybe we want to make this decision configurable in the
 	 * options menu, too, so that the user doesn't need -A.
@@ -355,22 +361,22 @@ authinfo_original (
 		int state = RawState();
 #endif /* USE_CURSES */
 
-		wait_message (0, _(txt_auth_needed));
+		wait_message(0, _(txt_auth_needed));
 #ifdef USE_CURSES
 		Raw(TRUE);
 #endif /* USE_CURSES */
 
 		if (!prompt_default_string(_(txt_auth_user), authuser, PATH_LEN, authusername, HIST_NONE)) {
 #ifdef DEBUG
-			debug_nntp ("authorization", "failed: no username");
+			debug_nntp("authorization", "failed: no username");
 #endif /* DEBUG */
 			return FALSE;
 		}
 
 #ifdef USE_CURSES
 		Raw(state);
-		my_printf ("%s", _(txt_auth_pass));
-		wgetnstr (stdscr, authpassword, sizeof(authpassword));
+		my_printf("%s", _(txt_auth_pass));
+		wgetnstr(stdscr, authpassword, sizeof(authpassword));
 		Raw(TRUE);
 #else
 #	if 0
@@ -379,18 +385,18 @@ authinfo_original (
 		 * we use tin_getline() till we have a config check
 		 * for getpass() or our own getpass()
 		 */
-		authpass = strncpy (authpassword, getpass (_(txt_auth_pass)), PATH_LEN);
+		authpass = strncpy(authpassword, getpass(_(txt_auth_pass)), sizeof(authpassword) - 1);
 #	else
-		authpass = strncpy (authpassword, tin_getline (_(txt_auth_pass), FALSE, (char *) 0, PATH_LEN, TRUE, HIST_NONE), PATH_LEN);
+		authpass = strncpy(authpassword, tin_getline(_(txt_auth_pass), FALSE, (char *) 0, PATH_LEN, TRUE, HIST_NONE), sizeof(authpassword) - 1);
 #	endif /* 0 */
 #endif /* USE_CURSES */
 
-		ret = do_authinfo_original (server, authuser, authpass);
+		ret = do_authinfo_original(server, authuser, authpass);
 		my_retouch();			/* Get rid of the chaff */
 	}
 
 #ifdef DEBUG
-	debug_nntp ("authorization", (ret == OK_AUTH ? "succeeded" : "failed"));
+	debug_nntp("authorization", (ret == OK_AUTH ? "succeeded" : "failed"));
 #endif /* DEBUG */
 
 	return (ret == OK_AUTH);
@@ -405,18 +411,18 @@ authinfo_original (
  * AUTHINFO method. Other authentication methods can easily be added.
  */
 t_bool
-authenticate (
+authenticate(
 	char *server,
 	char *user,
 	t_bool startup)
 {
-	return (authinfo_generic () || authinfo_original (server, user, startup));
+	return (authinfo_generic() || authinfo_original(server, user, startup));
 }
 
 #else
-static void no_authenticate (void);			/* proto-type */
+static void no_authenticate(void);			/* proto-type */
 static void
-no_authenticate (					/* ANSI C requires non-empty source file */
+no_authenticate(					/* ANSI C requires non-empty source file */
 	void)
 {
 }
