@@ -3,9 +3,9 @@
  *  Module    : header.c
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 1997-03-10
- *  Updated   : 1997-03-19
+ *  Updated   : 2003-09-19
  *
- * Copyright (c) 1997-2002 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2004 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,13 +40,13 @@
 #	include "tnntp.h"
 #endif /* !TNNTP_H */
 
-static const char * get_full_name (void);
-static const char * get_user_name (void);
+static const char *get_full_name(void);
+static const char *get_user_name(void);
 
 
 /* find hostname */
 const char *
-get_host_name (
+get_host_name(
 	void)
 {
 	char *ptr;
@@ -55,23 +55,18 @@ get_host_name (
 	hostname[0] = '\0';
 
 #ifdef HAVE_GETHOSTNAME
-	gethostname(hostname, sizeof(hostname));
-#else
-#	if defined(M_AMIGA)
-	if ((ptr = getenv("NodeName")) != (char *) 0)
-		strncpy(hostname, ptr, MAXHOSTNAMELEN);
-#	endif /* M_AMIGA */
+	gethostname(hostname, sizeof(hostname) - 1);
 #endif /* HAVE_GETHOSTNAME */
 #ifdef HAVE_SYS_UTSNAME_H
 	if (!*hostname)
-		strcpy(hostname, system_info.nodename);
+		strncpy(hostname, system_info.nodename, sizeof(hostname) - 1);
 #endif /* HAVE_SYS_UTSNAME_H */
 	if (!*hostname) {
-		if ((ptr = getenv("HOST")) != (char *) 0)
-			strncpy (hostname, ptr, MAXHOSTNAMELEN);
+		if ((ptr = getenv("HOST")) != NULL)
+			my_strncpy(hostname, ptr, sizeof(hostname) - 1);
 		else {
-			if ((ptr = getenv("HOSTNAME")) != (char *) 0)
-				strncpy (hostname, ptr, MAXHOSTNAMELEN);
+			if ((ptr = getenv("HOSTNAME")) != NULL)
+				my_strncpy(hostname, ptr, sizeof(hostname) - 1);
 			else
 				hostname[0] = '\0';
 		}
@@ -82,58 +77,46 @@ get_host_name (
 
 
 #ifdef DOMAIN_NAME
-/* find domainname - check DOMAIN_NAME */
-/* TODO: check /etc/defaultdomain as a last resort? */
+/*
+ * find domainname - check DOMAIN_NAME
+ * TODO: check /etc/defaultdomain as a last resort?
+ *       there is M_UNIX specific code in here, but no VMS
+ *       alternative
+ */
 const char *
-get_domain_name (
+get_domain_name(
 	void)
 {
-	char *ptr;
-	char buff[MAXHOSTNAMELEN];
 	FILE *fp;
+	char *ptr;
+	char buff[MAXHOSTNAMELEN + 1];
 	static char domain[8192];
 
-#	ifdef M_AMIGA
-/*
- * Damn compiler bugs...
- * Without this hack, SASC 6.55 produces a TST.B d16(pc),
- * which is illegal on a 68000
- */
-static const char *domain_name_hack = DOMAIN_NAME;
-#		undef DOMAIN_NAME
-#		define DOMAIN_NAME domain_name_hack
-#	endif /* M_AMIGA */
-
 	domain[0] = '\0';
 
 	if (strlen(DOMAIN_NAME))
 		strcpy(domain, DOMAIN_NAME);
 
-#	ifdef M_AMIGA
-	if (strchr(domain, ':')) /* absolute AmigaOS paths contain one, RFC-hostnames don't */
-#	else
-	if (domain[0] == '/' && domain[1])
-#	endif /* M_AMIGA */
-	{
-		/* If 1st letter is '/' read domainname from specified file */
-		if ((fp = fopen (domain, "r")) != (FILE *) 0) {
-			while (fgets (buff, (int) sizeof (buff), fp) != (char *) 0) {
+	if (domain[0] == '/' && domain[1]) {
+		/* read domainname from specified file */
+		if ((fp = fopen(domain, "r")) != NULL) {
+			while (fgets(buff, (int) sizeof(buff), fp) != NULL) {
 				if (buff[0] == '#' || buff[0] == '\n')
 					continue;
 
-				if ((ptr = strrchr (buff, '\n'))) {
+				if ((ptr = strrchr(buff, '\n'))) {
 					*ptr = '\0';
-					strcpy (domain, buff);
+					strcpy(domain, buff);
 				}
 			}
-			if (domain[0] == '/') /* file was empty */
+			if (domain[0] == '/')	/* '/' is not allowed in domainames -> file was empty */
 				domain[0] = '\0';
 
-			fclose (fp);
+			fclose(fp);
 		} else
 			domain[0] = '\0';
 	}
-	domain[MAXHOSTNAMELEN] = '\0';
+	domain[MAXHOSTNAMELEN + 1] = '\0';
 	return domain;
 }
 #endif /* DOMAIN_NAME */
@@ -144,12 +127,12 @@ static const char *domain_name_hack = DOMAIN_NAME;
 #	define WS	" \f\t\v"
 /* find FQDN - gethostbyaddr() */
 const char *
-get_fqdn (
+get_fqdn(
 	const char *host)
 {
 	char *domain;
 	char line[MAXLINELEN + 1];
-	char name[MAXHOSTNAMELEN + 2];
+	char name[MAXHOSTNAMELEN + 1];
 	static char fqdn[1024];
 	struct hostent *hp;
 	struct in_addr in;
@@ -161,29 +144,31 @@ get_fqdn (
 	if (host) {
 		if (strchr(host, '.'))
 			return host;
-		(void) strncpy(name, host, MAXHOSTNAMELEN);
+		strncpy(name, host, sizeof(name) - 1);
 	} else {
 #	ifdef HAVE_GETHOSTNAME
-		if (gethostname(name, MAXHOSTNAMELEN))
+		if (gethostname(name, sizeof(name) - 1))
 #	endif /* HAVE_GETHOSTNAME */
 			return NULL;
 	}
 
+#	ifdef HAVE_INET_ADDR
 	if ('0' <= *name && *name <= '9') {
 		in.s_addr = inet_addr(name);
 		if ((hp = gethostbyaddr((char *) &in.s_addr, 4, AF_INET)))
 			in.s_addr = (*hp->h_addr);
-		return(hp && strchr(hp->h_name, '.') ? hp->h_name : inet_ntoa(in));
+		return (hp && strchr(hp->h_name, '.') ? hp->h_name : inet_ntoa(in));
 	}
+#	endif /* HAVE_INET_ADDR */
 	if ((hp = gethostbyname(name)) && !strchr(hp->h_name, '.'))
 		if ((hp = gethostbyaddr(hp->h_addr, hp->h_length, hp->h_addrtype)))
 			in.s_addr = (*hp->h_addr);
 
-	sprintf(fqdn, "%s", hp
+	snprintf(fqdn, sizeof(fqdn), "%s", hp
 		? strchr(hp->h_name, '.')
-			? hp->h_name
-			: inet_ntoa(in)
+			? hp->h_name : inet_ntoa(in)
 		: "");
+
 	if (!*fqdn || (fqdn[strlen(fqdn) - 1] <= '9')) {
 		FILE *inf;
 
@@ -191,29 +176,25 @@ get_fqdn (
 
 		if ((inf = fopen("/etc/resolv.conf", "r")) != NULL) {
 			char *eos;
+			int j;
 
-			while(fgets(line, MAXLINELEN, inf)) {
-
+			while (fgets(line, MAXLINELEN, inf)) {
 				if (line[0] == '#' || line[0] == '\n')
 					continue;
 
 				line[MAXLINELEN] = '\0';
 
 				if ((eos = strpbrk(line, WS)) != NULL) {
-					int j = eos - line;
-
-					if (j) {
-						if ((strncmp(line, "domain", j) == 0) || (strncmp(line, "search", j) == 0)) {
+					if ((j = eos - line)) {
+						if (!strncmp(line, "domain", j) || !strncmp(line, "search", j)) {
 							domain = strtok(eos, WS);
 							break;
 						}
 					}
 				}
 			}
-			if (domain) {
-				strip_line(domain);
-				sprintf(fqdn, "%s.%s", name, domain);
-			}
+			if (domain)
+				snprintf(fqdn, sizeof(fqdn), "%s.%s", name, strip_line(domain));
 
 			fclose(inf);
 		}
@@ -227,7 +208,7 @@ get_fqdn (
  * Find username & fullname
  */
 void
-get_user_info (
+get_user_info(
 	char *user_name,
 	char *full_name)
 {
@@ -244,39 +225,43 @@ get_user_info (
 
 
 static const char *
-get_user_name (
+get_user_name(
 	void)
 {
-#	if defined (M_AMIGA) || (defined VMS)
-	char *p;
-#	endif /* M_AMIGA || VMS */
 	static char username[128];
 	struct passwd *pw;
+#ifdef VMS
+	char *p;
+#endif /* VMS */
 
 	username[0] = '\0';
-#	if defined (M_AMIGA) || defined (VMS)
-	if ((p = getenv ("USER"))) {
-		STRCPY(username, p);
-#		ifdef VMS
-		lower (username);
-#		endif /* VMS */
-	}
-#	else
-	pw = getpwuid (getuid ());
 
-	if (pw != (struct passwd *) 0)
-		strcpy (username, pw->pw_name);
-	else {
-		error_message (_(txt_error_passwd_missing));
-		tin_done (EXIT_FAILURE);
+#ifndef VMS
+	if ((pw = getpwuid(getuid())) != NULL)
+		strcpy(username, pw->pw_name);
+	else
+#else
+	if (((p = getlogin()) != NULL) && strlen(p)) {
+		if ((pw = getpwnam(p)) != NULL)
+			strcpy(username, pw->pw_name);
 	}
-#	endif /* M_AMIGA || VMS */
+#endif /* !VMS */
+
+	if (!*username) {
+		error_message(_(txt_error_passwd_missing));
+		tin_done(EXIT_FAILURE);
+	}
+
+#	ifdef VMS
+	lower(username);
+#	endif /* VMS */
+
 	return username;
 }
 
 
 static const char *
-get_full_name (
+get_full_name(
 	void)
 {
 	char *p;
@@ -289,12 +274,12 @@ get_full_name (
 
 	fullname[0] = '\0';
 
-	if ((p = getenv ("NAME")) != (char *) 0) {
-		strncpy (fullname, p, sizeof (fullname));
+	if ((p = getenv("NAME")) != NULL) {
+		my_strncpy(fullname, p, sizeof(fullname) - 1);
 		return fullname;
 	}
-	if ((p = getenv ("REALNAME")) != (char *) 0) {
-		strncpy (fullname, p, sizeof (fullname));
+	if ((p = getenv("REALNAME")) != NULL) {
+		my_strncpy(fullname, p, sizeof(fullname) - 1);
 		return fullname;
 	}
 
@@ -302,18 +287,19 @@ get_full_name (
 	STRCPY(fullname, fix_fullname(get_uaf_fullname()));
 #	else
 #		ifndef DONT_HAVE_PW_GECOS
-	pw = getpwuid (getuid ());
-	STRCPY(buf, pw->pw_gecos);
-	if ((p = strchr (buf, ',')))
-		*p = '\0';
-	if ((p = strchr (buf, '&'))) {
-		*p++ = '\0';
-		STRCPY(tmp, pw->pw_name);
-		if (*tmp && isalpha((int) *tmp) && islower((int) *tmp))
-			*tmp = toupper((int) *tmp);
-		snprintf (fullname, sizeof(fullname) -1, "%s%s%s", buf, tmp, p);
-	} else
-		STRCPY(fullname, buf);
+	if ((pw = getpwuid(getuid())) != NULL) {
+		STRCPY(buf, pw->pw_gecos);
+		if ((p = strchr(buf, ',')))
+			*p = '\0';
+		if ((p = strchr(buf, '&'))) {
+			*p++ = '\0';
+			STRCPY(tmp, pw->pw_name);
+			if (*tmp && isalpha((int)(unsigned char) *tmp) && islower((int)(unsigned char) *tmp))
+				*tmp = toupper((int)(unsigned char) *tmp);
+			snprintf(fullname, sizeof(fullname), "%s%s%s", buf, tmp, p);
+		} else
+			STRCPY(fullname, buf);
+	}
 #		endif /* !DONT_HAVE_PW_GECOS */
 #	endif /* VMS */
 	return fullname;
@@ -322,18 +308,18 @@ get_full_name (
 
 /*
  * FIXME to:
- * char * build_from(full_name, user_name, domain_name)
+ * char *build_from(full_name, user_name, domain_name)
  */
 /*
  * build From: in 'name <user@host.doma.in>' format
  */
 void
-get_from_name (
+get_from_name(
 	char *from_name,
 	struct t_group *thisgrp)
 {
 #	ifdef USE_INN_NNTPLIB
-	char *fromhost = GetConfigValue (_CONF_FROMHOST);
+	char *fromhost = GetConfigValue(_CONF_FROMHOST);
 #	else
 	char *fromhost = NULL;
 #	endif /* USE_INN_NNTPLIB */
@@ -346,11 +332,11 @@ get_from_name (
 		return;
 	}
 
-	sprintf (from_name, ((strpbrk(get_full_name(), "!()<>@,;:\\\".[]")) ? "\"%s\" <%s@%s>" : "%s <%s@%s>"), get_full_name(), get_user_name(), fromhost);
+	sprintf(from_name, ((strpbrk(get_full_name(), "!()<>@,;:\\\".[]")) ? "\"%s\" <%s@%s>" : "%s <%s@%s>"), get_full_name(), get_user_name(), fromhost);
 
 #	ifdef DEBUG
 	if (debug == 2)
-		error_message ("FROM=[%s] USER=[%s] HOST=[%s] NAME=[%s]", from_name, get_user_name(), domain_name, get_full_name());
+		error_message("FROM=[%s] USER=[%s] HOST=[%s] NAME=[%s]", from_name, get_user_name(), domain_name, get_full_name());
 #	endif /* DEBUG */
 
 }
@@ -362,7 +348,7 @@ get_from_name (
  */
 #ifndef FORGERY
 char *
-build_sender (
+build_sender(
 	void)
 {
 	const char *ptr;
@@ -371,8 +357,7 @@ build_sender (
 	sender[0] = '\0';
 
 	if ((ptr = get_full_name()))
-		sprintf (sender, ((strchr(ptr, '.')) ? "\"%s\" " : "%s "), ptr);
-
+		snprintf(sender, sizeof(sender), ((strpbrk(ptr, "\".:;<>@[]()\\")) ? "\"%s\"" : "%s "), ptr);
 	if ((ptr = get_user_name())) {
 		strcat(sender, "<");
 		strcat(sender, ptr);
