@@ -3,10 +3,10 @@
  *  Module    : mail.c
  *  Author    : I. Lea
  *  Created   : 1992-10-02
- *  Updated   : 2001-07-22
+ *  Updated   : 2004-03-23
  *  Notes     : Mail handling routines for creating pseudo newsgroups
  *
- * Copyright (c) 1992-2002 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1992-2004 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Iain Lea.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -45,53 +42,87 @@
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
 
-static void read_groups_descriptions (FILE *fp, FILE *fp_save);
+/*
+ * local prototypes
+ */
+static FILE *open_newsgroups_fp(void);
+static void read_groups_descriptions(FILE *fp, FILE *fp_save);
+static void read_newsgroups_file(t_bool verb);
+#ifdef HAVE_MH_MAIL_HANDLING
+	static FILE *open_mail_active_fp(const char *mode);
+	static FILE *open_mailgroups_fp(void);
+	static void read_mailgroups_file(t_bool verb);
+#endif /* HAVE_MH_MAIL_HANDLING */
+
+
+#ifdef HAVE_MH_MAIL_HANDLING
+/*
+ * Open the mail active file locally
+ */
+static FILE *
+open_mail_active_fp(
+	const char *mode)
+{
+	return fopen(mail_active_file, mode);
+}
+
+
+/*
+ * Open mail groups description file locally
+ */
+static FILE *
+open_mailgroups_fp(
+	void)
+{
+	return fopen(mailgroups_file, "r");
+}
 
 
 /*
  * Load the mail active file into active[]
  */
-#ifdef HAVE_MH_MAIL_HANDLING
 void
-read_mail_active_file (
+read_mail_active_file(
 	void)
 {
 	FILE *fp;
 	char buf[LEN];
 	char my_spooldir[PATH_LEN];
-	long count = -1L;
+	char buf2[PATH_LEN];
 	long min, max;
 	struct t_group *ptr;
 
 	if (!batch_mode)
-		wait_message (0, _(txt_reading_mail_active_file));
+		wait_message(0, _(txt_reading_mail_active_file));
 
 	/*
 	 * Open the mail active file
 	 */
-	if ((fp = open_mail_active_fp ("r")) == (FILE *) 0) {
+	if ((fp = open_mail_active_fp("r")) == NULL) {
 		if (cmd_line)
-			my_fputc ('\n', stderr);
+			my_fputc('\n', stderr);
 
-		error_message (_(txt_cannot_open), mail_active_file);
+		if (!created_rcdir) /* no error on first start */
+			error_message(_(txt_cannot_open), mail_active_file);
 		/*
-		 * FIXME - maybe do an autoscan of maildir, create & do a reopen ?
+		 * TODO: do an autoscan of maildir, create & reopen?
 		 */
-		write_mail_active_file ();
+		write_mail_active_file();
 		return;
 	}
 
-	while (fgets (buf, (int) sizeof (buf), fp) != (char *) 0) {
-		if (!parse_active_line (buf, &max, &min, my_spooldir) || *buf == '\0')
+	while (fgets(buf, (int) sizeof(buf), fp) != NULL) {
+		if (!parse_active_line(buf, &max, &min, my_spooldir) || *buf == '\0')
 			continue;
 
 		/*
 		 * Update mailgroup info
 		 */
-		if ((ptr = group_find (buf)) != NULL) {
+		if ((ptr = group_find(buf)) != NULL) {
 			if (strcmp(ptr->spooldir, my_spooldir) != 0) {
 				free(ptr->spooldir);
-				ptr->spooldir = my_strdup(my_spooldir);
+				strfpath(my_spooldir, buf2, sizeof(buf2) - 1, ptr);
+				ptr->spooldir = my_strdup(buf2);
 			}
 			ptr->xmax = max;
 			ptr->xmin = min;
@@ -101,19 +132,18 @@ read_mail_active_file (
 		/*
 		 * Load mailgroup into group hash table
 		 */
-		if ((ptr = group_add (buf)) == NULL)
+		if ((ptr = group_add(buf)) == NULL)
 			continue;
 
 		/*
-		 * Load group info. TODO - integrate with active_add()
+		 * Load group info. TODO: integrate with active_add()
 		 */
+		strfpath(my_spooldir, buf2, sizeof(buf2) - 1, ptr);
+		ptr->spooldir = my_strdup(buf2);
+		group_get_art_info(ptr->spooldir, buf, GROUP_TYPE_MAIL, &ptr->count, &ptr->xmax, &ptr->xmin);
 		ptr->aliasedto = (char *) 0;
 		ptr->description = (char *) 0;
-		ptr->spooldir = my_strdup (my_spooldir);
 		ptr->moderated = 'y';
-		ptr->count = count;
-		ptr->xmax = max;
-		ptr->xmin = min;
 		ptr->type = GROUP_TYPE_MAIL;
 		ptr->inrange = FALSE;
 		ptr->read_during_session = FALSE;
@@ -125,9 +155,9 @@ read_mail_active_file (
 		ptr->newsrc.xbitmap = (t_bitmap *) 0;
 		ptr->attribute = (struct t_attribute *) 0;
 		ptr->glob_filter = &glob_filter;
-		set_default_bitmap (ptr);
+		set_default_bitmap(ptr);
 	}
-	fclose (fp);
+	fclose(fp);
 
 	if (!batch_mode)
 		my_fputs("\n", stdout);
@@ -138,48 +168,48 @@ read_mail_active_file (
  * Write out mailgroups from active[] to ~/.tin/active.mail
  */
 void
-write_mail_active_file (
+write_mail_active_file(
 	void)
 {
 	FILE *fp;
 	char *file_tmp;
 	char group_path[PATH_LEN];
-	register int i;
+	int i;
 	struct t_group *group;
 
-	if (no_write && file_size (mail_active_file) != -1L)
+	if (no_write && file_size(mail_active_file) != -1L)
 		return;
 
 	/* generate tmp-filename */
 	file_tmp = get_tmpfilename(mail_active_file);
 
-	if (!backup_file (mail_active_file, file_tmp)) {
-		error_message (_(txt_filesystem_full_backup), mail_active_file);
+	if (!backup_file(mail_active_file, file_tmp)) {
+		error_message(_(txt_filesystem_full_backup), mail_active_file);
 		/* free memory for tmp-filename */
-		free (file_tmp);
+		free(file_tmp);
 		return;
 	}
 
-	print_active_head (mail_active_file);
+	print_active_head(mail_active_file);
 
-	if ((fp = open_mail_active_fp ("a+")) != (FILE *) 0) {
-		for (i = 0; i < num_active; i++) {
+	if ((fp = open_mail_active_fp("a+")) != NULL) {
+		for_each_group(i) {
 			group = &active[i];
 			if (group->type == GROUP_TYPE_MAIL) {
-				make_base_group_path (group->spooldir, group->name, group_path);
-				find_art_max_min (group_path, &group->xmax, &group->xmin);
-				print_group_line (fp, group->name, group->xmax, group->xmin, group->spooldir);
+				make_base_group_path(group->spooldir, group->name, group_path);
+				find_art_max_min(group_path, &group->xmax, &group->xmin);
+				print_group_line(fp, group->name, group->xmax, group->xmin, group->spooldir);
 			}
 		}
-		if (ferror (fp) || fclose (fp)) {
-			error_message (_(txt_filesystem_full), mail_active_file);
-			rename (file_tmp, mail_active_file);
+		if (ferror(fp) || fclose(fp)) {
+			error_message(_(txt_filesystem_full), mail_active_file);
+			rename(file_tmp, mail_active_file);
 		} else
-			unlink (file_tmp);
+			unlink(file_tmp);
 	}
 
 	/* free memory for tmp-filename */
-	free (file_tmp);
+	free(file_tmp);
 }
 
 
@@ -187,61 +217,113 @@ write_mail_active_file (
  * Load the text description from ~/.tin/mailgroups for each mail group into
  * the active[] array.
  */
-void
-read_mailgroups_file (
-	void)
+static void
+read_mailgroups_file(
+	t_bool verb)
 {
 	FILE *fp;
 
-	if ((fp = open_mailgroups_fp ()) != (FILE *) 0) {
-		if (!batch_mode)
-			wait_message (0, _(txt_reading_mailgroups_file));
+	if ((fp = open_mailgroups_fp()) != NULL) {
+		if (!batch_mode && verb)
+			wait_message(0, _(txt_reading_mailgroups_file));
 
-		read_groups_descriptions (fp, (FILE *) 0);
+		read_groups_descriptions(fp, (FILE *) 0);
 
-		fclose (fp);
+		fclose(fp);
 
-		if (!batch_mode)
+		if (!batch_mode && verb)
 			my_fputs("\n", stdout);
 	}
 }
 #endif /* HAVE_MH_MAIL_HANDLING */
 
 
+/*
+ * If reading via NNTP the newsgroups file will be saved to ~/.tin/newsgroups
+ * so that any subsequent rereads on the active file will not have to waste
+ * net bandwidth and the local copy of the newsgroups file can be accessed.
+ */
+static FILE *
+open_newsgroups_fp(
+	void)
+{
+#ifdef NNTP_ABLE
+	FILE *result;
+
+	if (read_news_via_nntp && !read_saved_news) {
+		if (read_local_newsgroups_file) {
+			result = fopen(local_newsgroups_file, "r");
+			if (result != NULL) {
+#	ifdef DEBUG
+				debug_nntp("open_newsgroups_fp", "Using local copy of newsgroups file");
+#	endif /* DEBUG */
+				return result;
+			}
+			read_local_newsgroups_file = FALSE;
+		}
+#	if 0 /* TODO: */
+		if (list_newsgroups_wildmat_supported && newsrc_active
+		    && !list_active && num_active < some_useful_limit) {
+			for_each_group(i) {
+				snprintf(buff, sizeof(buff), "LIST NEWSGROUPS %s", active[i].name);
+				nntp_command(buff, OK_LIST, NULL, 0);
+			}
+		} else
+#	endif /* 0 */
+		return (nntp_command("LIST NEWSGROUPS", OK_GROUPS, NULL, 0));
+	} else
+#endif /* NNTP_ABLE */
+		return fopen(newsgroups_file, "r");
+}
+
+
 /*
  * Load the text description from NEWSLIBDIR/newsgroups for each group into the
  * active[] array. Save a copy locally if reading via NNTP to save bandwidth.
  */
-void
-read_newsgroups_file (
-	void)
+static void
+read_newsgroups_file(
+	t_bool verb)
 {
 	FILE *fp;
 	FILE *fp_save = (FILE *) 0;
 
-	if ((fp = open_newsgroups_fp ()) != (FILE *) 0) {
-		if (!batch_mode)
-			wait_message (0, _(txt_reading_newsgroups_file));
-
+	if ((fp = open_newsgroups_fp()) != NULL) {
+		if (!batch_mode && verb)
+			wait_message(0, _(txt_reading_newsgroups_file));
 
 		if (read_news_via_nntp && !read_local_newsgroups_file && !no_write)
-			fp_save = fopen (local_newsgroups_file, "w" FOPEN_OPTS);
+			fp_save = fopen(local_newsgroups_file, "w" FOPEN_OPTS);
 
-		read_groups_descriptions (fp, fp_save);
+		read_groups_descriptions(fp, fp_save);
 
-		if (fp_save != (FILE *) 0) {
-			fclose (fp_save);
+		if (fp_save != NULL) {
+			fclose(fp_save);
 			read_local_newsgroups_file = TRUE;
 		}
 
-		TIN_FCLOSE (fp);
+		TIN_FCLOSE(fp);
 
-		if (!batch_mode)
+		if (!batch_mode && verb)
 			my_fputs("\n", stdout);
 	}
 }
 
 
+/*
+ * read group descriptions for news (and mailgroups)
+ */
+void
+read_descriptions(
+	t_bool verb)
+{
+#ifdef HAVE_MH_MAIL_HANDLING
+	read_mailgroups_file(verb);
+#endif /* HAVE_MH_MAIL_HANDLING */
+	read_newsgroups_file(verb);
+}
+
+
 /*
  * Read groups descriptions from opened file & make local backup copy
  * of all groups that don't have a 'x' in the active file moderated
@@ -250,7 +332,7 @@ read_newsgroups_file (
  * descriptions.
  */
 static void
-read_groups_descriptions (
+read_groups_descriptions(
 	FILE *fp,
 	FILE *fp_save)
 {
@@ -260,7 +342,7 @@ read_groups_descriptions (
 	size_t space = 0;
 	struct t_group *group;
 
-	while ((ptr = tin_fgets (fp, FALSE)) != (char *) 0) {
+	while ((ptr = tin_fgets(fp, FALSE)) != NULL) {
 		if (*ptr == '#' || *ptr == '\0')
 			continue;
 
@@ -271,8 +353,8 @@ read_groups_descriptions (
 		 * newsgroups file to only subscribed-to groups when tin is called
 		 * with the "-q" option.
 		 */
-		if ((fp_save != (FILE *) 0) && read_news_via_nntp && !read_local_newsgroups_file)
-			fprintf (fp_save, "%s\n", ptr);
+		if ((fp_save != NULL) && read_news_via_nntp && !read_local_newsgroups_file)
+			fprintf(fp_save, "%s\n", ptr);
 
 		if (!space) { /* initial malloc */
 			space = strlen(ptr) + 1;
@@ -280,11 +362,11 @@ read_groups_descriptions (
 		} else {
 			while (strlen(ptr) > space) { /* realloc needed? */
 				space <<= 1; /* double size */
-				groupname = (char *) my_realloc((void *) groupname, space);
+				groupname = my_realloc(groupname, space);
 			}
 		}
 
-		for (p = ptr, q = groupname ; *p && *p != ' ' && *p != '\t' ; p++, q++)
+		for (p = ptr, q = groupname; *p && *p != ' ' && *p != '\t'; p++, q++)
 			*q = *p;
 
 		*q = '\0';
@@ -292,52 +374,53 @@ read_groups_descriptions (
 		while (*p == '\t' || *p == ' ')
 			p++;
 
-		group = group_find (groupname);
+		group = group_find(groupname);
+
+		if (group != NULL && group->description == NULL) {
+			char *r;
+			int r_len;
 
-		if (group != (struct t_group *) 0 && group->description == (char *) 0) {
 			q = p;
-			while ((q = strchr (q, '\t')) != (char *) 0)
+			while ((q = strchr(q, '\t')) != NULL)
 				*q = ' ';
 
-			group->description = my_strdup (p);
-
-#	if 0 /* not useful for cache_overview_files */
-			if (group->type == GROUP_TYPE_NEWS) {
-				if (fp_save != (FILE *) 0 && read_news_via_nntp && !read_local_newsgroups_file)
-					fprintf (fp_save, "%s\n", ptr);
-			}
-#	endif /* 0 */
+			r = my_strdup(p);
+			r_len = strlen(r);
+			/*
+			 * Protect against invalid character sequences.
+			 *
+			 * TODO: change US-ASCII to UTF-8 when NNTP draft becomes RFC
+			 */
+			process_charsets(&r, &r_len, "US-ASCII", tinrc.mm_local_charset, FALSE);
+			group->description = convert_to_printable(r);
 		}
 
 		if (++count % 100 == 0)
-			spin_cursor ();
+			spin_cursor();
 	}
-	free(groupname);
+	FreeIfNeeded(groupname);
 }
 
 
 void
-print_active_head (
-	char *active_file)
+print_active_head(
+	const char *active_file)
 {
 	FILE *fp;
 
-	if (no_write && file_size (active_file) != -1L)
+	if (no_write && file_size(active_file) != -1L)
 		return;
 
-	if ((fp = fopen (active_file, "w")) != (FILE *) 0) {
-		/* FIXME: -> lang.c */
-		fprintf (fp, _("# [Mail/Save] active file. Format is like news active file:\n"));
-		fprintf (fp, _("#   groupname  max.artnum  min.artnum  /dir\n"));
-		fprintf (fp, _("# The 4th field is the basedir (ie. ~/Mail or ~/News)\n#\n"));
-		fclose (fp);
+	if ((fp = fopen(active_file, "w")) != NULL) {
+		fprintf(fp, _(txt_mail_save_active_head));
+		fclose(fp);
 	}
 }
 
 
 void
-find_art_max_min (
-	char *group_path,
+find_art_max_min(
+	const char *group_path,
 	long *art_max,
 	long *art_min)
 {
@@ -347,19 +430,13 @@ find_art_max_min (
 
 	*art_min = *art_max = 0L;
 
-	if (access (group_path, R_OK) != 0) {
-		*art_min = 1L;
-		return;
-	}
-
-	dir = opendir (group_path);
-	if (dir != (DIR *) 0) {
-		while ((direntry = readdir (dir)) != (DIR_BUF *) 0) {
-			art_num = atol (direntry->d_name);
-			if (art_num >= 1) {
+	if ((dir = opendir(group_path)) != NULL) {
+		while ((direntry = readdir(dir)) != NULL) {
+			art_num = atol(direntry->d_name);
+			if (art_num >= 1L) {
 				if (art_num > *art_max) {
 					*art_max = art_num;
-					if (*art_min == 0)
+					if (*art_min == 0L)
 						*art_min = art_num;
 				} else if (art_num < *art_min)
 					*art_min = art_num;
@@ -367,67 +444,45 @@ find_art_max_min (
 		}
 		CLOSEDIR(dir);
 	}
-	if (*art_min == 0)
-		*art_min = 1;
+	if (*art_min == 0L)
+		*art_min = 1L;
 }
 
 
 void
-print_group_line (
+print_group_line(
 	FILE *fp,
-	char *group_name,
+	const char *group_name,
 	long art_max,
 	long art_min,
-	char *base_dir)
+	const char *base_dir)
 {
-	fprintf (fp, "%s %05ld %05ld %s\n",
+	fprintf(fp, "%s %05ld %05ld %s\n",
 		group_name, art_max, art_min, base_dir);
 }
 
 
-/*
- * Given a base pathname & a newsgroup name build an absolute pathname.
- * base = /usr/spool/news
- * newsgroup = alt.sources
- * absolute path = /usr/spool/news/alt/sources
- */
 void
-make_base_group_path (
-	char *base_dir,
-	char *group_name,
-	char *group_path)
-{
-	char *ptr;
-
-	joinpath (group_path, base_dir, group_name);
-
-	ptr = group_path + strlen (base_dir);
-	while ((ptr = strchr (ptr, '.')) != (char *) 0)
-		*ptr = '/';
-}
-
-
-void
-vGrpDelMailArt (
+grp_del_mail_art(
 	struct t_article *article)
 {
 
-	if (article->delete_it) {
-		art_mark_undeleted (article);
-		info_message (_(txt_art_undeleted));
-	} else {
-		art_mark_deleted (article);
-		info_message (_(txt_art_deleted));
-	}
+	if (article->delete_it)
+		info_message(_(txt_art_undeleted));
+	else
+		info_message(_(txt_art_deleted));
+
+	article->delete_it = bool_not(article->delete_it);
 }
 
 
 void
-vGrpDelMailArts (
+grp_del_mail_arts(
 	struct t_group *group)
 {
 	char article_filename[PATH_LEN];
 	char group_path[PATH_LEN];
+	char artnum[LEN];
 	int i;
 	struct t_article *article;
 #if 0 /* see comment below */
@@ -435,13 +490,17 @@ vGrpDelMailArts (
 #endif /* 0 */
 
 	if (group->type == GROUP_TYPE_MAIL || group->type == GROUP_TYPE_SAVE) {
-		wait_message (1, (group->type == GROUP_TYPE_MAIL) ? _(txt_processing_mail_arts) : _(txt_processing_saved_arts));
-		make_base_group_path (group->spooldir, group->name, group_path);
-		for (i = 0; i < top_art; i++) {
+		/*
+		 * at least for GROUP_TYPE_SAVE a wait is annoying - nuke the message?
+		 */
+		wait_message(0, (group->type == GROUP_TYPE_MAIL) ? _(txt_processing_mail_arts) : _(txt_processing_saved_arts));
+		make_base_group_path(group->spooldir, group->name, group_path);
+		for_each_art(i) {
 			article = &arts[i];
 			if (article->delete_it) {
-				sprintf (article_filename, "%s/%ld", group_path, article->artnum);
-				unlink (article_filename);
+				snprintf(artnum, sizeof(artnum), "%ld", article->artnum);
+				joinpath(article_filename, group_path, artnum);
+				unlink(article_filename);
 				article->thread = ART_EXPIRED;
 #if 0 /* see comment below */
 				update_index_file = TRUE;
@@ -452,7 +511,7 @@ vGrpDelMailArts (
 #if 0
 /*
  * current tin's build_references() is changed to free msgid and refs,
- * therefore we cannot call write_nov_file after it. I simply commented
+ * therefore we cannot call write_overview after it. I simply commented
  * out this codes, NovFile will update at next time.
  */
 /*
@@ -460,20 +519,20 @@ vGrpDelMailArts (
  * the active[] entry for the group and rewrite the mail.active file
  */
 		if (update_index_file)
-			write_nov_file (group);
+			write_overview(group);
 #endif /* 0 */
 	}
 }
 
 
 t_bool
-art_edit (
+art_edit(
 	struct t_group *group,
 	struct t_article *article)
 {
 	char article_filename[PATH_LEN];
 	char temp_filename[PATH_LEN];
-	t_bool ret = FALSE;
+	char buf[PATH_LEN];
 
 	/*
 	 * Check if news / mail group
@@ -481,20 +540,19 @@ art_edit (
 	if (group->type == GROUP_TYPE_NEWS)
 		return FALSE;
 
-	make_base_group_path (group->spooldir, group->name, temp_filename);
-	snprintf (article_filename, sizeof(article_filename) - 1, "%s/%ld", temp_filename, article->artnum);
-	snprintf (temp_filename, sizeof(temp_filename) - 1, "%s%d.art", TMPDIR, (int) process_id);
+	make_base_group_path(group->spooldir, group->name, temp_filename);
+	snprintf(buf, sizeof(buf), "%ld", article->artnum);
+	joinpath(article_filename, temp_filename, buf);
+	snprintf(temp_filename, sizeof(temp_filename), "%s%d.art", TMPDIR, (int) process_id);
 
-	if (!backup_file (article_filename, temp_filename))
+	if (!backup_file(article_filename, temp_filename))
 		return FALSE;
 
-	if (invoke_editor (temp_filename, 1)) {
-		rename_file (temp_filename, article_filename);
-		ret = TRUE;
-	} else {
-		unlink (temp_filename);
-		ret = FALSE;
-	}
+	if (!invoke_editor(temp_filename, 1)) {
+		unlink(temp_filename);
+		return FALSE;
+	} else
+		rename_file(temp_filename, article_filename);
 
-	return ret;
+	return TRUE;
 }
