@@ -3,10 +3,10 @@
  *  Module    : prompt.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 1997-12-31
+ *  Updated   : 2004-06-12
  *  Notes     :
  *
- * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2004 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Iain Lea.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -48,30 +45,39 @@
 #	include "menukeys.h"
 #endif /* !MENUKEYS_H */
 
+static char *prompt_slk_message;	/* prompt message for prompt_slk_redraw */
+
+/*
+ * Local prototypes
+ */
+static int prompt_list(int row, int col, int var, constext *help_text, constext *prompt_text, constext *list[], int size);
+
+
 /*
  *  prompt_num
  *  get a number from the user
  *  Return -1 if missing or bad number typed
  */
 int
-prompt_num (
+prompt_num(
 	int ch,
 	const char *prompt)
 {
 	char *p;
+	char tmp[LEN];
 	int num;
 
-	clear_message ();
+	clear_message();
 
-	sprintf (mesg, "%c", ch);
+	snprintf(tmp, sizeof(tmp), "%c", ch);
 
-	if ((p = tin_getline (prompt, TRUE, mesg, 0, FALSE, HIST_OTHER)) != (char *) 0) {
-		strcpy (mesg, p);
-		num = atoi (mesg);
+	if ((p = tin_getline(prompt, TRUE, tmp, 0, FALSE, HIST_OTHER)) != NULL) {
+		STRCPY(tmp, p);
+		num = atoi(tmp);
 	} else
 		num = -1;
 
-	clear_message ();
+	clear_message();
 
 	return num;
 }
@@ -81,15 +87,15 @@ prompt_num (
  *  prompt_string
  *  get a string from the user
  *  Return TRUE if a valid string was typed, FALSE otherwise
- *  TODO no bounds checking on buf size, tin_getline() defaults to 1024
+ *  TODO: no bounds checking on buf size, tin_getline() defaults to 1024
  */
 t_bool
-prompt_string (
+prompt_string(
 	const char *prompt,
 	char *buf,
 	int which_hist)
 {
-	return prompt_default_string (prompt, buf, 0, (char *) NULL, which_hist);
+	return prompt_default_string(prompt, buf, 0, (char *) NULL, which_hist);
 }
 
 
@@ -99,7 +105,7 @@ prompt_string (
  * Return TRUE if a valid string was typed, FALSE otherwise
  */
 t_bool
-prompt_default_string (
+prompt_default_string(
 	const char *prompt,
 	char *buf,
 	int buf_len,
@@ -108,16 +114,16 @@ prompt_default_string (
 {
 	char *p;
 
-	clear_message ();
+	clear_message();
 
-	if ((p = tin_getline (prompt, FALSE, default_prompt, buf_len, FALSE, which_hist)) == (char *) 0) {
+	if ((p = tin_getline(prompt, FALSE, default_prompt, buf_len, FALSE, which_hist)) == NULL) {
 		buf[0] = '\0';
-		clear_message ();
+		clear_message();
 		return FALSE;
 	}
-	strcpy (buf, p);
+	strcpy(buf, p);
 
-	clear_message ();
+	clear_message();
 
 	return TRUE;
 }
@@ -129,7 +135,7 @@ prompt_default_string (
  *  Return TRUE if a valid string was typed, FALSE otherwise
  */
 t_bool
-prompt_menu_string (
+prompt_menu_string(
 	int line,
 	const char *prompt,
 	char *var)
@@ -143,12 +149,12 @@ prompt_menu_string (
 	 */
 	fflush(stdin);
 
-	MoveCursor (line, 0);
+	MoveCursor(line, 0);
 
-	if ((p = tin_getline (prompt, FALSE, var, 0, FALSE, HIST_OTHER)) == (char *) 0)
+	if ((p = tin_getline(prompt, FALSE, var, 0, FALSE, HIST_OTHER)) == NULL)
 		return FALSE;
 
-	strcpy (var, p);
+	strcpy(var, p);
 
 	return TRUE;
 }
@@ -158,42 +164,42 @@ prompt_menu_string (
  * prompt_yn
  * prompt user for 'y'es or 'n'o decision. "prompt" will be displayed in line
  * "line" giving the default answer "default_answer".
- * TODO 'line' is constant - can we remove it ?
+ * TODO: 'line' is constant - can we remove it ?
  * The function returns 1 if the user decided "yes", -1 if the user wanted
  * to escape, or 0 for any other key or decision.
  */
 int
-prompt_yn (
+prompt_yn(
 	int line,
 	const char *prompt,
 	t_bool default_answer)
 {
 	char *keyprompt;
 	char keyno[MAXKEYLEN], keyyes[MAXKEYLEN];
-	int ch = 'y', prompt_ch = 'y';
+	int ch = 'y', prompt_ch = 'y'; /* why not iKeyPromptYes? */
 	size_t maxlen;
 	t_bool yn_loop = TRUE;
 
 /*	fflush(stdin); */		/* Prevent finger trouble from making important decisions */
 
-	(void) printascii (keyyes, (default_answer ? toupper(map_to_local(iKeyPromptYes, &menukeymap.prompt_yn)) : map_to_local(iKeyPromptYes, &menukeymap.prompt_yn)));
-	(void) printascii (keyno, (!default_answer ? toupper(map_to_local(iKeyPromptNo, &menukeymap.prompt_yn)) : map_to_local(iKeyPromptNo, &menukeymap.prompt_yn)));
-	maxlen = MAX(strlen (keyyes), strlen (keyno));
+	(void) printascii(keyyes, (default_answer ? toupper(map_to_local(iKeyPromptYes, &menukeymap.prompt_yn)) : map_to_local(iKeyPromptYes, &menukeymap.prompt_yn)));
+	(void) printascii(keyno, (!default_answer ? toupper(map_to_local(iKeyPromptNo, &menukeymap.prompt_yn)) : map_to_local(iKeyPromptNo, &menukeymap.prompt_yn)));
+	maxlen = MAX(strlen(keyyes), strlen(keyno));
 
 	while (yn_loop) {
-		prompt_ch = map_to_local ((default_answer ? iKeyPromptYes : iKeyPromptNo), &menukeymap.prompt_yn);
+		prompt_ch = map_to_local((default_answer ? iKeyPromptYes : iKeyPromptNo), &menukeymap.prompt_yn);
 		keyprompt = (default_answer ? keyyes : keyno);
 
 		if (!cmd_line) {
-			MoveCursor (line, 0);
-			CleartoEOLN ();
+			MoveCursor(line, 0);
+			CleartoEOLN();
 		}
-		my_printf ("%s (%s/%s) %-*s", prompt, keyyes, keyno, (int)maxlen, keyprompt);
+		my_printf("%s (%s/%s) %-*s", prompt, keyyes, keyno, (int) maxlen, keyprompt);
 		if (!cmd_line)
-			cursoron ();
-		my_flush ();
+			cursoron();
+		my_flush();
 		if (!cmd_line)
-			MoveCursor (line, (int) strlen (prompt) + strlen(keyyes) + strlen(keyno) + 5);
+			MoveCursor(line, (int) strlen(prompt) + strlen(keyyes) + strlen(keyno) + 5);
 
 		if (((ch = (char) ReadCh()) == '\n') || (ch == '\r'))
 			ch = prompt_ch;
@@ -205,7 +211,7 @@ prompt_yn (
 #	ifdef HAVE_KEY_PREFIX
 			case KEY_PREFIX:
 #	endif /* HAVE_KEY_PREFIX */
-				switch (get_arrow_key (ch)) {
+				switch (get_arrow_key(ch)) {
 
 					case KEYMAP_UP:
 					case KEYMAP_DOWN:
@@ -232,16 +238,16 @@ prompt_yn (
 
 	if (!cmd_line) {
 		if (line == cLINES)
-			clear_message ();
+			clear_message();
 		else {
-			MoveCursor (line, (int) strlen (prompt));
-			my_fputc (((ch == iKeyAbort) ? prompt_ch : ch), stdout);
+			MoveCursor(line, (int) strlen(prompt));
+			my_fputc(((ch == iKeyAbort) ? prompt_ch : ch), stdout);
 		}
-		cursoroff ();
-		my_flush ();
+		cursoroff();
+		my_flush();
 	}
 
-	return (tolower ((unsigned char) map_to_default (ch, &menukeymap.prompt_yn)) == tolower ((unsigned char)iKeyPromptYes)) ? 1 : (ch == iKeyAbort) ? -1 : 0;
+	return (tolower((unsigned char) map_to_default(ch, &menukeymap.prompt_yn)) == tolower((unsigned char)iKeyPromptYes)) ? 1 : (ch == iKeyAbort) ? -1 : 0;
 }
 
 
@@ -253,8 +259,8 @@ prompt_yn (
  * ESC is used to abort any changes, RET saves changes.
  * The new value is returned.
  */
-int
-prompt_list (
+static int
+prompt_list(
 	int row,
 	int col,
 	int var,
@@ -264,7 +270,7 @@ prompt_list (
 	int size)
 {
 	int ch, var_orig;
-	int i;
+	int i, offset;
 	int adjust = (strcasecmp(_(list[0]), _(txt_default)) == 0);
 	size_t width = 0;
 
@@ -278,12 +284,28 @@ prompt_list (
 	for (i = 0; i < size; i++)
 		width = MAX(width, strlen(_(list[i])));
 
-	show_menu_help (help_text);
-	cursoron ();
+	show_menu_help(help_text);
+	cursoron();
 
 	do {
-		MoveCursor (row, col + (int) strlen (_(prompt_text)));
-		if ((ch = (char) ReadCh ()) == ' ') {
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+		wchar_t *wbuf;
+
+		if ((wbuf = char2wchar_t(_(prompt_text))) != NULL) {
+			wconvert_to_printable(wbuf);
+			offset = wcswidth(wbuf, wcslen(wbuf) + 1);
+			if (offset == -1) {
+				/* something went wrong, use wcslen as fallback */
+				offset = (int) wcslen(wbuf);
+			}
+
+			free(wbuf);
+		} else
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+			offset = (int) strlen(_(prompt_text));
+
+		MoveCursor(row, col + offset);
+		if ((ch = (char) ReadCh()) == ' ') {
 
 			/*
 			 * Increment list, looping around at the max
@@ -291,39 +313,56 @@ prompt_list (
 			++var;
 			var %= size;
 
-			my_printf("%-*s", (int)width, _(list[var]));
-			my_flush ();
+			my_printf("%-*s", (int) width, _(list[var]));
+			my_flush();
 		}
 	} while (ch != '\r' && ch != '\n' && ch != ESC);
 
 	if (ch == ESC) {
 		var = var_orig;
 
-		my_printf("%-*s", (int)width, _(list[var]));
-		my_flush ();
+		my_printf("%-*s", (int) width, _(list[var]));
+		my_flush();
 	}
 
-	cursoroff ();
+	cursoroff();
 
-	return(var - adjust);
+	return (var - adjust);
 }
 
 
 /*
- * Special case of prompt_list() Toggle between ON and OFF
+ * Special case of prompt_option_list() Toggle between ON and OFF
+ * The function returns TRUE, if the value was changed, FALSE otherwise.
  */
-void
-prompt_on_off (
-	int row,
-	int col,
-	t_bool *var,
-	constext *help_text,
-	constext *prompt_text)
+t_bool
+prompt_option_on_off(
+	int option)
 {
-	t_bool ret;
+	char prompt[LEN];
+	t_bool *variable = OPT_ON_OFF_list[option_table[option].var_index];
+	t_bool old_value = *variable;
 
-	ret = prompt_list (row, col, (int)*var, help_text, _(prompt_text), txt_onoff, 2);
-	*var = (ret != 0);
+	fmt_option_prompt(prompt, sizeof(prompt), TRUE, option);
+	*variable = prompt_list(option_row(option), 0, *variable, option_table[option].txt->help, prompt, txt_onoff, 2) ? TRUE: FALSE;
+	return *variable != old_value;
+}
+
+
+/*
+ * The function returns TRUE, if the value was changed, FALSE otherwise.
+ */
+t_bool
+prompt_option_list(
+	int option)
+{
+	char prompt[LEN];
+	int *variable = option_table[option].variable;
+	int old_value = *variable;
+
+	fmt_option_prompt(prompt, sizeof(prompt), TRUE, option);
+	*variable = prompt_list(option_row(option), 0, *variable, option_table[option].txt->help, prompt, option_table[option].opt_list, option_table[option].opt_count);
+	return *variable != old_value;
 }
 
 
@@ -336,15 +375,15 @@ prompt_on_off (
  * The function returns TRUE, if the value was changed, FALSE otherwise.
  */
 t_bool
-prompt_option_string (
+prompt_option_string(
 	int option) /* return value is always ignored */
 {
 	char prompt[LEN];
 	char *variable = OPT_STRING_list[option_table[option].var_index];
 
-	show_menu_help (option_table[option].txt->help);
-	fmt_option_prompt (prompt, sizeof(prompt)-1, TRUE, option);
-	return prompt_menu_string (option_row(option), prompt, variable);
+	show_menu_help(option_table[option].txt->help);
+	fmt_option_prompt(prompt, sizeof(prompt) - 1, TRUE, option);
+	return prompt_menu_string(option_row(option), prompt, variable);
 }
 
 
@@ -357,7 +396,7 @@ prompt_option_string (
  * The function returns TRUE if the value was changed, FALSE otherwise.
  */
 t_bool
-prompt_option_num (
+prompt_option_num(
 	int option) /* return value is always ignored */
 {
 	char prompt[LEN];
@@ -365,19 +404,19 @@ prompt_option_num (
 	char *p;
 	int num;
 
-	show_menu_help (option_table[option].txt->help);
-	MoveCursor (option_row(option), 0);
-	fmt_option_prompt (prompt, sizeof(prompt)-1, TRUE, option);
-	sprintf (&number[0], "%d", *(option_table[option].variable));
+	show_menu_help(option_table[option].txt->help);
+	MoveCursor(option_row(option), 0);
+	fmt_option_prompt(prompt, sizeof(prompt) - 1, TRUE, option);
+	snprintf(&number[0], sizeof(number), "%d", *(option_table[option].variable));
 
-	if ((p = tin_getline (prompt, 2, number, 0, FALSE, HIST_OTHER)) == (char *) 0)
+	if ((p = tin_getline(prompt, 2, number, 0, FALSE, HIST_OTHER)) == NULL)
 		return FALSE;
 
-	strcpy (number, p);
-	num = atoi (number);
+	STRCPY(number, p);
+	num = atoi(number);
 	*(option_table[option].variable) = num;
 
-	clear_message ();
+	clear_message();
 
 	return TRUE;
 }
@@ -391,7 +430,7 @@ prompt_option_num (
  * The function returns TRUE if the value was changed, FALSE otherwise.
  */
 t_bool
-prompt_option_char (
+prompt_option_char(
 	int option) /* return value is always ignored */
 {
 	char prompt[LEN];
@@ -402,27 +441,33 @@ prompt_option_char (
 	input[0] = *variable;
 	input[1] = '\0';
 
-	show_menu_help (option_table[option].txt->help);
-	MoveCursor (option_row(option), 0);
-	fmt_option_prompt (prompt, sizeof(prompt)-1, TRUE, option);
+	do {
+		show_menu_help(option_table[option].txt->help);
+		MoveCursor(option_row(option), 0);
+		fmt_option_prompt(prompt, sizeof(prompt) - 1, TRUE, option);
 
-	if ((p = tin_getline (prompt, FALSE, p, 1, FALSE, HIST_OTHER)) == (char *) 0)
-		return FALSE;
+		if ((p = tin_getline(prompt, FALSE, p, 1, FALSE, HIST_OTHER)) == NULL) {
+			clear_message();
+			return FALSE;
+		}
+		if (!*p)
+			info_message(_(txt_info_enter_valid_character));
+	} while (!*p);
 
 	*variable = p[0];
 
-	clear_message ();
-
+	clear_message();
 	return TRUE;
 }
 
+
 /*
  * Get a string. Make it the new default.
  * If none given, use the default.
  * Return the string or NULL if we can't get anything useful
  */
 char *
-prompt_string_default (
+prompt_string_default(
 	const char *prompt,
 	char *def,
 	const char *failtext,
@@ -432,16 +477,16 @@ prompt_string_default (
 
 	clear_message();
 
-	if (!prompt_string (prompt, pattern, history)) {
+	if (!prompt_string(prompt, pattern, history)) {
 		clear_message();
 		return NULL;
 	}
 
 	if (pattern[0] != '\0')			/* got a string - make it the default */
-		my_strncpy (def, pattern, LEN);
+		my_strncpy(def, pattern, LEN);
 	else {
-		if (def[0] == '\0')	{		/* no default - give up */
-			error_message (failtext);
+		if (def[0] == '\0') {		/* no default - give up */
+			error_message(failtext);
 			return NULL;
 		}
 	}
@@ -456,13 +501,13 @@ prompt_string_default (
  * in arts[], else ART_UNAVAILABLE
  */
 int
-prompt_msgid (
+prompt_msgid(
 	void)
 {
 	char buf[LEN];
 
 	if (prompt_string(_(txt_enter_message_id), buf + 1, HIST_MESSAGE_ID) && buf[1]) {
-		char *ptr = buf + 1;
+		char *ptr = str_trim(buf + 1);
 		struct t_msgid *msgid;
 
 		/*
@@ -492,7 +537,7 @@ prompt_msgid (
 		 * no way to display it
 		 */
 		if (which_thread(msgid->article) == -1) {
-			info_message (_(txt_no_last_message));
+			info_message(_(txt_no_last_message));
 			return ART_UNAVAILABLE;
 		}
 
@@ -506,23 +551,36 @@ prompt_msgid (
 /*
  * Format a message such that it'll fit within the screen width
  * Useful for fitting long Subjects and newsgroup names into prompts
- * TODO - maybe add a '...' to the string to show it was truncated. See center_line()
+ * result will contain a pointer to the malloced memory containing the
+ * sized message
  */
 char *
-sized_message (
+sized_message(
+	char **result,
 	const char *format,
 	const char *subject)
 {
-	/* The formatting info (%.*s) wastes 4 chars, but our prompt needs 1 char */
-	int have = cCOLS - strlen (format) + 4 - 1;
-	int want = strlen(subject);
+	char *buf;
+	int max_len;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	wchar_t *wformat;
 
-	if (want > 0 && subject[want - 1] == '\n')
-		want--;
-	if (have > want)
-		have = want;
-	sprintf (mesg, format, have, subject);
-	return mesg;
+	if ((wformat = char2wchar_t(format)) != NULL) {
+		wconvert_to_printable(wformat);
+		/* The formatting info (%s) wastes 2 chars, but our prompt needs 1 char */
+		max_len = cCOLS - wcswidth(wformat, wcslen(wformat) + 1) + 2 - 1;
+		free(wformat);
+	} else
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+		max_len = cCOLS - strlen(format) + 2 - 1;	/* The formatting info (%s) wastes 2 chars, but our prompt needs 1 char */
+
+	buf = my_malloc(strlen(subject) + 1);
+	strunc(subject, buf, strlen(subject) + 1, max_len);
+
+	*result = fmt_string(format, buf);
+	free(buf);
+
+	return *result;
 }
 
 
@@ -531,7 +589,7 @@ sized_message (
  * eg, Press a)ppend, o)verwrite, q)uit :
  */
 int
-prompt_slk_response (
+prompt_slk_response(
 	int ch_default,
 	const t_menukeys /* char */ *responses,
 	const char *fmt,
@@ -541,24 +599,79 @@ prompt_slk_response (
 	char ch;
 	char buf[LEN];
 
-	va_start (ap, fmt);
-	vsnprintf (buf, sizeof(buf) - 1, fmt, ap);	/* We need to do this, else wait_message() will clobber us */
-	va_end (ap);
+	va_start(ap, fmt);
+	vsnprintf(buf, sizeof(buf), fmt, ap);
+	va_end(ap);
+
+	prompt_slk_message = my_malloc(strlen(buf) + 2);
+	ch_default = map_to_local(ch_default, responses);
+	snprintf(prompt_slk_message, strlen(buf) + 2, "%s%c", buf, ch_default);
+
+	input_context = cPromptSLK;
 
-	ch_default = map_to_local (ch_default, responses);
 	do {
-		wait_message (0, "%s%c", buf, ch_default);
+		prompt_slk_redraw();		/* draw the prompt */
 
-		/* Get the cursor _just_ right */
-		MoveCursor (cLINES, (int) strlen (buf));
-
-		if ((ch = ReadCh ()) == '\r' || ch == '\n')
+		if ((ch = ReadCh()) == '\r' || ch == '\n')
 			ch = ch_default;
 
-	} while (!strchr (responses->localkeys, ch));
+		/*
+		 * TODO: ignore special-keys which are represented as a
+		 *       multibyte ESC-seq to avoid interpreting them as 'ESC' only
+		 *       like it's done in the ugly code below.
+		 */
+#if 0
+		if (ch == ESC) {
+			switch (get_arrow_key(ch)) {
+				case KEYMAP_UP:
+				case KEYMAP_DOWN:
+				case KEYMAP_LEFT:
+				case KEYMAP_RIGHT:
+				case KEYMAP_PAGE_DOWN:
+				case KEYMAP_PAGE_UP:
+				case KEYMAP_HOME:
+				case KEYMAP_END:
+					ch = '\0';
+					break;
 
-	clear_message ();
-	return map_to_default (ch, responses);
+				default:
+					break;
+			}
+		}
+#endif /* 0 */
+
+	} while (!strchr(responses->localkeys, ch));
+
+	input_context = cNone;
+	FreeAndNull(prompt_slk_message);
+
+	clear_message();
+	return map_to_default(ch, responses);
+}
+
+
+/* (Re)draws the prompt message for prompt_slk_response() */
+void
+prompt_slk_redraw(
+	void)
+{
+	int column;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	wchar_t *wtmp;
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+
+	wait_message(0, "%s", prompt_slk_message);
+
+	/* get the cursor _just_ right */
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	if ((wtmp = char2wchar_t(prompt_slk_message)) != NULL) {
+		wconvert_to_printable(wtmp);
+		column = wcswidth(wtmp, wcslen(wtmp) + 1) - 1;
+		free(wtmp);
+	} else
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+		column = (int) strlen(prompt_slk_message) - 1;
+	MoveCursor(cLINES, column);
 }
 
 
@@ -569,7 +682,7 @@ prompt_slk_response (
  * ii) CTRL, SHIFT etc don't work
  */
 void
-prompt_continue (
+prompt_continue(
 	void)
 {
 	int ch;
@@ -577,16 +690,15 @@ prompt_continue (
 #ifdef USE_CURSES
 	cmd_line = TRUE;
 #endif /* USE_CURSES */
-	info_message (_(txt_return_key));
-	ch = ReadCh ();
+	info_message(_(txt_return_key));
 
-	switch (ch) {
+	switch ((ch = ReadCh())) {
 		case ESC:
 #	ifdef HAVE_KEY_PREFIX
 		case KEY_PREFIX:
 #	endif /* HAVE_KEY_PREFIX */
 			(void) get_arrow_key(ch);
-			nobreak;	/* FALLTHROUGH */
+			/* FALLTHROUGH */
 		default:
 			break;
 	}
