@@ -1,14 +1,14 @@
 /*
  *  Project   : tin - a Usenet reader
  *  Module    : refs.c
- *  Author    : Jason Faultless <jason@radar.tele2.co.uk>
+ *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 1996-05-09
- *  Updated   : 1998-01-04
+ *  Updated   : 2004-03-01
  *  Notes     : Cacheing of message ids / References based threading
  *  Credits   : Richard Hodson <richard@macgyver.tele2.co.uk>
  *              hash_msgid, free_msgid
  *
- * Copyright (c) 1996-2002 Jason Faultless <jason@radar.tele2.co.uk>
+ * Copyright (c) 1996-2004 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -19,10 +19,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Jason Faultless.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -45,7 +42,7 @@
 #endif /* !TIN_H */
 
 #define MAX_REFS	100			/* Limit recursion depth */
-#define REF_SEP	" "			/* Separator chars in ref headers */
+#define REF_SEP	" \t"			/* Separator chars in ref headers */
 
 /* Produce disgusting amounts of output to help me tame this thing */
 #undef DEBUG_REFS
@@ -60,15 +57,16 @@ FILE *dbgfd;
 /*
  * local prototypes
  */
-static char * _get_references (struct t_msgid *refptr, int depth);
-static struct t_msgid *add_msgid (int key, const char *msgid, struct t_msgid *newparent);
-static struct t_msgid *find_next (struct t_msgid *ptr);
-static struct t_msgid *parse_references (char *r);
-static unsigned int hash_msgid (const char *key);
-static void add_to_parent (struct t_msgid *ptr);
-static void build_thread (struct t_msgid *ptr);
+static char *_get_references(struct t_msgid *refptr, int depth);
+static struct t_msgid *add_msgid(int key, const char *msgid, struct t_msgid *newparent);
+static struct t_msgid *find_next(struct t_msgid *ptr);
+static struct t_msgid *parse_references(char *r);
+static t_bool valid_msgid(const char *msgid);
+static unsigned int hash_msgid(const char *key);
+static void add_to_parent(struct t_msgid *ptr);
+static void build_thread(struct t_msgid *ptr);
 #ifdef DEBUG_REFS
-	static void dump_thread (FILE *fp, struct t_msgid *msgid, int level);
+	static void dump_thread(FILE *fp, struct t_msgid *msgid, int level);
 	static void dump_msgid_thread(struct t_msgid *ptr, int level);
 	static void dump_msgid_threads(void);
 #endif /* DEBUG_REFS */
@@ -77,7 +75,7 @@ static void build_thread (struct t_msgid *ptr);
 #endif /* 0 */
 
 /*
- * Set if the the sorting algorithm goes 'upwards'
+ * Set if the sorting algorithm goes 'upwards'
  */
 static t_bool sort_ascend;
 
@@ -87,16 +85,16 @@ static t_bool sort_ascend;
 static struct t_msgid *msgids[MSGID_HASH_SIZE] = {0};
 
 /*
- * This part of the code deals with the cacheing and retrieval
- * of Message-id and References headers
+ * This part of the code deals with the caching and retrieval
+ * of Message-ID and References headers
  *
  * Rationale:
- *    Even though the Message-id is unique to an article, the References
+ *    Even though the Message-ID is unique to an article, the References
  *    field contains msgids from elsewhere in the group. As the expiry
  *    period increases, so does the redundancy of data.
  *    At the time of writing, comp.os.ms-windows.advocacy held ~850
  *    articles. The references fields contained 192k of text, of which
- *    169k was saved using the new cacheing.
+ *    169k was saved using the new caching.
  *
  *    When threading on Refs, a much better view of the original thread
  *    can be built up using this data, and threading is much faster
@@ -117,7 +115,7 @@ static struct t_msgid *msgids[MSGID_HASH_SIZE] = {0};
  * MSGID_HASH_SIZE is a prime of order 2^11
  */
 static unsigned int
-hash_msgid (
+hash_msgid(
 	const char *key)
 {
 	unsigned int hash = 0;
@@ -137,7 +135,7 @@ hash_msgid (
  * Thread us into our parents' list of children.
  */
 static void
-add_to_parent (
+add_to_parent(
 	struct t_msgid *ptr)
 {
 	struct t_msgid *p;
@@ -148,7 +146,7 @@ add_to_parent (
 	/*
 	 * Trivial case - if we are the first child (followup)
 	 */
-	if (ptr->parent->child == NULL)	{
+	if (ptr->parent->child == NULL) {
 		ptr->parent->child = ptr;
 		return;
 	}
@@ -179,6 +177,36 @@ add_to_parent (
 }
 
 
+/*
+ * Checks if Message-ID has valid format
+ * Returns TRUE if it does, FALSE if it does not
+ *
+ * TODO: combine with post.c:damaged_id()?
+ */
+static t_bool
+valid_msgid(
+	const char *msgid)
+{
+	size_t mlen = 0;
+	t_bool at_present = 0;
+
+	if (!msgid || *msgid != '<')
+		return FALSE;
+
+	while (isascii((unsigned char) *msgid) && isgraph((unsigned char) *msgid) && !iscntrl((unsigned char) *msgid) && *msgid != '>') {
+		if (*msgid == '@')
+			at_present = TRUE;
+		mlen++;
+		msgid++;
+	}
+
+	if (!at_present || (*msgid != '>') || mlen <= 2 || *(msgid + 1))
+		return FALSE;
+
+	return TRUE;
+}
+
+
 /*
  * Adds or updates a message id in the cache.
  * We return a ptr to the msgid, whether located or newly created.
@@ -204,7 +232,7 @@ add_to_parent (
  *        overview database.
  */
 static struct t_msgid *
-add_msgid (
+add_msgid(
 	int key,
 	const char *msgid,
 	struct t_msgid *newparent)
@@ -227,7 +255,6 @@ add_msgid (
 	 * Broken software will sometimes damage the case of a message-id.
 	 */
 	for (i = msgids[h]; i != NULL; i = i->next) {
-
 		if (strcasecmp(i->txt, msgid) != 0)				/* No match yet */
 			continue;
 
@@ -252,7 +279,6 @@ add_msgid (
 		 *         Change parent from null -> not-null & update ptrs
 		 */
 		if (i->parent == NULL) {
-
 			/*
 			 * Detect & ignore circular reference paths by looking for the
 			 * new parent in this thread
@@ -295,7 +321,7 @@ add_msgid (
 	/*
 	 * This is a new node, so build a structure for it
 	 */
-	ptr = (struct t_msgid *)my_malloc(sizeof(struct t_msgid) + strlen(msgid));
+	ptr = my_malloc(sizeof(struct t_msgid) + strlen(msgid));
 
 	strcpy(ptr->txt, msgid);
 	ptr->parent = newparent;
@@ -319,7 +345,7 @@ add_msgid (
  * not found.
  */
 struct t_msgid *
-find_msgid (
+find_msgid(
 	const char *msgid)
 {
 	unsigned int h;
@@ -350,7 +376,7 @@ find_msgid (
  * The space saving vs. storing the refs as a single string is significant.
  */
 static struct t_msgid *
-parse_references (
+parse_references(
 	char *r)
 {
 	char *ptr;
@@ -362,8 +388,7 @@ parse_references (
 	DEBUG_PRINT((dbgfd, "parse_references: %s\n", r));
 
 	/*
-	 * Break the refs down, using a space as delimiters
-	 * A msgid can't contain a space, right ?
+	 * Break the refs down, using REF_SEP as delimiters
 	 */
 	if ((ptr = strtok(r, REF_SEP)) == NULL)
 		return NULL;
@@ -372,11 +397,17 @@ parse_references (
 	 * By definition, the head of the thread has no parent
 	 */
 	parent = NULL;
+
+	if (!valid_msgid(ptr))
+		return NULL;
+
 	current = add_msgid(REF_REF, ptr, parent);
 
 	while ((ptr = strtok(NULL, REF_SEP)) != NULL) {
-		parent = current;
-		current = add_msgid(REF_REF, ptr, parent);
+		if (valid_msgid(ptr)) {
+			parent = current;
+			current = add_msgid(REF_REF, ptr, parent);
+		}
 	}
 
 	return current;
@@ -385,47 +416,46 @@ parse_references (
 
 /*
  * Reconstruct the References: field from the parent pointers
- * NB: The original Refs: can be no longer than HEADER_LEN (see open.c)
- *     Broken headers sometimes have malformed or circular reference
- *	    lists, which we strive to work around.
+ * NB: In deep threads this can lead to a very long line. If you want to use
+ *     this function to build a Reference: line for posting be aware that the
+ *     server might refuse long lines -- short it accordingly!
  */
 static char *
-_get_references (
+_get_references(
 	struct t_msgid *refptr,
 	int depth)
 {
 	char *refs;
-	static size_t len;							/* Accumulated size */
+	static size_t len;		/* Accumulated size */
+	static size_t pos;		/* current insertion position */
 
+	if (depth == 1)
+		len = 0;
+
+	len += strlen(refptr->txt) + 1;	/* msgid + space */
 	if (refptr->parent == NULL || depth > MAX_REFS) {
 
 #ifdef DEBUG_REFS
 		if (depth > MAX_REFS)
 			error_message("Warning: Too many refs near to %s. Truncated\n", refptr->txt);
 #endif /* DEBUG_REFS */
-		refs = (char *) my_malloc(HEADER_LEN);
-		len = 0;
+		refs = my_malloc(len + 1);	/* total length + nullbyte */
+		pos = 0;
 	} else
 		refs = _get_references(refptr->parent, depth + 1);
 
-	/*
-	 * Attempt at damage limitation in case of broken Refs fields
-	 */
-	if (len + strlen(refptr->txt) < (998 - strlen("References: "))) {
-		sprintf(refs + len, "%s ", refptr->txt);
-		len = strlen(refs);
-	}
+	sprintf(refs + pos, "%s ", refptr->txt);
+	pos = strlen(refs);
 
 	return refs;
 }
 
 
 /*
- * A wrapper to the above, null terminate the string and shrink it
- * to correct size
+ * A wrapper to the above, null terminate the string
  */
 char *
-get_references (
+get_references(
 	struct t_msgid *refptr)
 {
 	char *refs;
@@ -437,7 +467,6 @@ get_references (
 	refs = _get_references(refptr, 1);
 	len = strlen(refs);
 	refs[len - 1] = '\0';
-	refs = (char *) my_realloc(refs, len);
 
 	return refs;
 }
@@ -448,7 +477,7 @@ get_references (
  * normally only needed when entering a new group
  */
 void
-free_msgids (
+free_msgids(
 	void)
 {
 	int i;
@@ -473,7 +502,7 @@ free_msgids (
 
 #if 0	/* But please don't remove it */
 static void
-dump_msgids (
+dump_msgids(
 	void)
 {
 	int i;
@@ -482,11 +511,8 @@ dump_msgids (
 	my_fprintf(stderr, "Dumping...\n");
 
 	for (i = 0; i < MSGID_HASH_SIZE; i++) {
-
 		if (msgids[i] != NULL) {
-
 			my_fprintf(stderr, "node %d", i);
-
 			for (ptr = msgids[i]; ptr != NULL; ptr = ptr->next)
 				my_fprintf(stderr, " -> %s", ptr->txt);
 
@@ -531,7 +557,7 @@ dump_msgids (
  * These pointers are automatically set up when we read in the
  * headers for a group.
  *
- * It remains for us to fill in the .thread and .inthread ptrs in
+ * It remains for us to fill in the .thread and .prev ptrs in
  * each article that exists in the spool, using the intelligence of
  * the reference tree to locate the 'next' article in the thread.
  *
@@ -541,7 +567,7 @@ dump_msgids (
  * rethread.
  */
 void
-clear_art_ptrs (
+clear_art_ptrs(
 	void)
 {
 	int i;
@@ -560,7 +586,7 @@ clear_art_ptrs (
  */
 #ifdef DEBUG_REFS
 static void
-dump_thread (
+dump_thread(
 	FILE *fp,
 	struct t_msgid *msgid,
 	int level)
@@ -600,7 +626,7 @@ dump_thread (
  * A thread is defined as a starting article with no parent
  */
 static void
-dump_msgid_thread (
+dump_msgid_thread(
 	struct t_msgid *ptr,
 	int level)
 {
@@ -617,7 +643,7 @@ dump_msgid_thread (
 
 
 static void
-dump_msgid_threads (
+dump_msgid_threads(
 	void)
 {
 	int i;
@@ -654,13 +680,13 @@ dump_msgid_threads (
  *  It doesn't point to an article OR
  *     (it's already threaded/expired OR it has been autokilled)
  */
-/* We SHOULD skip killed in the normal case */
-#	define SKIP_ART(ptr)	\
+#define SKIP_ART(ptr)	\
 	(ptr && (ptr->article == ART_UNAVAILABLE || \
-		(arts[ptr->article].thread != ART_NORMAL || (tinrc.kill_level != KILL_THREAD && arts[ptr->article].killed))))
+		(arts[ptr->article].thread != ART_UNTHREADED || \
+			(tinrc.kill_level == KILL_NOTHREAD && arts[ptr->article].killed))))
 
 static struct t_msgid *
-find_next (
+find_next(
 	struct t_msgid *ptr)
 {
 	static t_bool bottom = FALSE;
@@ -706,7 +732,7 @@ find_next (
 		 */
 		if (ptr->child == NULL && ptr->sibling == NULL) {
 
-			while(ptr != NULL && ptr->sibling == NULL)
+			while (ptr != NULL && ptr->sibling == NULL)
 				ptr = ptr->parent;
 
 			/*
@@ -725,11 +751,11 @@ find_next (
 
 
 /*
- * Run the .thread and .inthread pointers through the members of this
+ * Run the .thread and .prev pointers through the members of this
  * thread.
  */
 static void
-build_thread (
+build_thread(
 	struct t_msgid *ptr)
 {
 	struct t_msgid *newptr;
@@ -737,7 +763,7 @@ build_thread (
 	/*
 	 * If the root article is gone/expired/killed, find the first valid one
 	 */
-	if (SKIP_ART (ptr))
+	if (SKIP_ART(ptr))
 		ptr = find_next(ptr);
 
 	/*
@@ -745,7 +771,7 @@ build_thread (
 	 */
 	while ((newptr = find_next(ptr)) != NULL) {
 
-		arts[newptr->article].inthread = TRUE;
+		arts[newptr->article].prev = ptr->article;
 		arts[ptr->article].thread = newptr->article;
 
 		ptr = newptr;
@@ -759,7 +785,7 @@ build_thread (
  * parent / child / sibling  / article pointers in the msgid hash.
  */
 void
-thread_by_reference (
+thread_by_reference(
 	void)
 {
 	int i;
@@ -786,12 +812,12 @@ thread_by_reference (
 	fprintf(dbgfd, "Full dump of threading info...\n");
 	fprintf(dbgfd, "%3s %3s %3s %3s : %3s %3s\n", "#", "Par", "Sib", "Chd", "In", "Thd");
 
-	for (i = 0; i < top_art; i++) {
+	for_each_art(i) {
 		fprintf(dbgfd, "%3d %3d %3d %3d : %3d %3d : %.50s %s\n", i,
 			(arts[i].refptr->parent) ? arts[i].refptr->parent->article : -2,
 			(arts[i].refptr->sibling) ? arts[i].refptr->sibling->article : -2,
 			(arts[i].refptr->child) ? arts[i].refptr->child->article : -2,
-			arts[i].inthread, arts[i].thread, arts[i].refptr->txt, arts[i].subject);
+			arts[i].prev, arts[i].thread, arts[i].refptr->txt, arts[i].subject);
 	}
 
 	fclose(dbgfd);
@@ -812,7 +838,7 @@ thread_by_reference (
  * correct.
  */
 void
-collate_subjects (
+collate_subjects(
 	void)
 {
 	int i, j, art;
@@ -822,38 +848,36 @@ collate_subjects (
 	 * Run through the root messages of each thread. We have to traverse
 	 * using arts[] and not msgids[] to preserve the sorting.
 	 */
-	for (i = 0; i < top_art; i++) {
-
+	for_each_art(i) {
 		/*
 		 * Ignore already threaded and expired arts
 		 */
-		if (arts[i].inthread || IGNORE_ART(i))
+		if (arts[i].prev >= 0 || IGNORE_ART(i))
 			continue;
 
 		/*
 		 * Get the contents of the magic marker in the hashnode
 		 */
-		h = (struct t_hashnode *)(arts[i].subject - sizeof(int) - sizeof(void *)); /* FIXME: cast increases required alignment of target type */
-
+		h = (struct t_hashnode *) (arts[i].subject - sizeof(int) - sizeof(void *)); /* FIXME: cast increases required alignment of target type */
 		j = h->aptr;
 
 		if (j != -1 && j < i) {
-
 			/*
 			 * Modified form of the subject threading - the only difference
 			 * is that we have to add later threads onto the end of the
 			 * previous thread
 			 */
-			if (((arts[i].subject == arts[j].subject) ||
-						   ((arts[i].part || arts[i].patch) &&
-							 arts[i].archive == arts[j].archive))) {
-/*DEBUG_PRINT((dbgfd, "RES: %d is now inthread, at end of %d\n", i, j));*/
+			if (
+					(arts[i].subject == arts[j].subject) /* ||
+					(arts[i].archive && arts[j].archive && (arts[i].archive->name == arts[j].archive->name)) */
+			) {
+/*DEBUG_PRINT((dbgfd, "RES: %d is now previous, at end of %d\n", i, j));*/
 
 				for (art = j; arts[art].thread >= 0; art = arts[art].thread)
 					;
 
 				arts[art].thread = i;
-				arts[i].inthread = TRUE;
+				arts[i].prev = art;
 			}
 		}
 
@@ -883,7 +907,7 @@ collate_subjects (
  * 4) free() up the msgid and refs headers once cached
  */
 void
-build_references (
+build_references(
 	struct t_group *group)
 {
 	char *s;
@@ -894,64 +918,67 @@ build_references (
 	/*
 	 * The articles are currently unsorted, and are as they were put by setup_hard_base()
 	 */
-	if (group->attribute && group->attribute->sort_art_type != SORT_ARTICLES_BY_NOTHING)
-		sort_arts (group->attribute->sort_art_type);
+	if (group->attribute->sort_art_type != SORT_ARTICLES_BY_NOTHING)
+		sort_arts(group->attribute->sort_art_type);
 
-	if (group->attribute)
-		sort_ascend = (group->attribute->sort_art_type == SORT_ARTICLES_BY_SUBJ_ASCEND ||
-		               group->attribute->sort_art_type == SORT_ARTICLES_BY_FROM_ASCEND ||
-		               group->attribute->sort_art_type == SORT_ARTICLES_BY_DATE_ASCEND ||
-		               group->attribute->sort_art_type == SORT_ARTICLES_BY_SCORE_ASCEND);
+	sort_ascend = (group->attribute->sort_art_type == SORT_ARTICLES_BY_SUBJ_ASCEND ||
+	               group->attribute->sort_art_type == SORT_ARTICLES_BY_FROM_ASCEND ||
+	               group->attribute->sort_art_type == SORT_ARTICLES_BY_DATE_ASCEND ||
+	               group->attribute->sort_art_type == SORT_ARTICLES_BY_SCORE_ASCEND ||
+	               group->attribute->sort_art_type == SORT_ARTICLES_BY_LINES_ASCEND);
 
 #ifdef DEBUG_REFS
 	dbgfd = fopen("Refs.dump", "w");
 	SETVBUF(dbgfd, NULL, _IONBF, 0);
-	fprintf (dbgfd, "MSGID phase\n");
+	fprintf(dbgfd, "MSGID phase\n");
 #endif /* DEBUG_REFS */
 
 	/*
 	 * Add the Message-ID headers to the cache, using the last Reference
 	 * as the parent
 	 */
-	for (i = 0; i < top_art; i++) {
+	for_each_art(i) {
 		art = &arts[i];
 
 		if (art->refs) {
-
 			strip_line(art->refs);
 
 			/*
 			 * Add the last ref, and then trim it to save wasting time adding
 			 * it again later
 			 * Check for circular references to current article
+			 *
+			 * TODO: do this in a single pass
 			 */
-			while (((s = strrchr(art->refs, ' ')) != NULL) && (!strcmp(art->msgid, s + 1))) {
-				/*
-				 * Remove circular reference to current article
-				 */
-				DEBUG_PRINT((dbgfd, "removing circular reference to%s\n", s));
-				*s = '\0';
+			if ((s = strrchr(art->refs, '<')) != NULL) {
+				if (!strcmp(art->msgid, s)) {
+					/*
+					 * Remove circular reference to current article
+					 */
+					DEBUG_PRINT((dbgfd, "removing circular reference to: %s\n", s));
+					*s = '\0';
+				}
 			}
 			if (s != NULL) {
-				art->refptr = add_msgid(MSGID_REF, art->msgid, add_msgid(REF_REF, s + 1, NULL));
+				if (valid_msgid(art->msgid))
+					art->refptr = add_msgid(MSGID_REF, art->msgid, add_msgid(REF_REF, s, NULL));
 				*s = '\0';
 			} else {
-				art->refptr = add_msgid(MSGID_REF, art->msgid, add_msgid(REF_REF, art->refs, NULL));
-				free(art->refs);
-				art->refs = (char *) 0;
+				if (valid_msgid(art->msgid))
+					art->refptr = add_msgid(MSGID_REF, art->msgid, add_msgid(REF_REF, art->refs, NULL));
+				FreeAndNull(art->refs);
 			}
-
 		} else
-			art->refptr = add_msgid(MSGID_REF, art->msgid, NULL);
-
-		free(art->msgid);					/* Now cached - discard this */
+			if (valid_msgid(art->msgid))
+				art->refptr = add_msgid(MSGID_REF, art->msgid, NULL);
+		FreeAndNull(art->msgid);	/* Now cached - discard this */
 	}
 
 	DEBUG_PRINT((dbgfd, "REFS phase\n"));
 	/*
 	 * Add the References data to the cache
 	 */
-	for (i = 0; i < top_art; i++) {
+	for_each_art(i) {
 		if (!arts[i].refs)						/* No refs - skip */
 			continue;
 
@@ -964,15 +991,13 @@ build_references (
 
 		refs = parse_references(art->refs);
 
-		if (art->refptr->parent)
+		if (art->refptr && art->refptr->parent && valid_msgid(art->refptr->parent->txt))
 			add_msgid(REF_REF, art->refptr->parent->txt, refs);
 
-		free(art->refs);
+		FreeAndNull(art->refs);
 	}
 
 #ifdef DEBUG_REFS
 	fclose(dbgfd);
 #endif /* DEBUG_REFS */
 }
-
-/* end of refs.c */
