@@ -2,11 +2,11 @@
  *  Project   : tin - a Usenet reader
  *  Module    : rfc2047.c
  *  Author    : Chris Blum <chris@resolution.de>
- *  Created   : September '95
- *  Updated   : 1998-04-05
+ *  Created   : 1995-09-01
+ *  Updated   : 2004-03-10
  *  Notes     : MIME header encoding/decoding stuff
  *
- * Copyright (c) 1995-2002 Chris Blum <chris@resolution.de>
+ * Copyright (c) 1995-2004 Chris Blum <chris@resolution.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Chris Blum.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -41,6 +38,9 @@
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
+#ifndef RFC2046_H
+#	include "rfc2046.h"
+#endif /* !RFC2046_H */
 
 #define isreturn(c) ((c) == '\r' || ((c) == '\n'))
 
@@ -51,7 +51,7 @@
  * in unstructured headers like Subject, Keyword and Summary
  * c.f. RFC 2047
  */
-#define isbetween(c, s) (isspace((unsigned char)c) || ((s) && ((c) == '(' || (c) == ')' || (c) == '"')))
+#define isbetween(c, s) (isspace((unsigned char) c) || ((s) && ((c) == '(' || (c) == ')' || (c) == '"')))
 
 #define NOT_RANKED 255
 
@@ -69,23 +69,39 @@ const char base64_alphabet[64] =
 
 static unsigned char base64_rank[256];
 static int base64_rank_table_built;
-static int quoteflag;
+static t_bool quoteflag;
+
+/* fixed prefix and default part for tin-generated MIME boundaries */
+static const char MIME_BOUNDARY_PREFIX[]       = "=_tin=_";
+static const char MIME_BOUNDARY_DEFAULT_PART[] = "====____====____====____";
+/* required size of a buffer containing a MIME boundary, including the final '\0' */
+enum {
+	MIME_BOUNDARY_SIZE = sizeof(MIME_BOUNDARY_PREFIX) + sizeof(MIME_BOUNDARY_DEFAULT_PART) - 1
+};
 
 /*
  * local prototypes
  */
-static int do_b_encode (char *w, char *b, int max_ewsize, t_bool isstruct_head);
-static int rfc1522_do_encode (char *what, char **where, t_bool break_long_line);
-static int sizeofnextword (char *w);
-static int which_encoding (char *w);
-static t_bool contains_nonprintables (char *w, t_bool isstruct_head);
-static unsigned hex2bin (int x);
-static void build_base64_rank_table (void);
-static void str2b64 (char *from, char *to);
+static FILE *compose_message_rfc822(FILE *articlefp, t_bool *is_8bit);
+static FILE *compose_multipart_mixed(FILE *textfp, FILE *articlefp);
+static int do_b_encode(char *w, char *b, int max_ewsize, t_bool isstruct_head);
+static int sizeofnextword(char *w);
+static int which_encoding(char *w);
+static t_bool contains_8bit_characters(FILE *fp);
+static t_bool contains_nonprintables(char *w, t_bool isstruct_head);
+static t_bool contains_string(FILE *fp, const char *str);
+static t_bool rfc1522_do_encode(char *what, char **where, const char *charset, t_bool break_long_line);
+static t_bool split_mail(const char *filename, FILE **headerfp, FILE **textfp);
+static unsigned hex2bin(int x);
+static void build_base64_rank_table(void);
+static void do_rfc15211522_encode(FILE *f, constext * mime_encoding, struct t_group *group, t_bool allow_8bit_header, t_bool ismail, t_bool contains_headers);
+static void generate_mime_boundary(char *boundary, FILE *f, FILE *g);
+static void generate_random_mime_boundary(char *boundary, size_t len);
+static void str2b64(const char *from, char *to);
 
 
 static void
-build_base64_rank_table (
+build_base64_rank_table(
 	void)
 {
 	int i;
@@ -101,7 +117,7 @@ build_base64_rank_table (
 
 
 static unsigned
-hex2bin (
+hex2bin(
 	int x)
 {
 	if (x >= '0' && x <= '9')
@@ -119,27 +135,23 @@ hex2bin (
  * Return number of bytes decoded into 'where' or -1.
  */
 int
-mmdecode (
+mmdecode(
 	const char *what,
 	int encoding,
 	int delimiter,
-	char *where,
-	const char *charset)
+	char *where)
 {
 	char *t;
-	t_bool decode_gt128 = FALSE;
-
-#if defined(MIME_STRICT_CHARSET) && !defined(CHARSET_CONVERSION)
-	if (charset && !strcasecmp(charset, tinrc.mm_charset))
-#endif /* MIME_STRICT_CHARSET && !CHARSET_CONVERSION*/
-		decode_gt128 = TRUE;
 
 	t = where;
-	encoding = tolower((unsigned char)encoding);
+	encoding = tolower((unsigned char) encoding);
 	if (encoding == 'q') {		/* quoted-printable */
 		int x;
 		unsigned hi, lo;
 
+		if (!what || !where) /* should not happen with 'q'-encoding */
+			return -1;
+
 		while (*what != delimiter) {
 			if (*what != '=') {
 				if (!delimiter || *what != '_')
@@ -164,8 +176,6 @@ mmdecode (
 			if (hi == 255 || lo == 255)
 				return -1;
 			x = (hi << 4) + lo;
-			if (x >= 128 && !decode_gt128)
-				x = '?';
 			*EIGHT_BIT(t)++ = x;
 		}
 		return t - where;
@@ -174,12 +184,14 @@ mmdecode (
 		static int bits = 0;
 		unsigned char x;
 
-		build_base64_rank_table();
 		if (!what || !where) {		/* flush */
 			pattern = 0;
 			bits = 0;
 			return 0;
 		}
+
+		build_base64_rank_table();
+
 		while (*what != delimiter) {
 			x = base64_rank[(int) (*what++)];
 			/* ignore everything not in the alphabet, including '=' */
@@ -190,8 +202,6 @@ mmdecode (
 			bits += 6;
 			if (bits >= 8) {
 				x = (pattern >> (bits - 8)) & 0xff;
-				if (x >= 128 && !decode_gt128)
-					x = '?';
 				*t++ = x;
 				bits -= 8;
 			}
@@ -208,21 +218,38 @@ mmdecode (
  * format
  */
 char *
-rfc1522_decode (
+rfc1522_decode(
 	const char *s)
 {
-	const char *c, *d;
+	char *c, *sc;
+	const char *d;
 	char *t;
-	static char buffer[2048];
+#define BUFFER_LEN 2048
+	static char buffer[BUFFER_LEN];
+	int max_len;
 	char charset[1024];
 	char encoding;
 	t_bool adjacentflag = FALSE;
 
 	charset[0] = '\0';
-	c = s;
+	c = my_strdup(s);
+	max_len = strlen(c) + 1;
 	t = buffer;
 
-	while (*c && t - buffer < 2048) {
+	/*
+	 * remove non-ASCII chars if MIME_STRICT_CHARSET is set
+	 * must be changed if UTF-8 becomes default charset for headers:
+	 *
+	 * process_charsets(c, len, "UTF-8", tinrc.mm_local_charset, FALSE);
+	 */
+#ifndef CHARSET_CONVERSION
+	process_charsets(&c, &max_len, "US-ASCII", tinrc.mm_local_charset, FALSE);
+#else
+	process_charsets(&c, &max_len, (CURR_GROUP.attribute->undeclared_charset) ? (CURR_GROUP.attribute->undeclared_charset) : "US-ASCII", tinrc.mm_local_charset, FALSE);
+#endif /* !CHARSET_CONVERSION */
+	sc = c;
+
+	while (*c && t - buffer < BUFFER_LEN) {
 		if (*c != '=') {
 			if (adjacentflag && isspace((unsigned char) *c)) {
 				const char *dd;
@@ -252,16 +279,29 @@ rfc1522_decode (
 				c++;
 				encoding = tolower((unsigned char)*c);
 				if (encoding == 'b')
-					(void) mmdecode((char *) 0, 'b', 0, (char *) 0, (char *) 0);		/* flush */
+					(void) mmdecode((char *) 0, 'b', 0, (char *) 0);	/* flush */
 				c++;
 				if (*c == '?') {
 					c++;
 					if ((e = strchr(c, '?'))) {
 						int i;
 
-						i = mmdecode(c, encoding, '?', t, charset);
+						i = mmdecode(c, encoding, '?', t);
 						if (i > 0) {
-							t += i;
+							int chars_to_copy;
+							char *tmpbuf;
+
+							max_len = i + 1;
+							tmpbuf = my_malloc(max_len);
+							strncpy(tmpbuf, t, i);
+							*(tmpbuf + i) = '\0';
+							process_charsets(&tmpbuf, &max_len, charset, tinrc.mm_local_charset, FALSE);
+							chars_to_copy = strlen(tmpbuf);
+							if (chars_to_copy > BUFFER_LEN - (t - buffer) - 1)
+								chars_to_copy = BUFFER_LEN - (t - buffer) - 1;
+							strncpy(t, tmpbuf, chars_to_copy);
+							free(tmpbuf);
+							t += chars_to_copy;
 							e++;
 							if (*e == '=')
 								e++;
@@ -272,18 +312,11 @@ rfc1522_decode (
 				}
 			}
 		}
-		while (d != c)
+		while (d != c && t - buffer < BUFFER_LEN - 1)
 			*t++ = *d++;
 	}
 	*t = '\0';
-
-#if defined(LOCAL_CHARSET) || defined(MAC_OS_X)
-	buffer_to_local(buffer);
-#else
-#	ifdef CHARSET_CONVERSION
-	buffer_to_local(buffer, charset, tinrc.mm_local_charset);
-#	endif /* CHARSET_CONVERSION */
-#endif /* LOCAL_CHARSET || MAC_OS_X */
+	free(sc);
 
 	return buffer;
 }
@@ -294,8 +327,8 @@ rfc1522_decode (
  * Woohyung Choi's(whchoi@cosmos.kaist.ac.kr) sdn2ks and ks2sdn
  */
 static void
-str2b64 (
-	char *from,
+str2b64(
+	const char *from,
 	char *to)
 {
 	short int i, count;
@@ -321,7 +354,7 @@ str2b64 (
 
 
 static int
-do_b_encode (
+do_b_encode(
 	char *w,
 	char *b,
 	int max_ewsize,
@@ -330,11 +363,10 @@ do_b_encode (
 	char tmp[60];				/* strings to be B encoded */
 	char *t = tmp;
 	int len8 = 0;				/* the number of trailing 8bit chars, which
-									   should be even(i.e. the first and second byte
-									   of wide_char should NOT be split into two
-									   encoded words) in order to be compatible with
-									   some CJK mail client */
-
+								   should be even (i.e. the first and second byte
+								   of wide_char should NOT be split into two
+								   encoded words) in order to be compatible with
+								   some CJK mail client */
 	int count = max_ewsize / 4 * 3;
 	t_bool isleading_between = TRUE;		/* are we still processing leading space */
 
@@ -365,9 +397,11 @@ do_b_encode (
  * header (i.e. two encoding won't be mixed in a single header line).
  * Mixing two encodings is not a violation of RFC 2047 but may break
  * some news/mail clients.
+ *
+ * mmnwcharset is ignored unless CHARSET_CONVERSION
  */
 static int
-which_encoding (
+which_encoding(
 	char *w)
 {
 	int chars = 0;
@@ -387,18 +421,8 @@ which_encoding (
 		w++;
 	}
 	if (nonprint) {
-		/*
-		 * Always use B encoding regardless of the efficiency if charset is
-		 * EUC-KR for backward compatibility with old Korean mail program
-		 */
 		if (chars + 2 * (nonprint + schars) /* QP size */ >
-			 (chars * 4 + 3) / 3		/* B64 size */
-#ifdef CHARSET_CONVERSION
-			 || !strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], "EUC-KR")
-#else
-			 || !strcasecmp(tinrc.mm_charset, "EUC-KR")
-#endif /* CHARSET_CONVERSION */
-			)
+			 (chars * 4 + 3) / 3		/* B64 size */)
 			return 'B';
 		return 'Q';
 	}
@@ -408,7 +432,7 @@ which_encoding (
 
 /* now only checks if there's any 8bit chars in a given "fragment" */
 static t_bool
-contains_nonprintables (
+contains_nonprintables(
 	char *w,
 	t_bool isstruct_head)
 {
@@ -429,7 +453,7 @@ contains_nonprintables (
 			 * best would be using a regexp like
 			 * ^=\?\S+\?[qQbB]\?\S+\?=
 			 */
-			if (strstr(w, "?=") != (char *) 0)
+			if (strstr(w, "?=") != NULL)
 				nonprint = TRUE;
 		}
 		w++;
@@ -443,7 +467,7 @@ contains_nonprintables (
  * with rfc 2047 (rfc 1522)
  */
 static int
-sizeofnextword (
+sizeofnextword(
 	char *w)
 {
 	char *x;
@@ -457,10 +481,11 @@ sizeofnextword (
 }
 
 
-static int
+static t_bool
 rfc1522_do_encode(
 	char *what,
 	char **where,
+	const char *charset,
 	t_bool break_long_line)
 {
 	/*
@@ -486,6 +511,9 @@ rfc1522_do_encode(
 	 * (just checking that they don get longer than 76 characters),
 	 * then, in a second pass, we replace all SPACEs inside encoded
 	 * words by '_', break long lines, etc.
+	 *
+	 * *group is passed for evaluation of group-attributes,
+	 * set to NULL for e-mails
 	 */
 	char *c;
 	char *t;
@@ -501,7 +529,6 @@ rfc1522_do_encode(
 	t_bool isbroken_within = FALSE;	/* is word broken due to length restriction on encoded of word? */
 	t_bool isstruct_head = FALSE;		/* are we dealing with structured header? */
 	t_bool rightafter_ew = FALSE;
-
 /*
  * the list of structured header fields where '(' and ')' are
  * treated specially in rfc 1522 encoding
@@ -521,11 +548,7 @@ rfc1522_do_encode(
 
 	t = buf;
 	encoding = which_encoding(what);
-#ifdef CHARSET_CONVERSION
-	ew_taken_len = strlen(txt_mime_charsets[tinrc.mm_network_charset]) + 7 /* =?c?E?d?= */;
-#else
-	ew_taken_len = strlen(tinrc.mm_charset) + 7;		/* the minimum encoded word length without any encoded text */
-#endif /* CHARSET_CONVERSION */
+	ew_taken_len = strlen(charset) + 7 /* =?c?E?d?= */;
 	while (*what) {
 		if (break_long_line)
 			word_cnt++;
@@ -537,11 +560,7 @@ rfc1522_do_encode(
 		if (contains_nonprintables(what, isstruct_head) || isbroken_within) {
 			if (encoding == 'Q') {
 				if (!quoting) {
-#ifdef CHARSET_CONVERSION
-					sprintf(buf2, "=?%s?%c?", txt_mime_charsets[tinrc.mm_network_charset], encoding);
-#else
-					sprintf(buf2, "=?%s?%c?", tinrc.mm_charset, encoding);
-#endif /* CHARSET_CONVERSION */
+					snprintf(buf2, sizeof(buf2), "=?%s?%c?", charset, encoding);
 					ewsize = mystrcat(&t, buf2);
 					if (break_long_line) {
 						if (word_cnt == 2) {
@@ -560,12 +579,12 @@ rfc1522_do_encode(
 				isbroken_within = FALSE;
 				while (*what && !isbetween(*what, isstruct_head)) {
 #if 0
-					if (is_EIGHT_BIT(what) || (strchr (RFC2047_ESPECIALS, *what)))
+					if (is_EIGHT_BIT(what) || (strchr(RFC2047_ESPECIALS, *what)))
 #else
-					if (is_EIGHT_BIT(what) || !isalnum((int)*what))
+					if (is_EIGHT_BIT(what) || !isalnum((int)(unsigned char) *what))
 #endif /* 0 */
 					{
-						sprintf(buf2, "=%2.2X", *EIGHT_BIT(what));
+						snprintf(buf2, sizeof(buf2), "=%2.2X", *EIGHT_BIT(what));
 						*t++ = buf2[0];
 						*t++ = buf2[1];
 						*t++ = buf2[2];
@@ -588,12 +607,7 @@ rfc1522_do_encode(
 						break;
 					}
 				}
-#ifdef CHARSET_CONVERSION
-				if (!contains_nonprintables(what, isstruct_head) || ewsize >= 70 - strlen(txt_mime_charsets[tinrc.mm_network_charset]))
-#else
-				if (!contains_nonprintables(what, isstruct_head) || ewsize >= 70 - strlen(tinrc.mm_charset))
-#endif /* CHARSET_CONVERSION */
-				{
+				if (!contains_nonprintables(what, isstruct_head) || ewsize >= 70 - strlen(charset)) {
 					/* next word is 'clean', close encoding */
 					*t++ = '?';
 					*t++ = '=';
@@ -603,12 +617,7 @@ rfc1522_do_encode(
  * after the point where it's split should be encoded (i.e. even if
  * they are made of only 7bit chars)
  */
-#ifdef CHARSET_CONVERSION
-					if (ewsize >= 70 - strlen(txt_mime_charsets[tinrc.mm_network_charset]) && (contains_nonprintables(what, isstruct_head) || isbroken_within))
-#else
-					if (ewsize >= 70 - strlen(tinrc.mm_charset) && (contains_nonprintables(what, isstruct_head) || isbroken_within))
-#endif /* CHARSET_CONVERSION */
-					{
+					if (ewsize >= 70 - strlen(charset) && (contains_nonprintables(what, isstruct_head) || isbroken_within)) {
 						*t++ = ' ';
 						ewsize++;
 					}
@@ -620,28 +629,25 @@ rfc1522_do_encode(
 							*t++ = '_';
 							ewsize++;
 						} else {
-							sprintf(buf2, "=%2.2X", *EIGHT_BIT(what));
+							snprintf(buf2, sizeof(buf2), "=%2.2X", *EIGHT_BIT(what));
 							*t++ = buf2[0];
 							*t++ = buf2[1];
 							*t++ = buf2[2];
 							ewsize += 3;
 						}
 						what++;
-					}					  /* end of while */
-				}						  /* end of else */
-			} else {					  /* end of Q encoding and beg. of B encoding */
+					}					/* end of while */
+				}						/* end of else */
+			} else {					/* end of Q encoding and beg. of B encoding */
 				/*
-				 * if what immediately precedes the current fragment with 8bit char is
-				 * encoded word, the leading spaces should be encoded together with
-				 * 8bit chars following them. No need to worry about '(',')' and '"'
-				 * as they're already excluded with contain_nonprintables used in outer if-clause
+				 * if what immediately precedes the current fragment with 8bit
+				 * char is encoded word, the leading spaces should be encoded
+				 * together with 8bit chars following them. No need to worry
+				 * about '(',')' and '"' as they're already excluded with
+				 * contain_nonprintables used in outer if-clause
 				 */
 				while (*what && (!isbetween(*what, isstruct_head) || rightafter_ew)) {
-#ifdef CHARSET_CONVERSION
-					sprintf(buf2, "=?%s?%c?", txt_mime_charsets[tinrc.mm_network_charset], encoding);
-#else
-					sprintf(buf2, "=?%s?%c?", tinrc.mm_charset, encoding);
-#endif /* CHARSET_CONVERSION */
+					snprintf(buf2, sizeof(buf2), "=?%s?%c?", charset, encoding);
 					ewsize = mystrcat(&t, buf2);
 
 					if (word_cnt == 2)
@@ -658,7 +664,7 @@ rfc1522_do_encode(
 					any_quoting_done = TRUE;
 				}
 				rightafter_ew = TRUE;
-				word_cnt--;			  /* compensate double counting */
+				word_cnt--;		/* compensate double counting */
 				/*
 				 * if encoded word is followed by 7bit-only fragment, we need to
 				 * eliminate ' ' inserted in while-block above
@@ -667,38 +673,36 @@ rfc1522_do_encode(
 					t--;
 					ewsize--;
 				}
-			}							  /* end of B encoding */
+			}		/* end of B encoding */
 		} else {
 			while (*what && !isbetween(*what, isstruct_head))
-				*t++ = *what++;	  /* output word unencoded */
+				*t++ = *what++;		/* output word unencoded */
 			while (*what && isbetween(*what, isstruct_head))
-				*t++ = *what++;	  /* output trailing whitespace unencoded */
+				*t++ = *what++;		/* output trailing whitespace unencoded */
 			rightafter_ew = FALSE;
 		}
-	}									  /* end of pass 1 while loop */
+	}		/* end of pass 1 while loop */
 	*t = 0;
 	/* Pass 2: break long lines if there are MIME-sequences in the result */
 	c = buf;
 	if (break_long_line) {
 		column = 0;
 		if (any_quoting_done) {
-			word_cnt = 1;			  /*
-											* note, if the user has typed a
-										   * continuation line, we will consider the
-										   * initial whitespace to be delimiting
-										   * word one (well, just assume an empty
-										   * word).
-										   */
+			word_cnt = 1;			/*
+									 * note, if the user has typed a continuation
+									 * line, we will consider the initial
+									 * whitespace to be delimiting word one (well,
+									 * just assume an empty word).
+									 */
 			while (*c) {
 				if (isspace((unsigned char) *c)) {
-					/* According to rfc1522, header lines
-					 * containing encoded words are limited to 76
-					 * chars, but if the first line is too long
-					 * (due to a long header keyword), we cannot
-					 * stick to that, since we would break the line
-					 * directly after the keyword's colon, which is
-					 * not allowed. The same is necessary for a
-					 * continuation line with an unencoded word
+					/*
+					 * According to rfc1522, header lines containing encoded
+					 * words are limited to 76 chars, but if the first line is
+					 * too long (due to a long header keyword), we cannot stick
+					 * to that, since we would break the line directly after the
+					 * keyword's colon, which is not allowed. The same is
+					 * necessary for a continuation line with an unencoded word
 					 * that is too long.
 					 */
 					if (sizeofnextword(c) + column > 76 && word_cnt != 1) {
@@ -719,7 +723,7 @@ rfc1522_do_encode(
 			while (*c)
 				*((*where)++) = *c++;
 		}
-	} else {							  /* !break_long_line */
+	} else {		/* !break_long_line */
 		while (*c)
 			*((*where)++) = *c++;
 	}
@@ -729,198 +733,536 @@ rfc1522_do_encode(
 }
 
 
+/*
+ * calling code must free() the result if it's no longer needed
+ */
 char *
-rfc1522_encode (
+rfc1522_encode(
 	char *s,
+	const char *charset,
 	t_bool ismail)
 {
-	char *b;
-	int x;
-	static char buf[2048];
-
+	char *b, *buf;
+	t_bool x;
 /*
- * break_long_line is FALSE for news posting unless MIME_BREAK_LONG_LINES
- * is defined, but it's TRUE for mail messages regardless of whether or not
+ * break_long_line is FALSE for news posting unless MIME_BREAK_LONG_LINES is
+ * defined, but it's TRUE for mail messages regardless of whether or not
  * MIME_BREAK_LONG_LINES is defined
  */
 #ifdef MIME_BREAK_LONG_LINES
 	t_bool break_long_line = TRUE;
 #else
 /*
- * Even if MIME_BREAK_LONG_LINES is NOT defined,
- * long headers in mail messages should be broken up in
- * accordance with RFC 2047(1522)
+ * Even if MIME_BREAK_LONG_LINES is NOT defined, long headers in mail
+ * messages should be broken up in accordance with RFC 2047(1522)
  */
 	t_bool break_long_line = ismail;
 #endif /* MIME_BREAK_LONG_LINES */
 
-	b = buf;
-	x = rfc1522_do_encode(s, &b, break_long_line);
+	b = buf = my_malloc(2048);
+	x = rfc1522_do_encode(s, &b, charset, break_long_line);
 	quoteflag = quoteflag || x;
 	return buf;
 }
 
 
-void
-rfc15211522_encode (
-	const char *filename,
+/*
+ * Helper function doing the hard work for rfc15211522_encode().
+ * Code moved from rfc15211522_encode(), with some adjustments to work on a
+ * file specified by a FILE* instead of a filename.
+ */
+static void
+do_rfc15211522_encode(
+	FILE *f,
 	constext * mime_encoding,
+	struct t_group *group,
 	t_bool allow_8bit_header,
-	t_bool ismail)
+	t_bool ismail,
+	t_bool contains_headers)
 {
-	FILE *f;
 	FILE *g;
 	char *c;
 	char *header;
 	char encoding;
 	char buffer[2048];
-	t_bool body_encoding_needed = FALSE;
-	t_bool umlauts = FALSE;
+	t_bool mime_headers_needed = FALSE;
 	BodyPtr body_encode;
+	int i;
+#ifdef CHARSET_CONVERSION
+	int mmnwcharset = 0;
+
+	if (group) /* Posting */
+		mmnwcharset = group->attribute->mm_network_charset;
+	else /* E-Mail */
+		mmnwcharset = tinrc.mm_network_charset;
+#endif /* CHARSET_CONVERSION */
 
 	if ((g = tmpfile()) == NULL)
 		return;
-	if ((f = fopen(filename, "r")) == NULL) {
-		fclose(g);
-		return;
-	}
 
-	quoteflag = 0;
+	quoteflag = FALSE;
 
-	while ((header = tin_fgets(f, TRUE))) {
-#if defined(LOCAL_CHARSET) || defined(MAC_OS_X) || defined(CHARSET_CONVERSION)
-		buffer_to_network(header);
-#endif /* LOCAL_CHARSET || MAC_OS_X || defined(CHARSET_CONVERSION) */
+	while (contains_headers && (header = tin_fgets(f, TRUE))) {
+#ifdef CHARSET_CONVERSION
+		buffer_to_network(header, mmnwcharset);
+#endif /* CHARSET_CONVERSION */
 		if (*header == '\0')
 			break;
+
 		if (allow_8bit_header)
 			fputs(header, g);
-		else
-			fputs(rfc1522_encode(header, ismail), g);
+		else {
+			char *p;
+
+#ifdef CHARSET_CONVERSION
+			p = rfc1522_encode(header, txt_mime_charsets[mmnwcharset], ismail);
+#else
+			p = rfc1522_encode(header, tinrc.mm_charset, ismail);
+#endif /* CHARSET_CONVERSION */
+
+			fputs(p, g);
+			free(p);
+		}
 		fputc('\n', g);
 	}
 
 	fputc('\n', g);
 
 	while (fgets(buffer, 2048, f)) {
-#if defined(LOCAL_CHARSET) || defined(MAC_OS_X) || defined(CHARSET_CONVERSION)
-		buffer_to_network(buffer);
-#endif /* LOCAL_CHARSET || MAC_OS_X || defined(CHARSET_CONVERSION) */
+#ifdef CHARSET_CONVERSION
+		buffer_to_network(buffer, mmnwcharset);
+#endif /* CHARSET_CONVERSION */
 		fputs(buffer, g);
-		/* see if there are any umlauts in the body... */
-		for (c = buffer; *c && !isreturn(*c); c++)
-			if (is_EIGHT_BIT(c)) {
-				umlauts = TRUE;
-				body_encoding_needed = TRUE;
-				break;
+		if (!allow_8bit_header) {
+			/* see if there are any 8bit chars in the body... */
+			for (c = buffer; *c && !isreturn(*c); c++) {
+				if (is_EIGHT_BIT(c)) {
+					mime_headers_needed = TRUE;
+					break;
+				}
 			}
+		}
 	}
-	fclose(f);
+	/*
+	 * This should work as g should be at least as long as f.
+	 * We could use ftruncate(), but this isn't part of ISO-C.
+	 */
+	rewind(f);
 	rewind(g);
 
-	if ((f = fopen(filename, "w")) == NULL) {
-		fclose(g);
-		return;
-	}
-
+	/* copy header */
 	while (fgets(buffer, 2048, g) && !isreturn(buffer[0]))
 		fputs(buffer, f);
 
-	/* now add MIME headers as necessary */
-#if 0									  /* RFC1522 does not require MIME headers just because there are
-										     encoded header lines */
-	if (quoteflag || umlauts)
-#else
-	if (umlauts)
-#endif /* 0 */
-	{
-		fputs("MIME-Version: 1.0\n", f);
-		if (body_encoding_needed) {
-		/* added for CJK charsets like EUC-KR/JP/CN and others */
+	if (!allow_8bit_header) {
+		/*
+		 * 7bit charsets except US-ASCII also need mime headers
+		 */
+		for (i = 1; *txt_mime_7bit_charsets[i]; i++) {
 #ifdef CHARSET_CONVERSION
-			if (!strncasecmp(txt_mime_charsets[tinrc.mm_network_charset], "EUC-", 4) &&
-				 !strcasecmp(mime_encoding, txt_7bit))
-				fprintf(f, "Content-Type: text/plain; charset=ISO-2022-%s\n", &txt_mime_charsets[tinrc.mm_network_charset][4]);
-			else
-				fprintf(f, "Content-Type: text/plain; charset=%s\n", txt_mime_charsets[tinrc.mm_network_charset]);
+			if (!strcasecmp(txt_mime_charsets[mmnwcharset], txt_mime_7bit_charsets[i])) {
+				mime_headers_needed = TRUE;
+				break;
+			}
 #else
-			if (!strncasecmp(tinrc.mm_charset, "EUC-", 4) &&
-				 !strcasecmp(mime_encoding, txt_7bit))
-				fprintf(f, "Content-Type: text/plain; charset=ISO-2022-%s\n", &tinrc.mm_charset[4]);
-			else
-				fprintf(f, "Content-Type: text/plain; charset=%s\n", tinrc.mm_charset);
+			if (!strcasecmp(tinrc.mm_charset, txt_mime_7bit_charsets[i])) {
+				mime_headers_needed = TRUE;
+				break;
+			}
+#endif /* CHARSET_CONVERSION */
+		}
+
+		/*
+		 * now add MIME headers as necessary
+		 */
+		if (mime_headers_needed) {
+			if (contains_headers)
+				fprintf(f, "MIME-Version: %s\n", MIME_SUPPORTED_VERSION);
+#ifdef CHARSET_CONVERSION
+			fprintf(f, "Content-Type: text/plain; charset=%s\n", txt_mime_charsets[mmnwcharset]);
+#else
+			fprintf(f, "Content-Type: text/plain; charset=%s\n", tinrc.mm_charset);
 #endif /* CHARSET_CONVERSION */
 			fprintf(f, "Content-Transfer-Encoding: %s\n", mime_encoding);
-		} else {
-			fputs("Content-Type: text/plain; charset=US-ASCII\n", f);
-			fputs("Content-Transfer-Encoding: 7bit\n", f);
 		}
 	}
 	fputc('\n', f);
 
-	if (!strcasecmp(mime_encoding, txt_base64))
-		encoding = 'b';
-	else if (!strcasecmp(mime_encoding, txt_quoted_printable))
-		encoding = 'q';
-	else
-		encoding = '8';
-
-	if (!body_encoding_needed)
-		encoding = '8';
-
-/* added for CJK charsets like EUC-KR/JP/CN and others */
-
-	if (!strcasecmp(mime_encoding, txt_7bit)) {
-		encoding = '7';
-
-/* For EUC-KR, 7bit means conversion to ISO-2022-KR specified in RFC 1557 */
-
-#ifdef CHARSET_CONVERSION
-		if (!strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], "EUC-KR"))
-#else
-		if (!strcasecmp(tinrc.mm_charset, "EUC-KR"))
-#endif /* CHARSET_CONVERSION */
-			body_encode = rfc1557_encode;
-
-#if 0
-/*
- * Not only EUC-JP but also other Japanese charsets such as
- * SJIS might need RFC 1468 encoding. To be confirmed.
- */
-#	ifdef CHARSET_CONVERSION
-		else if (!strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], "EUC-JP"))
-			body_encode = rfc1468_encode;
-#	else
-		else if (!strcasecmp(tinrc.mm_charset, "EUC-JP"))
-			body_encode = rfc1468_encode;
-#	endif /* CHARSET_CONVERSION */
-
-/*
- * Not only EUC-CN but also other Chinese charsets such as
- * BIG5 and EUC-TW might need RFC 1922 encoding. To be confirmed.
- */
-#	ifdef CHARSET_CONVERSION
-		else if (!strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], "EUC-CN"))
-			body_encode = rfc1922_encode;
-#	else
-		else if (!strcasecmp(tinrc.mm_charset, "EUC-CN"))
-			body_encode = rfc1922_encode;
-#	endif /* CHARSET_CONVERSION */
-#endif /* 0 */
-
-		else {
-			body_encode = rfc1521_encode;
+	if (!allow_8bit_header) {
+		if (!strcasecmp(mime_encoding, txt_base64))
+			encoding = 'b';
+		else if (!strcasecmp(mime_encoding, txt_quoted_printable))
+			encoding = 'q';
+		else if (!strcasecmp(mime_encoding, txt_7bit))
+			encoding = '7';
+		else
 			encoding = '8';
-		}
-	} else
+
+		/* avoid break of long lines for US-ASCII/quoted-printable */
+		if (!mime_headers_needed)
+			encoding = '8';
+
 		body_encode = rfc1521_encode;
 
-	while (fgets(buffer, 2048, g))
-		body_encode(buffer, f, encoding);
+		while (fgets(buffer, 2048, g))
+			body_encode(buffer, f, encoding);
 
-	if (encoding == 'b' || encoding == 'q' || encoding == '7')
-		body_encode(NULL, f, encoding);	/* flush */
+		if (encoding == 'b' || encoding == 'q' || encoding == '7')
+			body_encode(NULL, f, encoding);	/* flush */
+	} else {
+		while (fgets(buffer, 2048, g))
+			fputs(buffer, f);
+	}
 
 	fclose(g);
-	fclose(f);
+}
+
+
+void
+rfc15211522_encode(
+	const char *filename,
+	constext * mime_encoding,
+	struct t_group *group,
+	t_bool allow_8bit_header,
+	t_bool ismail)
+{
+	FILE *fp;
+
+	if ((fp = fopen(filename, "r+")) == NULL)
+		return;
+
+	do_rfc15211522_encode(fp, mime_encoding, group, allow_8bit_header, ismail, TRUE);
+
+	fclose(fp);
+}
+
+
+/*
+ * Generate a MIME boundary being unique with high probability, consisting
+ * of len - 1 random characters.
+ * This function is used as a last resort if anything else failed to
+ * generate a truly unique boundary.
+ */
+static void
+generate_random_mime_boundary(
+	char *boundary,
+	size_t len)
+{
+	size_t i;
+
+	srand((unsigned int) time(NULL));
+	for (i = 0; i < len - 1; i++)
+		boundary[i] = base64_alphabet[rand() % sizeof(base64_alphabet)];
+	boundary[len - 1] = '\0';
+}
+
+
+/*
+ * Generate a unique MIME boundary.
+ * boundary must have enough space for at least MIME_BOUNDARY_SIZE characters.
+ */
+static void
+generate_mime_boundary(
+	char *boundary,
+	FILE *f,
+	FILE *g)
+{
+	const char nice_chars[] = { '-', '_', '=' };
+	const size_t prefix_len = sizeof(MIME_BOUNDARY_PREFIX) - 1;
+	char *s;
+	size_t i = 0;
+	t_bool unique = FALSE;
+
+	/*
+	 * Choose MIME boundary as follows:
+	 *   - Always start with MIME_BOUNDARY_PREFIX.
+	 *   - Append MIME_BOUNDARY_DEFAULT_PART.
+	 *   - If necessary, change it from right to left, choosing from a set of
+	 *     `nice_chars' characters.
+	 *   - After that, if it is still not unique, replace MIME_BOUNDARY_DEFAULT_PART
+	 *     with random characters and hope the best.
+	 */
+
+	strcpy(boundary, MIME_BOUNDARY_PREFIX);
+	strcat(boundary, MIME_BOUNDARY_DEFAULT_PART);
+
+	s = boundary + MIME_BOUNDARY_SIZE - 2; /* set s to last character before '\0' */
+	do {
+		/*
+		 * Scan for entire boundary in both f and g.
+		 * When found: modify and redo.
+		 */
+		if (contains_string(f, boundary) || contains_string(g, boundary)) {
+			*s = nice_chars[i];
+			if ((i = (i + 1) % sizeof(nice_chars)) == 0)
+				--s;
+		} else
+			unique = TRUE;
+	} while (!unique && s >= boundary + prefix_len);
+
+	if (!unique)
+		generate_random_mime_boundary(boundary + prefix_len, sizeof(MIME_BOUNDARY_DEFAULT_PART));
+}
+
+
+/*
+ * Split mail into header and (optionally) body.
+ *
+ * If textfp is not NULL, everything behind the header is stored in it.
+ * Whenever an error is encountered, all files are closed and FALSE is returned.
+ */
+static t_bool
+split_mail(
+	const char *filename,
+	FILE **headerfp,
+	FILE **textfp)
+{
+	FILE *fp;
+	char *line;
+
+	if ((fp = fopen(filename, "r")) == NULL)
+		return FALSE;
+
+	/* Header */
+	if ((*headerfp = tmpfile()) == NULL) {
+		fclose(fp);
+		return FALSE;
+	}
+
+	while ((line = tin_fgets(fp, TRUE))) {
+		if (*line == '\0')
+			break;
+		else
+			fprintf(*headerfp, "%s\n", line);
+	}
+
+	/* Body */
+	if (textfp != NULL) {
+		if ((*textfp = tmpfile()) == NULL) {
+			fclose(fp);
+			fclose(*headerfp);
+			return FALSE;
+		}
+
+		while ((line = tin_fgets(fp, FALSE)))
+			fprintf(*textfp, "%s\n", line);
+	}
+
+	fclose(fp);
+	return TRUE;
+}
+
+
+/*
+ * Compose a mail consisting of a sole text/plain MIME part.
+ */
+void
+compose_mail_text_plain(
+	const char *filename,
+	struct t_group *group)
+{
+	rfc15211522_encode(filename, txt_mime_encodings[tinrc.mail_mime_encoding], group, tinrc.mail_8bit_header, TRUE);
+}
+
+
+/*
+ * Compose a mail consisting of an optional text/plain and a message/rfc822
+ * part.
+ *
+ * At this point, the file denoted by `filename' contains some common headers
+ * and any text the user entered. The file `articlefp' contains the forwarded
+ * article in raw form.
+ */
+void
+compose_mail_mime_forwarded(
+	const char *filename,
+	FILE *articlefp,
+	t_bool include_text,
+	struct t_group *group)
+{
+	FILE *fp;
+	FILE *headerfp;
+	FILE *textfp = NULL;
+	FILE *entityfp;
+	char *line;
+	constext* encoding = txt_mime_encodings[tinrc.mail_mime_encoding];
+	t_bool allow_8bit_header = tinrc.mail_8bit_header;
+	t_bool _8bit;
+
+	/* Split mail into headers and text */
+	if (!split_mail(filename, &headerfp, include_text ? &textfp : NULL))
+		return;
+
+	/* Encode header and text */
+	rewind(headerfp);
+	do_rfc15211522_encode(headerfp, encoding, group, allow_8bit_header, TRUE, TRUE);
+
+	if (textfp) {
+		rewind(textfp);
+		do_rfc15211522_encode(textfp, encoding, group, allow_8bit_header, TRUE, FALSE);
+	}
+
+	/* Compose top-level MIME entity */
+	if (include_text)
+		entityfp = compose_multipart_mixed(textfp, articlefp);
+	else
+		entityfp = compose_message_rfc822(articlefp, &_8bit);
+
+	if (entityfp == NULL) {
+		fclose(headerfp);
+		if (textfp)
+			fclose(textfp);
+		return;
+	}
+
+	/* Put it all together */
+	if ((fp = fopen(filename, "w")) == NULL)
+		return;
+
+	rewind(headerfp);
+	while ((line = tin_fgets(headerfp, TRUE))) {
+		if (*line != '\0')
+			fprintf(fp, "%s\n", line);
+	}
+	fprintf(fp, "MIME-Version: %s\n", MIME_SUPPORTED_VERSION);
+	rewind(entityfp);
+	copy_fp(entityfp, fp);
+
+	/* Clean up */
+	fclose(fp);
+	fclose(headerfp);
+	fclose(entityfp);
+	if (textfp)
+		fclose(textfp);
+}
+
+
+/*
+ * Compose a message/rfc822 MIME entity containing articlefp.
+ */
+static FILE *
+compose_message_rfc822(
+	FILE *articlefp,
+	t_bool *is_8bit)
+{
+	FILE *fp;
+
+	if ((fp = tmpfile()) == NULL)
+		return NULL;
+
+	*is_8bit = contains_8bit_characters(articlefp);
+
+	/* Header: CT, CD, CTE */
+	fprintf(fp, "Content-Type: message/rfc822\n");
+	fprintf(fp, "Content-Disposition: inline\n");
+	fprintf(fp, "Content-Transfer-Encoding: %s\n", *is_8bit ? txt_8bit : txt_7bit);
+	fputc('\n', fp);
+
+	/* Body: articlefp */
+	rewind(articlefp);
+	copy_fp(articlefp, fp);
+
+	return fp;
+}
+
+
+/*
+ * Compose a multipart/mixed MIME entity consisting of a text/plain and a
+ * message/rfc822 part.
+ */
+static FILE *
+compose_multipart_mixed(
+	FILE *textfp,
+	FILE *articlefp)
+{
+	FILE *fp;
+	FILE *messagefp;
+	char boundary[MIME_BOUNDARY_SIZE];
+	t_bool requires_8bit;
+
+	if ((fp = tmpfile()) == NULL)
+		return NULL;
+
+	/* First compose message/rfc822 part (needed for choosing the appropriate CTE) */
+	if ((messagefp = compose_message_rfc822(articlefp, &requires_8bit)) == NULL) {
+		fclose(fp);
+		return NULL;
+	}
+
+	requires_8bit = (requires_8bit || contains_8bit_characters(textfp));
+
+	/* Header: CT with multipart boundary, CTE */
+	generate_mime_boundary(boundary, textfp, articlefp);
+	fprintf(fp, "Content-Type: multipart/mixed; boundary=\"%s\"\n", boundary);
+	fprintf(fp, "Content-Transfer-Encoding: %s\n\n", requires_8bit ? txt_8bit : txt_7bit);
+
+	/*
+	 * preamble
+	 * TODO: -> lang.c
+	 */
+	fprintf(fp, _("This message has been composed in the 'multipart/mixed' MIME-format. If you\n\
+are reading this prefix, your mail reader probably has not yet been modified\n\
+to understand the new format, and some of what follows may look strange.\n\n"));
+
+	/*
+	 * Body: boundary+text, message/rfc822 part, closing boundary
+	 */
+	/* text */
+	fprintf(fp, "--%s\n", boundary);
+	rewind(textfp);
+	copy_fp(textfp, fp);
+	fputc('\n', fp);
+
+	/* message/rfc822 part */
+	fprintf(fp, "--%s\n", boundary);
+	rewind(messagefp);
+	copy_fp(messagefp, fp);
+	fputc('\n', fp);
+
+	/* closing boundary */
+	fprintf(fp, "--%s--\n", boundary);
+	/* TODO: insert an epilogue here? */
+
+	return fp;
+}
+
+
+/*
+ * Determines whether the file denoted by fp contains 8bit characters.
+ */
+static t_bool
+contains_8bit_characters(
+	FILE *fp)
+{
+	char *line;
+
+	rewind(fp);
+	while ((line = tin_fgets(fp, FALSE))) {
+		for ( ; *line != '\0'; line++) {
+			if (is_EIGHT_BIT(line))
+				return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+
+/*
+ * Determines whether any line of the file denoted by fp contains str.
+ */
+static t_bool
+contains_string(
+	FILE *fp,
+	const char *str)
+{
+	char *line;
+
+	rewind(fp);
+	while ((line = tin_fgets(fp, FALSE))) {
+		if (strstr(line, str))
+			return TRUE;
+	}
+
+	return FALSE;
 }
