@@ -3,10 +3,10 @@
  *  Module    : screen.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 1997-12-31
+ *  Updated   : 2004-06-07
  *  Notes     :
  *
- * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2004 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Iain Lea, Rich Skrenta.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -45,43 +42,77 @@
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
 
-char mesg[LEN];
-
 #ifndef USE_CURSES
 	struct t_screen *screen;
 #endif /* !USE_CURSES */
 
+/*
+ * Local prototypes
+ */
+static char *fmt_message(const char *fmt, va_list ap);
+
+
 /*
  * Move the cursor to the lower-left of the screen, where it won't be annoying
  */
 void
-stow_cursor (
+stow_cursor(
 	void)
 {
 	if (!cmd_line)
-		MoveCursor (cLINES, 0);
+		MoveCursor(cLINES, 0);
 }
 
+
+/*
+ * helper for the varius *_message() functions
+ * returns a pointer to an allocated buffer with the formated message
+ * must be freed if not needed anymore
+ */
+static char *
+fmt_message(
+	const char *fmt,
+	va_list ap)
+{
+	char *msg;
+#ifdef HAVE_VASPRINTF
+	int n;
+
+	if ((n = vasprintf(&msg, fmt, ap)) == -1)	/* something went wrong */
+#endif /* HAVE_VASPRINTF */
+	{
+		size_t size = LEN;
+
+		msg = my_malloc(size);
+		/* TODO: realloc msg if necessary */
+		vsnprintf(msg, size, fmt, ap);
+	}
+
+	return msg;
+}
+
+
 /*
  * Centre a formatted colour message at the bottom of the screen
  */
 void
-info_message (
+info_message(
 	const char *fmt,
 	...)
 {
+	char *buf;
 	va_list ap;
 
 	va_start(ap, fmt);
 
-	clear_message ();
+	clear_message();
 #ifdef HAVE_COLOR
 	fcol(tinrc.col_message);
 #endif /* HAVE_COLOR */
 
-	vsprintf (mesg, fmt, ap);
-
-	center_line (cLINES, FALSE, mesg);	/* center the message at screen bottom */
+	buf = fmt_message(fmt, ap);
+	center_line(cLINES, FALSE, buf);	/* center the message at screen bottom */
+	free(buf);
 
 #ifdef HAVE_COLOR
 	fcol(tinrc.col_normal);
@@ -96,27 +127,29 @@ info_message (
  * Print a formatted colour message at the bottom of the screen, wait a while
  */
 void
-wait_message (
-	int sdelay,
+wait_message(
+	unsigned int sdelay,
 	const char *fmt,
 	...)
 {
+	char *buf;
 	va_list ap;
 
 	va_start(ap, fmt);
 
-	clear_message ();
+	clear_message();
 #ifdef HAVE_COLOR
 	fcol(tinrc.col_message);
 #endif /* HAVE_COLOR */
 
-	vsprintf (mesg, fmt, ap);
-	my_fputs (mesg, stdout);
+	buf = fmt_message(fmt, ap);
+	my_fputs(buf, stdout);
+	free(buf);
 
 #ifdef HAVE_COLOR
 	fcol(tinrc.col_normal);
 #endif /* HAVE_COLOR */
-	cursoron ();
+	cursoron();
 	my_flush();
 
 	(void) sleep(sdelay);
@@ -130,30 +163,29 @@ wait_message (
  * Interesting - this function implicitly clears 'errno'
  */
 void
-error_message (
+error_message(
 	const char *fmt,
 	...)
 {
+	char *buf;
 	va_list ap;
 
 	va_start(ap, fmt);
 
 	errno = 0;
+	clear_message();
 
-	clear_message ();
-
-	vsprintf (mesg, fmt, ap);
-
-/*	my_fprintf (stderr, mesg); */
-	my_fputs (mesg, stderr);
-	my_fflush (stderr);
+	buf = fmt_message(fmt, ap);
+	my_fputs(buf, stderr);	/* don't use my_fprintf() here due to %format chars */
+	my_fflush(stderr);
+	free(buf);
 
 	if (cmd_line) {
-		my_fputc ('\n', stderr);
-		fflush (stderr);
+		my_fputc('\n', stderr);
+		fflush(stderr);
 	} else {
 		stow_cursor();
-		(void) sleep (2);
+		(void) sleep(2);
 		clear_message();
 	}
 
@@ -166,37 +198,38 @@ error_message (
  * This function implicitly clears 'errno'
  */
 void
-perror_message (
+perror_message(
 	const char *fmt,
 	...)
 {
-	char buf[LEN];
+	char *buf;
 	int err;
 	va_list ap;
 
 	err = errno;
 	va_start(ap, fmt);
 
-	clear_message ();
+	clear_message();
 
-	vsnprintf (buf, sizeof(LEN) - 1, fmt, ap);
+	if ((buf = fmt_message(fmt, ap)) != NULL) {
+		error_message("%s: Error: %s", buf, strerror(err));
+		free(buf);
+	}
 
 	va_end(ap);
 
-	error_message("%s: Error: %s", buf, strerror(err));
-
 	return;
 }
 
 
 void
-clear_message (
+clear_message(
 	void)
 {
 	if (!cmd_line) {
-		MoveCursor (cLINES, 0);
-		CleartoEOLN ();
-		cursoroff ();
+		MoveCursor(cLINES, 0);
+		CleartoEOLN();
+		cursoroff();
 #ifndef USE_CURSES
 		my_flush();
 #endif /* !USE_CURSES */
@@ -205,56 +238,72 @@ clear_message (
 
 
 void
-center_line (
+center_line(
 	int line,
 	t_bool inverse,
 	const char *str)
 {
-	int pos;
 	char buffer[256];
+	int pos;
+	int len;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	int width;
+	wchar_t *wbuffer;
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	STRCPY(buffer, str);
 
+	/* protect terminal... */
+	convert_to_printable(buffer);
+
+	len = strlen(buffer);
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	if ((wbuffer = char2wchar_t(buffer)) != NULL) {
+		if ((width = wcswidth(wbuffer, wcslen(wbuffer) + 1)) > 0)
+			len = width;
+		free(wbuffer);
+	}
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+
 	if (!cmd_line) {
-		if (cCOLS >= (int) strlen (str))
-			pos = (cCOLS - (int) strlen (str)) / 2;
+		if (cCOLS >= len)
+			pos = (cCOLS - len) / 2;
 		else
 			pos = 1;
 
-		MoveCursor (line, pos);
+		MoveCursor(line, pos);
 		if (inverse) {
-			StartInverse ();
+			StartInverse();
 			my_flush();
 		}
 	}
 
-	/* protect terminal... */
-	convert_to_printable (buffer);
+	if (len >= cCOLS) {
+		char *buf;
 
-	if ((int) strlen (buffer) >= cCOLS) {
-		char buf[256];
-		sprintf(buf, "%-.*s%s", cCOLS-6, buffer, " ...");
-		my_fputs (buf, stdout);
-	} else
-		my_fputs (buffer, stdout);
+		buf = my_strdup(buffer);
+		strunc(buf, buffer, sizeof(buffer), cCOLS - 2);
+		free(buf);
+	}
+	my_fputs(buffer, stdout);
 
 	if (cmd_line)
 		my_flush();
 	else {
 		if (inverse)
-			EndInverse ();
+			EndInverse();
 	}
 }
 
 
 void
-draw_arrow_mark (
+draw_arrow_mark(
 	int line)
 {
-	MoveCursor (line, 0);
+	MoveCursor(line, 0);
 
 	if (tinrc.draw_arrow)
-		my_fputs ("->", stdout);
+		my_fputs("->", stdout);
 	else {
 #ifdef USE_CURSES
 		char buffer[BUFSIZ];
@@ -262,13 +311,13 @@ draw_arrow_mark (
 #else
 		char *s = screen[line - INDEX_TOP].col;
 #endif /* USE_CURSES */
-		StartInverse ();
-		my_fputs (s, stdout);
-		EndInverse ();
+		StartInverse();
+		my_fputs(s, stdout);
+		EndInverse();
 		if (s[MARK_OFFSET] == tinrc.art_marked_selected) {
-			MoveCursor (line, MARK_OFFSET);
-			EndInverse ();
-			my_fputc (s[MARK_OFFSET], stdout);
+			MoveCursor(line, MARK_OFFSET);
+			EndInverse();
+			my_fputc(s[MARK_OFFSET], stdout);
 		}
 	}
 	stow_cursor();
@@ -276,7 +325,7 @@ draw_arrow_mark (
 
 
 void
-erase_arrow (
+erase_arrow(
 	void)
 {
 	int line = INDEX_TOP + currmenu->curr - currmenu->first;
@@ -284,10 +333,10 @@ erase_arrow (
 	if (!currmenu->max)
 		return;
 
-	MoveCursor (line, 0);
+	MoveCursor(line, 0);
 
 	if (tinrc.draw_arrow)
-		my_fputs ("  ", stdout);
+		my_fputs("  ", stdout);
 	else {
 #ifdef USE_CURSES
 		char buffer[BUFSIZ];
@@ -300,43 +349,43 @@ erase_arrow (
 
 		s = screen[line - INDEX_TOP].col;
 #endif /* USE_CURSES */
-		EndInverse ();
-		my_fputs (s, stdout);
+		EndInverse();
+		my_fputs(s, stdout);
 		if (s[MARK_OFFSET] == tinrc.art_marked_selected) {
-			MoveCursor (line, MARK_OFFSET);
-			StartInverse ();
-			my_fputc (s[MARK_OFFSET], stdout);
-			EndInverse ();
+			MoveCursor(line, MARK_OFFSET);
+			StartInverse();
+			my_fputc(s[MARK_OFFSET], stdout);
+			EndInverse();
 		}
 	}
 }
 
 
 void
-show_title (
-	char *title)
+show_title(
+	const char *title)
 {
 	int col;
 
-	col = (cCOLS - (int) strlen (_(txt_type_h_for_help))) + 1;
+	col = (cCOLS - (int) strlen(_(txt_type_h_for_help))) + 1;
 	if (col) {
-		MoveCursor (0, col);
+		MoveCursor(0, col);
 #ifdef HAVE_COLOR
 		fcol(tinrc.col_title);
 #endif /* HAVE_COLOR */
 		/* you have mail message in */
-		my_fputs ((mail_check () ? _(txt_you_have_mail) : _(txt_type_h_for_help)), stdout);
+		my_fputs((mail_check() ? _(txt_you_have_mail) : _(txt_type_h_for_help)), stdout);
 
 #ifdef HAVE_COLOR
 		fcol(tinrc.col_normal);
 #endif /* HAVE_COLOR */
 	}
-	center_line (0, TRUE, title);
+	center_line(0, TRUE, title); /* wastes some space on the left */
 }
 
 
 void
-ring_bell (
+ring_bell(
 	void)
 {
 #ifdef USE_CURSES
@@ -344,7 +393,7 @@ ring_bell (
 		beep();
 	else {
 #endif /* USE_CURSES */
-	my_fputc ('\007', stdout);
+	my_fputc('\007', stdout);
 	my_flush();
 #ifdef USE_CURSES
 	}
@@ -353,10 +402,10 @@ ring_bell (
 
 
 void
-spin_cursor (
+spin_cursor(
 	void)
 {
-	static const char buf[] = "|/-\\|/-\\ "; /* don't remove the taling sapce! */
+	static const char buf[] = "|/-\\|/-\\ "; /* don't remove the tailing space! */
 	static unsigned short int i = 0;
 
 	if (batch_mode)
@@ -368,7 +417,7 @@ spin_cursor (
 #ifdef HAVE_COLOR
 	fcol(tinrc.col_message);
 #endif /* HAVE_COLOR */
-	my_printf ("\b%c", buf[i++]);
+	my_printf("\b%c", buf[i++]);
 	my_flush();
 #ifdef HAVE_COLOR
 	fcol(tinrc.col_normal);
@@ -380,46 +429,59 @@ spin_cursor (
  * progressmeter in %
  */
 void
-show_progress (
+show_progress(
 	const char *txt,
 	long count,
 	long total)
 {
-#ifdef HAVE_GETTIMEOFDAY
+	char display[LEN];
+	int ratio;
+	time_t curr_time;
+	static char last_display[LEN];
 	static const char *last_txt;
 	static int last_length;
-	static int last_total;
-	static int last_count;
+	static int last_ratio;
+	static long last_total;
+	static time_t last_update;
+#ifdef HAVE_GETTIMEOFDAY
+	static long last_count;
 	static int average;
 	static int samples;
 	static int sum;
 	static struct timeval last_time;
 	static struct timeval this_time;
 	int time_diff;
-	int count_diff;
 	int secs_left;
+	long count_diff;
+#endif /* HAVE_GETTIMEOFDAY */
 
 	if (batch_mode || count <= 0 || total == 0)
 		return;
 
 	/* If this is a new progress meter, start recalculating */
-	if ((last_txt != txt) || (last_total != total))
+	if ((last_txt != txt) || (last_total != total)) {
 		last_length = 0;
-
-	MoveCursor(cLINES, 0);
-
-	/* Erase the previous text */
-	if (last_length) {
-		my_printf("%*s", last_length, " ");
-		MoveCursor(cLINES, 0);
+		last_ratio = -1;
+		last_display[0] = '\0';
+		last_update = time(NULL) - 2;
 	}
 
-	if (!last_length) {
-#	ifdef HAVE_COLOR
-		fcol(tinrc.col_message);
-#	endif /* HAVE_COLOR */
-		/* Don't print a time left this time */
-		/* last_length = */ my_printf ("%s %3d%%", txt, (int)(count * 100 / total));
+	curr_time = time(NULL);
+	ratio = (int) ((count * 100) / total);
+	if ((ratio == last_ratio) && (curr_time - last_update < 2))
+		/*
+		 * return if ratio did not change and less than 1-2 seconds since last
+		 * update to reduce output
+		 */
+		return;
+
+	last_update = curr_time;
+
+#ifdef HAVE_GETTIMEOFDAY
+	if (last_length == 0) {
+		/* Don't print a "time remaining" this time */
+		snprintf(display, sizeof(display), "%s %3d%%", txt, ratio);
+		display[sizeof(display) - 1] = '\0';
 		last_length = strlen(txt) + 5;
 
 		/* Reset the variables */
@@ -435,6 +497,16 @@ show_progress (
 		if (!count_diff) /* avoid div by zero */
 			count_diff++;
 
+		/*
+		 * Calculate a running average based on the last 20 samples. For the
+		 * first 19 samples just add all and divide by the number of samples.
+		 * From the 20th sample on use only the last 20 samples to calculate
+		 * the running averave. To make things easier we don't want to store
+		 * and keep track of all of them, so we assume that the first sample
+		 * was close to the current average and substract it from sum. Then,
+		 * the new sample is added to the sum and the sum is divided by 20 to
+		 * get the new average.
+		 */
 		if (samples == 20) {
 			sum -= average;
 			sum += (time_diff / count_diff);
@@ -452,39 +524,38 @@ show_progress (
 		if (secs_left < 0)
 			secs_left = 0;
 
-#	ifdef HAVE_COLOR
-		fcol(tinrc.col_message);
-#	endif /* HAVE_COLOR */
 		/* TODO: -> lang.c, difficult with hardcoded last_length */
-		/* last_length = */ my_printf ("%s %3d%% (%d:%02d remaining)", txt, (int)(count * 100 / total), secs_left / 60, secs_left % 60);
+		snprintf(display, sizeof(display), "%s %3d%% (%d:%02d remaining)", txt, ratio, secs_left / 60, secs_left % 60);
 		last_length = strlen(txt) + 21 + secs_left / 600;
 	}
 
-	my_flush();
-#	ifdef HAVE_COLOR
-	fcol(tinrc.col_normal);
-#	endif /* HAVE_COLOR */
-
-	last_txt = txt;
-	last_total = total;
 	last_count = count;
 	gettimeofday(&last_time, NULL);
 
-#else
-	if (batch_mode || count <= 0 || total == 0)
-		return;
-
-	MoveCursor(cLINES, 0);
-
-#	ifdef HAVE_COLOR
-	fcol(tinrc.col_message);
-#	endif /* HAVE_COLOR */
-
-	my_printf ("%s %3d%%", txt, (int) (count * 100 / total));
-	my_flush();
-
-#	ifdef HAVE_COLOR
-	fcol(tinrc.col_normal);
-#	endif /* HAVE_COLOR */
+#else /* HAVE_GETTIMEOFDAY */
+	snprintf(display, sizeof(display), "%s %3d%%", txt, ratio);
 #endif /* HAVE_GETTIMEOFDAY */
+
+	/* Only display text if it changed from last time */
+	if (strcmp(display, last_display)) {
+		clear_message();
+		MoveCursor(cLINES, 0);
+
+#	ifdef HAVE_COLOR
+		fcol(tinrc.col_message);
+#	endif /* HAVE_COLOR */
+
+		my_printf("%s", display);
+
+#	ifdef HAVE_COLOR
+		fcol(tinrc.col_normal);
+#	endif /* HAVE_COLOR */
+
+		my_flush();
+		STRCPY(last_display, display);
+	}
+
+	last_txt = txt;
+	last_total = total;
+	last_ratio = ratio;
 }
