@@ -1,13 +1,13 @@
 /*
  *  Project   : tin - a Usenet reader
  *  Module    : tcurses.c
- *  Author    : Thomas Dickey <dickey@herndon4.his.com>
+ *  Author    : Thomas Dickey <dickey@invisible-island.net>
  *  Created   : 1997-03-02
- *  Updated   : 2001-07-22
+ *  Updated   : 2004-06-07
  *  Notes     : This is a set of wrapper functions adapting the termcap
  *	             interface of tin to use SVr4 curses (e.g., ncurses).
  *
- * Copyright (c) 1997-2002 Thomas Dickey <dickey@herndon4.his.com>
+ * Copyright (c) 1997-2004 Thomas Dickey <dickey@invisible-island.net>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -18,10 +18,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Thomas Dickey.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -52,11 +49,17 @@
 #		define KEY_MIN KEY_BREAK	/* SVr3 curses */
 #	endif /* !KEY_MIN */
 
+#	ifndef KEY_CODE_YES
+#		define KEY_CODE_YES (KEY_MIN - 1)	/* PDCurses */
+#	endif /* !KEY_CODE_YES */
+
 #	include "trace.h"
 
 int cLINES;
 int cCOLS;
 
+static int my_innstr(char *str, int n);
+
 
 #	ifdef HAVE_XCURSES
 static int
@@ -69,7 +72,7 @@ vwprintw(
 	char *string = buffer;
 	int y, x, code;
 
-	vsprintf(buffer, fmt, ap);
+	vsnprintf(buffer, sizeof(buffer), fmt, ap);
 	getyx(w, y, x);
 	TRACE(("vwprintw[%d/%d,%d/%d]:%s", y, cLINES, x, cCOLS, buffer));
 	while (*string == '\b')
@@ -88,7 +91,7 @@ vwprintw(
  * Most of the logic corresponding to the termcap version is done in InitScreen.
  */
 void
-setup_screen (
+setup_screen(
 	void)
 {
 	cmd_line = FALSE;
@@ -96,14 +99,14 @@ setup_screen (
 	bcol(tinrc.col_back);
 #	endif /* HAVE_COLOR */
 	scrollok(stdscr, TRUE);
-	set_win_size (&cLINES, &cCOLS);
+	set_win_size(&cLINES, &cCOLS);
 }
 
 
 /*
  */
 int
-InitScreen (
+InitScreen(
 	void)
 {
 #	ifdef NCURSES_VERSION
@@ -120,7 +123,7 @@ InitScreen (
 	cCOLS = COLS;
 	cLINES = LINES - 1;
 	TRACE(("screen size %d rows by %d cols", LINES, COLS));
-/*	set_win_size(&cLINES, &cCOLS); */
+/*	set_win_size(&cLINES, &cCOLS); */ /* will be done in setup_screen() */
 /*	raw(); */ /* breaks serial terminal using software flow control and cbreak() below does most of the stuff raw() does */
 	noecho();
 	cbreak();
@@ -143,7 +146,7 @@ InitScreen (
 	postinit_colors();
 #	endif /* HAVE_COLOR */
 	set_xclick_on();
-	return (TRUE);
+	return TRUE;
 }
 
 
@@ -166,7 +169,7 @@ void
 EndWin(
 	void)
 {
-	TRACE(("EndWin (%d)", cmd_line));
+	TRACE(("EndWin(%d)", cmd_line));
 	if (!cmd_line) {
 		Raw(FALSE);
 		endwin();
@@ -225,23 +228,26 @@ StartInverse(
 }
 
 
+#	if 0 /* not used */
 static int
 isInverse(
 	void)
 {
-#	ifdef HAVE_COLOR
+#		ifdef HAVE_COLOR
 	if (use_color) {
 		short pair = PAIR_NUMBER(getattrs(stdscr));
 		short fg, bg;
 		pair_content(pair, &fg, &bg);
 		return (fg == tinrc.col_invers_fg) && (bg == tinrc.col_invers_bg);
 	}
-#	endif /* HAVE_COLOR */
+#		endif /* HAVE_COLOR */
 
 	return (getattrs(stdscr) & A_REVERSE);
 }
+#	endif /* 0 */
 
 
+#	if 0 /* doesn't work correct with ncurses4.x */
 /*
  */
 void
@@ -253,6 +259,7 @@ ToggleInverse(
 	else
 		StartInverse();
 }
+#	endif /* 0 */
 
 
 /*
@@ -274,7 +281,7 @@ EndInverse(
 /*
  */
 void
-cursoron (
+cursoron(
 	void)
 {
 	if (!cmd_line)
@@ -285,7 +292,7 @@ cursoron (
 /*
  */
 void
-cursoroff (
+cursoroff(
 	void)
 {
 	if (!cmd_line)
@@ -296,7 +303,7 @@ cursoroff (
 /*
  */
 void
-set_keypad_on (
+set_keypad_on(
 	void)
 {
 	if (!cmd_line)
@@ -307,7 +314,7 @@ set_keypad_on (
 /*
  */
 void
-set_keypad_off (
+set_keypad_off(
 	void)
 {
 	if (!cmd_line)
@@ -320,14 +327,14 @@ set_keypad_off (
  * as well as when we enable/disable the mousemask.
  */
 void
-set_xclick_on (
+set_xclick_on(
 	void)
 {
 #	ifdef NCURSES_MOUSE_VERSION
 	if (tinrc.use_mouse)
 		mousemask(
 			(BUTTON1_CLICKED|BUTTON2_CLICKED|BUTTON3_CLICKED),
-			(mmask_t *)0);
+			(mmask_t *) 0);
 #	endif /* NCURSES_MOUSE_VERSION */
 }
 
@@ -335,11 +342,11 @@ set_xclick_on (
 /*
  */
 void
-set_xclick_off (
+set_xclick_off(
 	void)
 {
 #	ifdef NCURSES_MOUSE_VERSION
-	(void) mousemask(0, (mmask_t *)0);
+	(void) mousemask(0, (mmask_t *) 0);
 #	endif /* NCURSES_MOUSE_VERSION */
 }
 
@@ -359,19 +366,139 @@ MoveCursor(
  * Inverse 'size' chars at (row,col)
  */
 void
-highlight_string (
+highlight_string(
 	int row,
 	int col,
 	int size)
 {
 	char tmp[LEN];
 
-	MoveCursor (row, col);
-	innstr (tmp, size);
-	StartInverse ();
-	my_fputs (tmp, stdout);
-	my_flush ();
-	EndInverse ();
+#	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	/*
+	 * In a multibyte locale we get byte offsets instead of character
+	 * offsets calculate now the correct starting column
+	 */
+	if (col > 0) {
+		wchar_t *wtmp;
+
+		MoveCursor(row, 0);
+		my_innstr(tmp, cCOLS);
+		tmp[col] = '\0';
+		if ((wtmp = char2wchar_t(tmp)) != NULL) {
+			col = wcswidth(wtmp, wcslen(wtmp) + 1);
+			free(wtmp);
+		}
+	}
+#	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+
+	MoveCursor(row, col);
+	my_innstr(tmp, size);
+	tmp[size] = '\0';
+	StartInverse();
+	my_fputs(tmp, stdout);
+	my_flush();
+	EndInverse();
+	stow_cursor();
+}
+
+
+/*
+ * Color 'size' chars at (row,col) with 'color' and handle marks
+ */
+void
+word_highlight_string(
+	int row,
+	int col,
+	int size,
+	int color)
+{
+	/*
+	 * Mapping of the tinrc.mono_mark* values to the ncurses attributes
+	 */
+	int attributes[] = {
+		A_NORMAL,
+		A_STANDOUT,
+		A_UNDERLINE,
+		A_REVERSE,
+		A_BLINK,
+		A_DIM,
+		A_BOLD
+	};
+	char tmp[LEN];
+	int wsize = size;
+#		if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	wchar_t *wtmp;
+
+	/*
+	 * In a multibyte locale we get byte offsets instead of character offsets
+	 * calculate now the correct correct starting column
+	 */
+	if (col > 0) {
+		MoveCursor(row, 0);
+		my_innstr(tmp, cCOLS);
+		tmp[col] = '\0';
+		if ((wtmp = char2wchar_t(tmp)) != NULL) {
+			col = wcswidth(wtmp, wcslen(wtmp) + 1);
+			free(wtmp);
+		}
+	}
+#		endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+
+	MoveCursor(row, col);
+	my_innstr(tmp, size);
+
+#		if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	tmp[size] = '\0';
+	if ((wtmp = char2wchar_t(tmp)) != NULL) {
+		wsize = wcswidth(wtmp, wcslen(wtmp) + 1);
+		free(wtmp);
+	}
+#		endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+
+	/* safegurad against bogus regexps */
+	if ((tmp[0] == '*' && tmp[size - 1] == '*') ||
+		 (tmp[0] == '/' && tmp[size - 1] == '/') ||
+		 (tmp[0] == '_' && tmp[size - 1] == '_') ||
+		 (tmp[0] == '-' && tmp[size - 1] == '-')) {
+
+		switch (tinrc.word_h_display_marks) {
+			case 0:
+				delch();
+				mvdelch(row, col + wsize - 2);
+				MoveCursor(row, col);
+				tmp[0] = tmp[size - 1] = ' ';
+				str_trim(tmp);
+				break;
+
+			case 2: /* print space */
+				MoveCursor(row, col + wsize - 1);
+				my_fputs(" ", stdout);
+				MoveCursor(row, col);
+				my_fputs(" ", stdout);
+				tmp[0] = tmp[size - 1] = ' ';
+				str_trim(tmp);
+				break;
+
+			default: /* print mark (case 1) */
+				break;
+		}
+	}
+#	ifdef HAVE_COLOR
+	if (use_color)
+		fcol(color);
+	else
+#	endif /* HAVE_COLOR */
+		if (color > 0 && color <= MAX_ATTR)
+			attron(attributes[color]);
+	my_fputs(tmp, stdout);
+	my_flush();
+#	ifdef HAVE_COLOR
+	if (use_color)
+		fcol(tinrc.col_text);
+	else
+#	endif /* HAVE_COLOR */
+		if (color > 0 && color <= MAX_ATTR)
+			attroff(attributes[color]);
 	stow_cursor();
 }
 
@@ -385,21 +512,31 @@ ReadCh(
 	if (cmd_line)
 		ch = cmdReadCh();
 	else {
-#	if 0
 again:
-#	endif /* 0 */
-		allow_resize (TRUE);
+		allow_resize(TRUE);
+#	if defined(KEY_RESIZE) && defined(USE_CURSES)
+		if ((ch = getch()) == KEY_RESIZE)
+			need_resize = cYes;
+#		if 0
+		/*
+		 * disable checking for ERR until all callers of ReadCh() doesn't
+		 * depend on ERR for redrawing
+		 */
+		if (ch == ERR)
+			goto again;
+#		endif /* 0 */
+#	else
 		ch = getch();
-		allow_resize (FALSE);
+#	endif /* KEY_RESIZE && USE_CURSES */
+		allow_resize(FALSE);
 		if (need_resize) {
-			handle_resize ((need_resize == cRedraw) ? TRUE : FALSE);
-#	if 0
-			if (need_resize == cRedraw) {
-				need_resize = cNo;
-				goto again;				/* Shouldn't fall through if doing resize */
-			}
-#	endif /* 0 */
+			handle_resize((need_resize == cRedraw) ? TRUE : FALSE);
 			need_resize = cNo;
+#	if defined(KEY_RESIZE) && defined(USE_CURSES)
+			if (ch == KEY_RESIZE)
+				goto again;
+#	endif /* KEY_RESIZE && USE_CURSES */
+
 		}
 		if (ch == KEY_BACKSPACE)
 			ch = '\010';	/* fix for Ctrl-H - show headers */
@@ -413,6 +550,90 @@ again:
 }
 
 
+#	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+wint_t
+ReadWch(
+	void)
+{
+	wint_t wch;
+	int res;
+
+	if (cmd_line)
+		wch = cmdReadWch();
+	else {
+again:
+		allow_resize(TRUE);
+#		ifdef HAVE_NCURSESW
+#			if defined(KEY_RESIZE) && defined(USE_CURSES)
+		if ((res = get_wch(&wch)) == KEY_CODE_YES && wch == KEY_RESIZE)
+			need_resize = cYes;
+		if (res == ERR)
+			goto again;
+#			else
+		res = get_wch(&wch);
+#			endif /* KEY_RESIZE && USE_CURSES */
+#		else
+		wch = (wint_t) getch();
+
+		if (wch == (wint_t) ERR)
+			goto again;
+
+		if (wch < KEY_MIN) {
+			/* read in the multibyte sequence */
+			char *mbs = my_malloc(MB_CUR_MAX + 1);
+			int i, ch;
+
+			mbs[0] = (char) wch;
+			nodelay(stdscr, TRUE);
+			for (i = 1; i < (int) MB_CUR_MAX; i++) {
+				if ((ch = getch()) != ERR)
+					mbs[i] = (char) ch;
+				else
+					break;
+			}
+			nodelay(stdscr, FALSE);
+
+			mbs[i] = '\0';
+			res = mbtowc((wchar_t *) (&wch), mbs, MB_CUR_MAX);
+			free(mbs);
+			if (res == -1)
+				return WEOF; /* error */
+			else
+				res = OK;
+		} else {
+			res = KEY_CODE_YES;
+#			if defined(KEY_RESIZE) && defined(USE_CURSES)
+			if (wch == KEY_RESIZE)
+				need_resize = cYes;
+#			endif /* KEY_RESIZE && USE_CURSES */
+		}
+#		endif /* HAVE_NCURSESW */
+		allow_resize(FALSE);
+		if (need_resize) {
+			handle_resize((need_resize == cRedraw) ? TRUE : FALSE);
+			need_resize = cNo;
+#		if defined(KEY_RESIZE) && defined(USE_CURSES)
+			if (wch == KEY_RESIZE)
+				goto again;
+#		endif /* KEY_RESIZE && USE_CURSES */
+		}
+		if (wch == KEY_BACKSPACE)
+			wch = (wint_t) '\010';	/* fix for Ctrl-H - show headers */
+		else if (wch == ESC || res == KEY_CODE_YES) {
+			/* TODO:
+			 * check out why using unget_wch() here causes problems at
+			 * get_arrow_key()
+			 */
+			/* unget_wch(wch); */
+			ungetch((int) wch);
+			wch = ESC;
+		}
+	}
+	return wch;
+}
+#	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+
+
 void
 my_printf(
 	const char *fmt,
@@ -428,9 +649,9 @@ my_printf(
 		vprintf(fmt, ap);
 		if (flag)
 			Raw(TRUE);
-	} else {
-		vwprintw(stdscr, (char *)fmt, ap);
-	}
+	} else
+		vwprintw(stdscr, fmt, ap);
+
 	va_end(ap);
 }
 
@@ -452,9 +673,9 @@ my_fprintf(
 		vfprintf(stream, fmt, ap);
 		if (flag)
 			Raw(TRUE);
-	} else {
-		vwprintw(stdscr, (char *)fmt, ap);
-	}
+	} else
+		vwprintw(stdscr, fmt, ap);
+
 	va_end(ap);
 }
 
@@ -467,10 +688,10 @@ my_fputc(
 	TRACE(("my_fputc(%s)", tin_tracechar(ch)));
 	if (cmd_line) {
 		if (_inraw && ch == '\n')
-			fputc ('\r', fp);
-		fputc (ch, fp);
+			fputc('\r', fp);
+		fputc(ch, fp);
 	} else {
-		addch ((unsigned char) ch);
+		addch((unsigned char) ch);
 	}
 }
 
@@ -486,13 +707,77 @@ my_fputs(
 			while (*str)
 				my_fputc(*str++, fp);
 		} else
-			fputs (str, fp);
+			fputs(str, fp);
 	} else {
-		addstr((char *)str);
+		addstr(str);
 	}
 }
 
 
+#	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+void
+my_fputwc(
+	wint_t wc,
+	FILE *fp)
+{
+	if (cmd_line) {
+		if (_inraw && wc == (wint_t) '\n')
+			fputwc((wint_t) '\r', fp);
+		fputwc(wc, fp);
+	} else {
+#		ifdef HAVE_NCURSESW
+		cchar_t cc;
+
+		cc.attr = A_NORMAL;
+		cc.chars[0] = (wchar_t) wc;
+		cc.chars[1] = (wchar_t) '\0';
+
+		add_wch(&cc);
+#	else
+		char *mbs;
+		int len;
+
+		mbs = my_malloc(MB_CUR_MAX + 1);
+
+		if ((len = wctomb(mbs, wc)) != -1) {
+			mbs[len] = '\0';
+			addstr(mbs);
+		} else
+			addch('?');
+
+		free(mbs);
+#		endif /* HAVE_NCURSESW */
+	}
+}
+
+
+void
+my_fputws(
+	const wchar_t *wstr,
+	FILE *fp)
+{
+	if (cmd_line) {
+		if (_inraw) {
+			while (*wstr)
+				my_fputwc(*wstr++, fp);
+		} else
+			fputws(wstr, fp);
+	} else {
+#		ifdef HAVE_NCURSESW
+		addwstr(wstr);
+#		else
+		char *mbs;
+
+		if ((mbs = wchar_t2char(wstr)) != NULL) {
+			addstr(mbs);
+			free(mbs);
+		}
+#		endif /* HAVE_NCURSESW */
+	}
+}
+#	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+
+
 void
 my_erase(
 	void)
@@ -502,10 +787,11 @@ my_erase(
 	if (!cmd_line) {
 		erase();
 
-		/* Curses doesn't actually do an erase() until refresh() is
-		 * called.  Ncurses 4.0 (and lower) reset the background color
-		 * when doing an erase().  So the only way to ensure we'll get
-		 * the same background colors is to reset them here.
+		/*
+		 * Curses doesn't actually do an erase() until refresh() is called.
+		 * Ncurses 4.0 (and lower) reset the background color when doing an
+		 * erase(). So the only way to ensure we'll get the same background
+		 * colors is to reset them here.
 		 */
 		refresh();
 #	ifdef HAVE_COLOR
@@ -542,6 +828,36 @@ my_retouch(
 }
 
 
+/*
+ * innstr can't read multibyte chars
+ * we use innwstr (if avaible) and convert to multibyte chars
+ */
+static int
+my_innstr(
+	char *str,
+	int n)
+{
+#	if defined(HAVE_NCURSESW) && defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	size_t len = 0;
+	wchar_t *buffer;
+
+	buffer = my_malloc(sizeof(wchar_t) * (n + 1));
+
+	if (innwstr(buffer, n) != ERR) {
+		if ((len = wcstombs(str, buffer, 2 * n)) == (size_t) (-1))
+			len = 0;
+		str[len] = '\0';
+	}
+
+	free(buffer);
+	return len;
+#	else
+	int len = innstr(str, n);
+	return (len == ERR ? 0 : len);
+#	endif /* HAVE_NCURSESW && MULTIBYTE_ABLE && !NO_LOCALE */
+}
+
+
 char *
 screen_contents(
 	int row,
@@ -553,8 +869,7 @@ screen_contents(
 	getyx(stdscr, y, x);
 	move(row, col);
 	TRACE(("screen_contents(%d,%d)", row, col));
-	if (innstr(buffer, len) == ERR)
-		len = 0;
+	len = my_innstr(buffer, len);
 	buffer[len] = '\0';
 	TRACE(("...screen_contents(%d,%d) %s", y, x, _nc_visbuf(buffer)));
 	return buffer;
@@ -566,18 +881,12 @@ write_line(
 	int row,
 	char *buffer)
 {
-	int len = strlen(buffer);
-
-	if (len > cCOLS)
-		buffer[len = cCOLS] = '\0';
-	mvaddstr(row, 0, buffer);
-	if (len < cCOLS)
-		clrtoeol();
+	mvaddnstr(row, 0, buffer, -1);
 }
 
 
 int
-get_arrow_key (
+get_arrow_key(
 	int prech) /* unused */
 {
 #	ifdef NCURSES_MOUSE_VERSION
@@ -590,33 +899,43 @@ get_arrow_key (
 		case KEY_DC:
 			code = KEYMAP_DEL;
 			break;
+
 		case KEY_IC:
 			code = KEYMAP_INS;
 			break;
+
 		case KEY_UP:
 			code = KEYMAP_UP;
 			break;
+
 		case KEY_DOWN:
 			code = KEYMAP_DOWN;
 			break;
+
 		case KEY_LEFT:
 			code = KEYMAP_LEFT;
 			break;
+
 		case KEY_RIGHT:
 			code = KEYMAP_RIGHT;
 			break;
+
 		case KEY_NPAGE:
 			code = KEYMAP_PAGE_DOWN;
 			break;
+
 		case KEY_PPAGE:
 			code = KEYMAP_PAGE_UP;
 			break;
+
 		case KEY_HOME:
 			code = KEYMAP_HOME;
 			break;
+
 		case KEY_END:
 			code = KEYMAP_END;
 			break;
+
 #	ifdef NCURSES_MOUSE_VERSION
 		case KEY_MOUSE:
 			if (getmouse(&my_event) != ERR) {
@@ -624,9 +943,11 @@ get_arrow_key (
 					case BUTTON1_CLICKED:
 						xmouse = MOUSE_BUTTON_1;
 						break;
+
 					case BUTTON2_CLICKED:
 						xmouse = MOUSE_BUTTON_2;
 						break;
+
 					case BUTTON3_CLICKED:
 						xmouse = MOUSE_BUTTON_3;
 						break;
